@@ -142,7 +142,7 @@ pkg_from_package_id (const gchar *package_id)
 		alpm_list_t *iterator;
 		for (iterator = alpm_option_get_syncdbs (); iterator; iterator = alpm_list_next (iterator)) {
 			repo = alpm_list_getdata (iterator);
-			if (g_strcmp0 (alpm_db_get_name(repo), package_id_data[PK_PACKAGE_ID_DATA]) == 0)
+			if (g_strcmp0 (alpm_db_get_name (repo), package_id_data[PK_PACKAGE_ID_DATA]) == 0)
 				break;
 		}
 	}
@@ -296,7 +296,7 @@ cb_dl_progress (const char *filename, off_t file_xfered, off_t file_total)
 	file_percent = (int) ((float) file_xfered) / ((float) file_total) * 100;
 	trans_percent = (int) ((float) (trans_xfered + file_xfered)) / ((float) trans_total) * 100;
 	pk_backend_set_sub_percentage ((PkBackend *) backend_instance, file_percent);
-	pk_backend_set_percentage((PkBackend *) backend_instance, trans_percent);
+	pk_backend_set_percentage ((PkBackend *) backend_instance, trans_percent);
 
 	if (file_xfered == file_total) {
 		downloaded_files = alpm_list_add (downloaded_files, g_strdup (filename));
@@ -351,14 +351,14 @@ strtrim (char *str)
  * Helper function for strreplace
  */
 static void
-_strnadd(char **str, const char *append, unsigned int count)
+_strnadd (char **str, const char *append, unsigned int count)
 {
 	if (*str)
 		*str = realloc (*str, strlen (*str) + count + 1);
 	else
 		*str = calloc (sizeof (char), count + 1);
 
-	strncat(*str, append, count);
+	strncat (*str, append, count);
 }
 
 /**
@@ -367,7 +367,7 @@ _strnadd(char **str, const char *append, unsigned int count)
  * a new string (must be free'd)
  */
 static char *
-strreplace(const char *str, const char *needle, const char *replace)
+strreplace (const char *str, const char *needle, const char *replace)
 {
 	const char *p, *q;
 	char *newstr = NULL;
@@ -376,18 +376,18 @@ strreplace(const char *str, const char *needle, const char *replace)
 	p = q = str;
 
 	while (1) {
-		q = strstr(p, needle);
+		q = strstr (p, needle);
 		if (!q) {
 			/* not found */
 			if (*p) /* add the rest of 'p' */
-				_strnadd(&newstr, p, strlen(p));
+				_strnadd (&newstr, p, strlen (p));
 
 			break;
 		} else { /* found match */
 			if (q > p) /* add chars between this occurance and last occurance, if any */
-				_strnadd(&newstr, p, q - p);
+				_strnadd (&newstr, p, q - p);
 
-			_strnadd(&newstr, replace, replacesz);
+			_strnadd (&newstr, replace, replacesz);
 			p = q + needlesz;
 		}
 	}
@@ -403,7 +403,7 @@ strreplace(const char *str, const char *needle, const char *replace)
  * @param optionfunc a function pointer to an alpm_option_add_* function
  */
 static void
-set_repeating_option(const char *ptr, const char *option, void (*optionfunc) (const char*))
+set_repeating_option (const char *ptr, const char *option, void (*optionfunc) (const char*))
 {
 	char *p = (char*) ptr;
 	char *q;
@@ -462,10 +462,10 @@ parse_config (const char *file, const char *givensection, pmdb_t * const givendb
 	alpm_option_add_cachedir (ALPM_CACHEDIR);
 	alpm_option_set_logfile (ALPM_LOGFILE);
 
-	fp = fopen(file, "r");
+	fp = fopen (file, "r");
 	if (fp == NULL) {
 		egg_error ("config file %s could not be read", file);
-		return (1);
+		return 1;
 	}
 
 	/* if we are passed a section, use it as our starting point */
@@ -525,24 +525,20 @@ parse_config (const char *file, const char *givensection, pmdb_t * const givendb
 
 			if (ptr == NULL && g_strcmp0 (section, "options") == 0) {
 				/* directives without settings, all in [options] */
-				if (g_strcmp0 (key, "NoPassiveFTP") == 0) {
-					alpm_option_set_nopassiveftp (1);
-					egg_debug ("config: nopassiveftp");
-				} else if (g_strcmp0 (key, "UseSyslog") == 0) {
+				if (g_strcmp0 (key, "UseSyslog") == 0) {
 					alpm_option_set_usesyslog (1);
 					egg_debug ("config: usesyslog");
 				} else if (g_strcmp0 (key, "UseDelta") == 0) {
 					alpm_option_set_usedelta (1);
 					egg_debug ("config: usedelta");
-				} else {
-					egg_error ("config file %s, line %d: directive '%s' not recognized.", file, linenum, key);
-					return 1;
+				} else if (g_strcmp0 (key, "ILoveCandy") != 0 && g_strcmp0 (key, "ShowSize") != 0 && g_strcmp0 (key, "TotalDownload") != 0 && g_strcmp0 (key, "NoPassiveFTP") != 0) {
+					egg_warning ("config file %s, line %d: directive '%s' not recognized.", file, linenum, key);
 				}
 			} else {
 				/* directives with settings */
 				if (g_strcmp0 (key, "Include") == 0) {
 					egg_debug ("config: including %s", ptr);
-					parse_config(ptr, section, db);
+					parse_config (ptr, section, db);
 					/* Ignore include failures... assume non-critical */
 				} else if (g_strcmp0 (section, "options") == 0) {
 					if (g_strcmp0 (key, "NoUpgrade") == 0) {
@@ -560,7 +556,7 @@ parse_config (const char *file, const char *givensection, pmdb_t * const givendb
 					} else if (g_strcmp0 (key, "DBPath") == 0) {
 						alpm_option_set_dbpath (ptr);
 					} else if (g_strcmp0 (key, "CacheDir") == 0) {
-						if (alpm_option_add_cachedir(ptr) != 0) {
+						if (alpm_option_add_cachedir (ptr) != 0) {
 							egg_error ("problem adding cachedir '%s' (%s)", ptr, alpm_strerrorlast ());
 							return 1;
 						}
@@ -571,9 +567,8 @@ parse_config (const char *file, const char *givensection, pmdb_t * const givendb
 					} else if (g_strcmp0 (key, "LogFile") == 0) {
 						alpm_option_set_logfile (ptr);
 						egg_debug ("config: logfile: %s", ptr);
-					} else {
-						egg_error ("config file %s, line %d: directive '%s' not recognized.", file, linenum, key);
-						return 1;
+					} else if (g_strcmp0 (key, "XferCommand") != 0 && g_strcmp0 (key, "CleanMethod") != 0) {
+						egg_warning ("config file %s, line %d: directive '%s' not recognized.", file, linenum, key);
 					}
 				} else if (g_strcmp0 (key, "Server") == 0) {
 					/* let's attempt a replacement for the current repo */
@@ -581,12 +576,13 @@ parse_config (const char *file, const char *givensection, pmdb_t * const givendb
 
 					if (alpm_db_setserver (db, server) != 0) {
 						/* pm_errno is set by alpm_db_setserver */
+						egg_error ("config file %s, line %d: could not add server URL to database (%s).", file, linenum, alpm_strerrorlast ());
+						free (server);
 						return 1;
 					}
 					free (server);
 				} else {
-					egg_error ("config file %s, line %d: directive '%s' not recognized.", file, linenum, key);
-					return 1;
+					egg_warning ("config file %s, line %d: directive '%s' not recognized.", file, linenum, key);
 				}
 			}
 		}
@@ -753,7 +749,7 @@ backend_download_packages_thread (PkBackend *backend)
 	alpm_option_add_cachedir (directory);
 
 	/* create a new transaction */
-	if (alpm_trans_init (PM_TRANS_TYPE_SYNC, PM_TRANS_FLAG_NODEPS | PM_TRANS_FLAG_DOWNLOADONLY, cb_trans_evt, cb_trans_conv, cb_trans_progress) != 0) {
+	if (alpm_trans_init (PM_TRANS_FLAG_NODEPS | PM_TRANS_FLAG_DOWNLOADONLY, cb_trans_evt, cb_trans_conv, cb_trans_progress) != 0) {
 		pk_backend_error_code (backend, PK_ERROR_ENUM_TRANSACTION_ERROR, alpm_strerrorlast ());
 		pk_backend_finished (backend);
 		return FALSE;
@@ -763,7 +759,7 @@ backend_download_packages_thread (PkBackend *backend)
 	for (iterator = 0; iterator < g_strv_length (package_ids); ++iterator) {
 		gchar **package_id_data = pk_package_id_split (package_ids[iterator]);
 
-		if (alpm_trans_addtarget (package_id_data[PK_PACKAGE_ID_NAME]) != 0) {
+		if (alpm_sync_target (package_id_data[PK_PACKAGE_ID_NAME]) != 0) {
 			pk_backend_error_code (backend, PK_ERROR_ENUM_TRANSACTION_ERROR, alpm_strerrorlast ());
 			alpm_trans_release ();
 			pk_backend_finished (backend);
@@ -929,7 +925,7 @@ backend_get_details (PkBackend *backend, gchar **package_ids)
 		licenses = g_string_free (licenses_str, FALSE);
 
 		/* return details */
-		pk_backend_details (backend, package_ids[iterator], licenses, PK_GROUP_ENUM_OTHER, alpm_pkg_get_desc (pkg), alpm_pkg_get_url(pkg), alpm_pkg_get_size (pkg));
+		pk_backend_details (backend, package_ids[iterator], licenses, PK_GROUP_ENUM_OTHER, alpm_pkg_get_desc (pkg), alpm_pkg_get_url (pkg), alpm_pkg_get_size (pkg));
 
 		/* free licenses array as we no longer need it */
 		g_free (licenses);
@@ -1018,7 +1014,7 @@ backend_search (PkBackend *backend, pmdb_t *repo, const gchar *needle, PkAlpmSea
 				match = TRUE;
 				break;
 			case PK_ALPM_SEARCH_TYPE_RESOLVE:
-				match = g_strcmp0 (alpm_pkg_get_name(pkg), needle) == 0;
+				match = g_strcmp0 (alpm_pkg_get_name (pkg), needle) == 0;
 				break;
 			case PK_ALPM_SEARCH_TYPE_NAME:
 				match = strstr (alpm_pkg_get_name (pkg), needle) != NULL;
@@ -1045,7 +1041,7 @@ backend_search (PkBackend *backend, pmdb_t *repo, const gchar *needle, PkAlpmSea
 				match = FALSE;
 				/* iterate provides */
 				for (provides = alpm_pkg_get_provides (pkg); provides && !match; provides = alpm_list_next (provides))
-					match = (g_strcmp0 (needle, alpm_list_getdata(provides)) == 0);
+					match = (g_strcmp0 (needle, alpm_list_getdata (provides)) == 0);
 				break;
 			default:
 				match = FALSE;
@@ -1058,6 +1054,17 @@ backend_search (PkBackend *backend, pmdb_t *repo, const gchar *needle, PkAlpmSea
 	}
 }
 
+/**
+ * backend_search_values:
+ */
+static void
+backend_search_values (PkBackend *backend, pmdb_t *repo, gchar **values, PkAlpmSearchType search_type) {
+	unsigned int iterator;
+
+	for (iterator = 0; iterator < g_strv_length (values); ++iterator)
+		backend_search (backend, repo, values[iterator], search_type);
+}
+
 /**
  * backend_get_packages_thread:
  */
@@ -1193,7 +1200,7 @@ backend_install_files_thread (PkBackend *backend)
 	gchar **full_paths = pk_backend_get_strv (backend, "full_paths");
 
 	/* create a new transaction */
-	if (alpm_trans_init (PM_TRANS_TYPE_UPGRADE, 0, cb_trans_evt, cb_trans_conv, cb_trans_progress) != 0) {
+	if (alpm_trans_init (0, cb_trans_evt, cb_trans_conv, cb_trans_progress) != 0) {
 		pk_backend_error_code (backend, PK_ERROR_ENUM_TRANSACTION_ERROR, alpm_strerrorlast ());
 		pk_backend_finished (backend);
 		return FALSE;
@@ -1201,7 +1208,7 @@ backend_install_files_thread (PkBackend *backend)
 
 	/* add targets to the transaction */
 	for (iterator = 0; iterator < g_strv_length (full_paths); ++iterator) {
-		if (alpm_trans_addtarget (full_paths[iterator]) != 0) {
+		if (alpm_add_target (full_paths[iterator]) != 0) {
 			pk_backend_error_code (backend, PK_ERROR_ENUM_TRANSACTION_ERROR, alpm_strerrorlast ());
 			alpm_trans_release ();
 			pk_backend_finished (backend);
@@ -1249,7 +1256,7 @@ backend_install_packages_thread (PkBackend *backend)
 	gchar **package_ids = pk_backend_get_strv (backend, "package_ids");
 
 	/* create a new transaction */
-	if (alpm_trans_init (PM_TRANS_TYPE_SYNC, 0, cb_trans_evt, cb_trans_conv, cb_trans_progress) != 0) {
+	if (alpm_trans_init (0, cb_trans_evt, cb_trans_conv, cb_trans_progress) != 0) {
 		pk_backend_error_code (backend, PK_ERROR_ENUM_TRANSACTION_ERROR, alpm_strerrorlast ());
 		pk_backend_finished (backend);
 		return FALSE;
@@ -1259,7 +1266,7 @@ backend_install_packages_thread (PkBackend *backend)
 	for (iterator = 0; iterator < g_strv_length (package_ids); ++iterator) {
 		gchar **package_id_data = pk_package_id_split (package_ids[iterator]);
 
-		if (alpm_trans_addtarget (package_id_data[PK_PACKAGE_ID_NAME]) != 0) {
+		if (alpm_sync_target (package_id_data[PK_PACKAGE_ID_NAME]) != 0) {
 			pk_backend_error_code (backend, PK_ERROR_ENUM_TRANSACTION_ERROR, alpm_strerrorlast ());
 			alpm_trans_release ();
 			pk_backend_finished (backend);
@@ -1302,7 +1309,7 @@ backend_refresh_cache_thread (PkBackend *backend)
 {
 	alpm_list_t *list_iterator;
 
-	if (alpm_trans_init (PM_TRANS_TYPE_SYNC, PM_TRANS_FLAG_NOSCRIPTLET, cb_trans_evt, cb_trans_conv, cb_trans_progress) != 0) {
+	if (alpm_trans_init (PM_TRANS_FLAG_NOSCRIPTLET, cb_trans_evt, cb_trans_conv, cb_trans_progress) != 0) {
 		pk_backend_error_code (backend, PK_ERROR_ENUM_TRANSACTION_ERROR, alpm_strerrorlast ());
 		pk_backend_finished (backend);
 		return FALSE;
@@ -1348,17 +1355,22 @@ static gboolean
 backend_remove_packages_thread (PkBackend *backend)
 {
 	unsigned int iterator;
+	alpm_list_t *list_iterator = NULL;
 	alpm_list_t *data = NULL;
+	gchar *holdpkgs = NULL;
 
 	gchar **package_ids = pk_backend_get_strv (backend, "package_ids");
 	gboolean allow_deps = pk_backend_get_bool (backend, "allow_deps");
+	gboolean autoremove = pk_backend_get_bool (backend, "autoremove");
 
 	pmtransflag_t flags = 0;
 	if (allow_deps)
 		flags |= PM_TRANS_FLAG_CASCADE;
+	if (autoremove)
+		flags |= PM_TRANS_FLAG_RECURSE;
 
 	/* create a new transaction */
-	if (alpm_trans_init (PM_TRANS_TYPE_REMOVE, flags, cb_trans_evt, cb_trans_conv, cb_trans_progress) != 0) {
+	if (alpm_trans_init (flags, cb_trans_evt, cb_trans_conv, cb_trans_progress) != 0) {
 		pk_backend_error_code (backend, PK_ERROR_ENUM_TRANSACTION_ERROR, alpm_strerrorlast ());
 		pk_backend_finished (backend);
 		return FALSE;
@@ -1368,7 +1380,7 @@ backend_remove_packages_thread (PkBackend *backend)
 	for (iterator = 0; iterator < g_strv_length (package_ids); ++iterator) {
 		gchar **package_id_data = pk_package_id_split (package_ids[iterator]);
 
-		if (alpm_trans_addtarget (package_id_data[PK_PACKAGE_ID_NAME]) != 0) {
+		if (alpm_remove_target (package_id_data[PK_PACKAGE_ID_NAME]) != 0) {
 			pk_backend_error_code (backend, PK_ERROR_ENUM_TRANSACTION_ERROR, alpm_strerrorlast ());
 			alpm_trans_release ();
 			pk_backend_finished (backend);
@@ -1378,8 +1390,41 @@ backend_remove_packages_thread (PkBackend *backend)
 		g_strfreev (package_id_data);
 	}
 
-	/* prepare and commit transaction */
-	if (alpm_trans_prepare (&data) != 0 || alpm_trans_commit (&data) != 0) {
+	/* prepare transaction */
+	if (alpm_trans_prepare (&data) != 0) {
+		pk_backend_error_code (backend, PK_ERROR_ENUM_TRANSACTION_ERROR, alpm_strerrorlast ());
+		alpm_trans_release ();
+		pk_backend_finished (backend);
+		return FALSE;
+	}
+
+	/* search for HoldPkg's in target list */
+	for (list_iterator = alpm_trans_get_remove (); list_iterator; list_iterator = alpm_list_next (list_iterator)) {
+		pmpkg_t *pkg = alpm_list_getdata (list_iterator);
+		const gchar *pkgname = alpm_pkg_get_name (pkg);
+
+		if (alpm_list_find_str (holdpkg, pkgname) != NULL) {
+			if (holdpkgs == NULL)
+				holdpkgs = g_strdup (pkgname);
+			else {
+				gchar *new_holdpkgs = g_strdup_printf ("%s, %s", holdpkgs, pkgname);
+				g_free (holdpkgs);
+				holdpkgs = new_holdpkgs;
+			}
+		}
+	}
+
+	/* pacman just asks for confirmation, but here we fail to be safe */
+	if (holdpkgs != NULL) {
+		pk_backend_error_code (backend, PK_ERROR_ENUM_CANNOT_REMOVE_SYSTEM_PACKAGE, "The following packages are designated HoldPkg: %s", holdpkgs);
+		free (holdpkgs);
+		alpm_trans_release ();
+		pk_backend_finished (backend);
+		return FALSE;
+	}
+
+	/* commit transaction */
+	if (alpm_trans_commit (&data) != 0) {
 		pk_backend_error_code (backend, PK_ERROR_ENUM_TRANSACTION_ERROR, alpm_strerrorlast ());
 		alpm_trans_release ();
 		pk_backend_finished (backend);
@@ -1436,6 +1481,7 @@ backend_resolve_thread (PkBackend *backend)
 			}
 		} else {
 			/* good old way with backend_search */
+			/* TODO: check, is it still needed? */
 			if (!search_not_installed) {
 				/* search in local db */
 				backend_search (backend, alpm_option_get_localdb (), package_ids[iterator], PK_ALPM_SEARCH_TYPE_RESOLVE);
@@ -1473,7 +1519,7 @@ backend_resolve (PkBackend *backend, PkBitfield filters, gchar **package_ids)
 static gboolean
 backend_search_thread (PkBackend *backend)
 {
-	const gchar *search = pk_backend_get_string (backend, "search");
+	gchar **values = pk_backend_get_strv (backend, "values");
 	PkBitfield filters = pk_backend_get_uint (backend, "filters");
 	PkAlpmSearchType search_type = pk_backend_get_uint (backend, "search-type");
 
@@ -1482,7 +1528,7 @@ backend_search_thread (PkBackend *backend)
 
 	if (!search_not_installed) {
 		/* search in local db */
-		backend_search (backend, alpm_option_get_localdb (), search, search_type);
+		backend_search_values (backend, alpm_option_get_localdb (), values, search_type);
 	}
 
 	if (!search_installed) {
@@ -1490,7 +1536,7 @@ backend_search_thread (PkBackend *backend)
 		alpm_list_t *repos;
 		/* iterate repos */
 		for (repos = alpm_option_get_syncdbs (); repos; repos = alpm_list_next (repos))
-			backend_search (backend, alpm_list_getdata (repos), search, search_type);
+			backend_search_values (backend, alpm_list_getdata (repos), values, search_type);
 	}
 
 	pk_backend_finished (backend);
@@ -1501,37 +1547,40 @@ backend_search_thread (PkBackend *backend)
  * backend_search_details:
  */
 static void
-backend_search_details (PkBackend *backend, PkBitfield filters, const gchar *search)
+backend_search_details (PkBackend *backend, PkBitfield filters, gchar **values)
 {
 	pk_backend_set_status (backend, PK_STATUS_ENUM_QUERY);
 	pk_backend_set_percentage (backend, PK_BACKEND_PERCENTAGE_INVALID);
 	pk_backend_set_uint (backend, "search-type", PK_ALPM_SEARCH_TYPE_DETAILS);
+	pk_backend_set_strv (backend, "values", values);
 
 	pk_backend_thread_create (backend, backend_search_thread);
 }
 
 /**
- * backend_search_group:
+ * backend_search_groups:
  */
 static void
-backend_search_group (PkBackend *backend, PkBitfield filters, const gchar *search)
+backend_search_groups (PkBackend *backend, PkBitfield filters, gchar **values)
 {
 	pk_backend_set_status (backend, PK_STATUS_ENUM_QUERY);
 	pk_backend_set_percentage (backend, PK_BACKEND_PERCENTAGE_INVALID);
 	pk_backend_set_uint (backend, "search-type", PK_ALPM_SEARCH_TYPE_GROUP);
+	pk_backend_set_strv (backend, "values", values);
 
 	pk_backend_thread_create (backend, backend_search_thread);
 }
 
 /**
- * backend_search_name:
+ * backend_search_names:
  */
 static void
-backend_search_name (PkBackend *backend, PkBitfield filters, const gchar *search)
+backend_search_names (PkBackend *backend, PkBitfield filters, gchar **values)
 {
 	pk_backend_set_status (backend, PK_STATUS_ENUM_QUERY);
 	pk_backend_set_percentage (backend, PK_BACKEND_PERCENTAGE_INVALID);
 	pk_backend_set_uint (backend, "search-type", PK_ALPM_SEARCH_TYPE_NAME);
+	pk_backend_set_strv (backend, "values", values);
 
 	pk_backend_thread_create (backend, backend_search_thread);
 }
@@ -1557,18 +1606,15 @@ backend_update_system_thread (PkBackend *backend)
 
 	/* FIXME: support only_trusted */
 
-	/* don't specify any flags for now */
-	pmtransflag_t flags = 0;
-
 	/* create a new transaction */
-	if (alpm_trans_init (PM_TRANS_TYPE_SYNC, flags, cb_trans_evt, cb_trans_conv, cb_trans_progress) != 0) {
+	if (alpm_trans_init (0, cb_trans_evt, cb_trans_conv, cb_trans_progress) != 0) {
 		pk_backend_error_code (backend, PK_ERROR_ENUM_TRANSACTION_ERROR, alpm_strerrorlast ());
 		pk_backend_finished (backend);
 		return FALSE;
 	}
 
 	/* set action, prepare and commit transaction */
-	if (alpm_trans_sysupgrade (FALSE) != 0 || alpm_trans_prepare (&data) != 0 || alpm_trans_commit (&data) != 0) {
+	if (alpm_sync_sysupgrade (FALSE) != 0 || alpm_trans_prepare (&data) != 0 || alpm_trans_commit (&data) != 0) {
 		pk_backend_error_code (backend, PK_ERROR_ENUM_TRANSACTION_ERROR, alpm_strerrorlast ());
 		alpm_trans_release ();
 		pk_backend_finished (backend);
@@ -1594,11 +1640,12 @@ backend_update_system (PkBackend *backend, gboolean only_trusted)
  * backend_what_provides:
  */
 static void
-backend_what_provides (PkBackend *backend, PkBitfield filters, PkProvidesEnum provides, const gchar *search)
+backend_what_provides (PkBackend *backend, PkBitfield filters, PkProvidesEnum provides, gchar **values)
 {
 	pk_backend_set_status (backend, PK_STATUS_ENUM_QUERY);
 	pk_backend_set_percentage (backend, PK_BACKEND_PERCENTAGE_INVALID);
 	pk_backend_set_uint (backend, "search-type", PK_ALPM_SEARCH_TYPE_PROVIDES);
+	pk_backend_set_strv (backend, "values", values);
 
 	pk_backend_thread_create (backend, backend_search_thread);
 }
@@ -1634,9 +1681,9 @@ PK_BACKEND_OPTIONS (
 	backend_resolve,				/* resolve */
 	NULL,						/* rollback */
 	backend_search_details,				/* search_details */
-	NULL,						/* search_file */
-	backend_search_group,				/* search_group */
-	backend_search_name,				/* search_name */
+	NULL,						/* search_files */
+	backend_search_groups,				/* search_groups */
+	backend_search_names,				/* search_names */
 	backend_update_packages,			/* update_packages */
 	backend_update_system,				/* update_system */
 	backend_what_provides,				/* what_provides */
