@@ -3,7 +3,7 @@
 // $Id: acqprogress.h,v 1.5 2003/02/02 22:24:11 jgg Exp $
 /* ######################################################################
 
-   Acquire Progress - Command line progress meter 
+   Acquire Progress - Command line progress meter
 
    ##################################################################### */
 									/*}}}*/
@@ -17,23 +17,10 @@
 
 class AcqPackageKitStatus : public pkgAcquireStatus
 {
-	PkBackend *m_backend;
-	char BlankLine[1024];
-	unsigned long ID;
-	unsigned long Quiet;
-	bool &_cancelled;
-
-	unsigned long last_percent;
-	unsigned long last_sub_percent;
-	string last_package_name;
-	aptcc *m_apt;
-
-	vector<pair<pkgCache::PkgIterator, pkgCache::VerIterator> > packages;
-
-	void emit_package(const string &name);
-
 public:
-	virtual bool MediaChange(string Media,string Drive);
+	AcqPackageKitStatus(aptcc *apt, PkBackend *backend, bool &cancelled);
+
+	virtual bool MediaChange(string Media, string Drive);
 	virtual void IMSHit(pkgAcquire::ItemDesc &Itm);
 	virtual void Fetch(pkgAcquire::ItemDesc &Itm);
 	virtual void Done(pkgAcquire::ItemDesc &Itm);
@@ -45,7 +32,21 @@ public:
 
 	void addPackagePair(pair<pkgCache::PkgIterator, pkgCache::VerIterator> packagePair);
 
-	AcqPackageKitStatus(aptcc *apt, PkBackend *backend, bool &cancelled);
+private:
+	PkBackend *m_backend;
+	unsigned long ID;
+	bool &_cancelled;
+
+	unsigned long last_percent;
+	unsigned long last_sub_percent;
+	double        last_CPS;
+	string        last_package_name;
+	aptcc         *m_apt;
+
+	vector<pair<pkgCache::PkgIterator, pkgCache::VerIterator> > packages;
+	set<string> currentPackages;
+
+	void emit_package(const string &name, bool finished);
 };
 
 #endif
