@@ -57,7 +57,8 @@ aptcc::aptcc(PkBackend *backend, bool &cancel)
 	Policy(0),
 	m_backend(backend),
 	_cancel(cancel),
-	m_terminalTimeout(120)
+	m_terminalTimeout(120),
+	m_lastSubProgress(0)
 {
 	_cancel = false;
 }
@@ -380,8 +381,7 @@ void aptcc::emit_packages(vector<pair<pkgCache::PkgIterator, pkgCache::VerIterat
 void aptcc::emitUpdates(vector<pair<pkgCache::PkgIterator, pkgCache::VerIterator> > &output,
 			 PkBitfield filters)
 {
-	// the default update info
-	PkInfoEnum state = PK_INFO_ENUM_NORMAL;
+	PkInfoEnum state;
 	// Sort so we can remove the duplicated entries
 	sort(output.begin(), output.end(), compare());
 	// Remove the duplicated entries
@@ -397,6 +397,9 @@ void aptcc::emitUpdates(vector<pair<pkgCache::PkgIterator, pkgCache::VerIterator
 			break;
 		}
 
+		// the default update info
+		state = PK_INFO_ENUM_NORMAL;
+
 		// let find what kind of upgrade this is
 		pkgCache::VerFileIterator vf = i->second.FileList();
 		std::string origin  = vf.File().Origin();
@@ -412,7 +415,8 @@ void aptcc::emitUpdates(vector<pair<pkgCache::PkgIterator, pkgCache::VerIterator
 			} else if (ends_with(archive, "-updates")) {
 				state = PK_INFO_ENUM_BUGFIX;
 			}
-		} else if (origin.compare("Backports.org archive") == 0) {
+		} else if (origin.compare("Backports.org archive") == 0 ||
+				   ends_with(origin, "-backports")) {
 			state = PK_INFO_ENUM_ENHANCEMENT;
 		}
 
@@ -491,7 +495,7 @@ void aptcc::povidesCodec(vector<pair<pkgCache::PkgIterator, pkgCache::VerIterato
 			}
 			g_free(itemreg);
 		} else {
-			egg_debug("Did not match: %", value);
+			egg_debug("Did not match: %s", value);
 		}
 	}
 	regfree(&pkre);
@@ -692,15 +696,23 @@ void aptcc::get_requires(vector<pair<pkgCache::PkgIterator, pkgCache::VerIterato
 }
 
 // used to return files it reads, using the info from the files in /var/lib/dpkg/info/
-vector<string> search_file (PkBackend *backend, const string &file_name, bool &_cancel)
+vector<string> search_files (PkBackend *backend, gchar **values, bool &_cancel)
 {
 	vector<string> packageList;
 	regex_t re;
+	gchar *search;
+	gchar *values_str;
 
-	if(regcomp(&re, file_name.c_str(), REG_NOSUB) != 0) {
+	values_str = g_strjoinv("$|^", values);
+	search = g_strdup_printf("^%s$",
+				 values_str);
+	g_free(values_str);
+	if(regcomp(&re, search, REG_NOSUB) != 0) {
 		egg_debug("Regex compilation error");
+		g_free(search);
 		return vector<string>();
 	}
+	g_free(search);
 
 	DIR *dp;
 	struct dirent *dirp;
@@ -808,19 +820,19 @@ vector<string> searchMimeType (PkBackend *backend, gchar **values, bool &error,
 // 	regex_t re;
 // 	gchar *value;
 // 	gchar *values_str;
-// 
+//
 // 	values_str = g_strjoinv("|", values);
 // 	value = g_strdup_printf("^X-AppInstall-Codecs=\\(.*;\\)\\?\\(%s\\)\\(;.*\\)\\?$",
 // 				values_str);
 // 	g_free(values_str);
-// 
+//
 // 	if(regcomp(&re, value, REG_NOSUB) != 0) {
 // 		egg_debug("Regex compilation error");
 // 		g_free(value);
 // 		return vector<string>();
 // 	}
 // 	g_free(value);
-// 
+//
 // 	DIR *dp;
 // 	struct dirent *dirp;
 // 	if (!(dp = opendir("/usr/share/app-install/desktop/"))) {
@@ -829,7 +841,7 @@ vector<string> searchMimeType (PkBackend *backend, gchar **values, bool &error,
 // 		error = true;
 // 		return vector<string>();
 // 	}
-// 
+//
 // 	string line;
 // 	while ((dirp = readdir(dp)) != NULL) {
 // 		if (_cancel) {
@@ -859,7 +871,7 @@ vector<string> searchMimeType (PkBackend *backend, gchar **values, bool &error,
 // 			}
 // 		}
 // 	}
-// 
+//
 // 	closedir(dp);
 // 	regfree(&re);
 // 	return packageList;
@@ -1071,7 +1083,7 @@ bool aptcc::removingEssentialPackages(pkgCacheFile &Cache)
 						      List.c_str()));
 		return true;
 	}
-	
+
 	return false;
 }
 
@@ -1188,11 +1200,11 @@ void aptcc::updateInterface(int fd, int writeFd)
 			}
 			//cout << "got line: " << line << endl;
 
-			gchar **split = g_strsplit(line, ":",5);
-			gchar *status = g_strstrip(split[0]);
-			gchar *pkg = g_strstrip(split[1]);
+			gchar **split  = g_strsplit(line, ":",5);
+			gchar *status  = g_strstrip(split[0]);
+			gchar *pkg     = g_strstrip(split[1]);
 			gchar *percent = g_strstrip(split[2]);
-			gchar *str = g_strdup(g_strstrip(split[3]));
+			gchar *str     = g_strdup(g_strstrip(split[3]));
 
 			// major problem here, we got unexpected input. should _never_ happen
 			if(!(pkg && status)) {
@@ -1212,24 +1224,24 @@ void aptcc::updateInterface(int fd, int writeFd)
 				string orig_file, new_file;
 
 				// go to first ' and read until the end
-				for(;str[i] != '\'' || str[i] == 0; i++) 
+				for(;str[i] != '\'' || str[i] == 0; i++)
 					/*nothing*/
 					;
 				i++;
-				for(;str[i] != '\'' || str[i] == 0; i++) 
+				for(;str[i] != '\'' || str[i] == 0; i++)
 					orig_file.append(1, str[i]);
 				i++;
 
 				// same for second ' and read until the end
-				for(;str[i] != '\'' || str[i] == 0; i++) 
+				for(;str[i] != '\'' || str[i] == 0; i++)
 					/*nothing*/
 				;
 				i++;
-				for(;str[i] != '\'' || str[i] == 0; i++) 
+				for(;str[i] != '\'' || str[i] == 0; i++)
 					new_file.append(1, str[i]);
 				i++;
 
-				gchar *confmsg; 
+				gchar *confmsg;
 				confmsg = g_strdup_printf("The configuration file '%s' "
 							  "(modified by you or a script) "
 							  "has a newer version '%s'.\n"
@@ -1239,38 +1251,113 @@ void aptcc::updateInterface(int fd, int writeFd)
 				pk_backend_message(m_backend,
 						   PK_MESSAGE_ENUM_CONFIG_FILES_CHANGED,
 						   confmsg);
-				write(writeFd, "N\n", 2);
+				if (write(writeFd, "N\n", 2) != 2) {
+					// TODO we need a DPKG patch to use debconf
+					egg_debug("Failed to write");
+				}
 			} else if (strstr(status, "pmstatus") != NULL) {
+				// INSTALL & UPDATE
+				// - Running dpkg
+				// loops ALL
+				// -  0 Installing pkg (sometimes this is skiped)
+				// - 25 Preparing pkg
+				// - 50 Unpacking pkg
+				// - 75 Preparing to configure pkg
+				//   ** Some pkgs have
+				//   - Running post-installation
+				//   - Running dpkg
+				// reloops all
+				// -   0 Configuring pkg
+				// - +25 Configuring pkg (SOMETIMES)
+				// - 100 Installed pkg
+				// after all
+				// - Running post-installation
+
+				// REMOVE
+				// - Running dpkg
+				// loops
+				// - 25  Removing pkg
+				// - 50  Preparing for removal of pkg
+				// - 75  Removing pkg
+				// - 100 Removed pkg
+				// after all
+				// - Running post-installation
+
 				// Let's start parsing the status:
-				if (starts_with(str, "Preparing")) {
+				if (starts_with(str, "Preparing to configure")) {
+					// Preparing to Install/configure
+					cout << "Found Preparing to configure! " << line << endl;
+					// The next item might be Configuring so better it be 100
+					m_lastSubProgress = 100;
+					emitTransactionPackage(pkg, PK_INFO_ENUM_PREPARING);
+					pk_backend_set_sub_percentage(m_backend, 75);
+				} else if (starts_with(str, "Preparing for removal")) {
+					// Preparing to Install/configure
+					cout << "Found Preparing for removal! " << line << endl;
+					m_lastSubProgress = 50;
+					emitTransactionPackage(pkg, PK_INFO_ENUM_REMOVING);
+					pk_backend_set_sub_percentage(m_backend, m_lastSubProgress);
+				} else if (starts_with(str, "Preparing")) {
 					// Preparing to Install/configure
 					cout << "Found Preparing! " << line << endl;
+					// if last package is different then finish it
+					if (!m_lastPackage.empty() && m_lastPackage.compare(pkg) != 0) {
+						cout << "FINISH the last package: " << m_lastPackage << endl;
+						emitTransactionPackage(m_lastPackage, PK_INFO_ENUM_FINISHED);
+					}
 					emitTransactionPackage(pkg, PK_INFO_ENUM_PREPARING);
+					pk_backend_set_sub_percentage(m_backend, 25);
 				} else if (starts_with(str, "Unpacking")) {
 					cout << "Found Unpacking! " << line << endl;
 					emitTransactionPackage(pkg, PK_INFO_ENUM_DECOMPRESSING);
+					pk_backend_set_sub_percentage(m_backend, 50);
 				} else if (starts_with(str, "Configuring")) {
 					// Installing Package
 					cout << "Found Configuring! " << line << endl;
+					if (m_lastSubProgress >= 100 && !m_lastPackage.empty()) {
+						cout << "FINISH the last package: " << m_lastPackage << endl;
+						emitTransactionPackage(m_lastPackage, PK_INFO_ENUM_FINISHED);
+						m_lastSubProgress = 0;
+					}
 					emitTransactionPackage(pkg, PK_INFO_ENUM_INSTALLING);
+					pk_backend_set_sub_percentage(m_backend, m_lastSubProgress);
+					m_lastSubProgress += 25;
 				} else if (starts_with(str, "Running dpkg")) {
 					cout << "Found Running dpkg! " << line << endl;
 				} else if (starts_with(str, "Running")) {
 					cout << "Found Running! " << line << endl;
-					emitTransactionPackage(pkg, PK_INFO_ENUM_CLEANUP);
+					pk_backend_set_status (m_backend, PK_STATUS_ENUM_COMMIT);
 				} else if (starts_with(str, "Installing")) {
 					cout << "Found Installing! " << line << endl;
+					// FINISH the last package
+					if (!m_lastPackage.empty()) {
+						cout << "FINISH the last package: " << m_lastPackage << endl;
+						emitTransactionPackage(m_lastPackage, PK_INFO_ENUM_FINISHED);
+					}
+					m_lastSubProgress = 0;
 					emitTransactionPackage(pkg, PK_INFO_ENUM_INSTALLING);
+					pk_backend_set_sub_percentage(m_backend, 0);
 				} else if (starts_with(str, "Removing")) {
 					cout << "Found Removing! " << line << endl;
+					if (m_lastSubProgress >= 100 && !m_lastPackage.empty()) {
+						cout << "FINISH the last package: " << m_lastPackage << endl;
+						emitTransactionPackage(m_lastPackage, PK_INFO_ENUM_FINISHED);
+					}
+					m_lastSubProgress += 25;
 					emitTransactionPackage(pkg, PK_INFO_ENUM_REMOVING);
+					pk_backend_set_sub_percentage(m_backend, m_lastSubProgress);
 				} else if (starts_with(str, "Installed") ||
-					   starts_with(str, "Removed")) {
+					       starts_with(str, "Removed")) {
 					cout << "Found FINISHED! " << line << endl;
+					m_lastSubProgress = 100;
 					emitTransactionPackage(pkg, PK_INFO_ENUM_FINISHED);
 				} else {
 					cout << ">>>Unmaped value<<< :" << line << endl;
 				}
+
+				if (!starts_with(str, "Running")) {
+					m_lastPackage = pkg;
+				}
 				m_startCounting = true;
 			} else {
 				m_startCounting = true;
@@ -1314,7 +1401,7 @@ void aptcc::updateInterface(int fd, int writeFd)
 /* Remove unused automatic packages */
 bool aptcc::DoAutomaticRemove(pkgCacheFile &Cache)
 {
-	bool doAutoRemove = _config->FindB("APT::Get::AutomaticRemove", true);
+	bool doAutoRemove = _config->FindB("APT::Get::AutomaticRemove", false);
 	pkgDepCache::ActionGroup group(*Cache);
 
 	if (_config->FindB("APT::Get::Remove",true) == false &&
@@ -1367,11 +1454,10 @@ bool aptcc::runTransaction(vector<pair<pkgCache::PkgIterator, pkgCache::VerItera
 		// failed to open cache, try checkDeps then..
 		// || Cache.CheckDeps(CmdL.FileSize() != 1) == false
 		if (WithLock == false || (timeout <= 0)) {
-			pk_backend_error_code(m_backend,
-					      PK_ERROR_ENUM_NO_CACHE,
-					      "Could not open package cache.");
+			show_errors(m_backend, PK_ERROR_ENUM_CANNOT_GET_LOCK);
 			return false;
 		} else {
+			_error->Discard();
 			pk_backend_set_status (m_backend, PK_STATUS_ENUM_WAITING_FOR_LOCK);
 			sleep(1);
 			timeout--;
@@ -1555,7 +1641,7 @@ bool aptcc::installPackages(pkgCacheFile &Cache)
 	{
  	    cout << DebBytes << ',' << Cache->DebSize() << endl;
 cout << "How odd.. The sizes didn't match, email apt@packages.debian.org";
-		_error->Warning("How odd.. The sizes didn't match, email apt@packages.debian.org");
+/*		_error->Warning("How odd.. The sizes didn't match, email apt@packages.debian.org");*/
 	}
 
 	// Number of bytes
@@ -1605,6 +1691,8 @@ cout << "How odd.. The sizes didn't match, email apt@packages.debian.org";
 		return false;
 	}
 
+	pk_backend_set_status (m_backend, PK_STATUS_ENUM_DOWNLOAD);
+	pk_backend_set_simultaneous_mode(m_backend, true);
 	// Download and check if we can continue
 	if (fetcher.Run() != pkgAcquire::Continue
 	    && _cancel == false)
@@ -1613,12 +1701,16 @@ cout << "How odd.. The sizes didn't match, email apt@packages.debian.org";
 		show_errors(m_backend, PK_ERROR_ENUM_PACKAGE_DOWNLOAD_FAILED);
 		return false;
 	}
+	pk_backend_set_simultaneous_mode(m_backend, false);
 
 	if (_error->PendingError() == true) {
 		cout << "PendingError download" << endl;
 		return false;
 	}
 
+	// Right now it's not safe to cancel
+	pk_backend_set_allow_cancel (m_backend, false);
+
 	// TODO true or false?
 	if (_cancel) {
 		return true;
@@ -1673,7 +1765,15 @@ cout << "How odd.. The sizes didn't match, email apt@packages.debian.org";
 		setlocale(LC_ALL, "C");
 
 		// Debconf handlying
-		setenv("DEBIAN_FRONTEND", "dbus", 1);
+		gchar *socket;
+		if (socket = pk_backend_get_frontend_socket(m_backend)) {
+			setenv("DEBIAN_FRONTEND", "passthrough", 1);
+			setenv("DEBCONF_PIPE", socket, 1);
+		} else {
+			// we don't have a socket set, let's fallback to noninteractive
+			setenv("DEBIAN_FRONTEND", "noninteractive", 1);
+		}
+
 		gchar *locale;
 		// Set the LANGUAGE so debconf messages get localization
 		if (locale = pk_backend_get_locale(m_backend)) {
