@@ -20,6 +20,7 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#include <stdio.h>
 #include <apt-pkg/init.h>
 #include <apt-pkg/algorithms.h>
 
@@ -33,9 +34,12 @@
 #include "rsources.h"
 
 #include <config.h>
+#include <pk-backend.h>
+#include <pk-backend-spawn.h>
 
 /* static bodges */
 static bool _cancel = false;
+static PkBackendSpawn *spawn;
 
 /**
  * backend_initialize:
@@ -45,15 +49,14 @@ backend_initialize (PkBackend *backend)
 {
 	egg_debug ("APTcc Initializing");
 
-	// make sure we do not get a graphical debconf
-	setenv("DEBIAN_FRONTEND", "noninteractive", 1);
-	setenv("APT_LISTCHANGES_FRONTEND", "none", 1);
-
 	if (pkgInitConfig(*_config) == false ||
 	    pkgInitSystem(*_config, _system) == false)
 	{
 		egg_debug ("ERROR initializing backend");
 	}
+
+	spawn = pk_backend_spawn_new ();
+	pk_backend_spawn_set_name (spawn, "aptcc");
 }
 
 /**
@@ -220,6 +223,15 @@ backend_get_requires (PkBackend *backend, PkBitfield filters, gchar **package_id
 	pk_backend_thread_create (backend, backend_get_depends_or_requires_thread);
 }
 
+/**
+ * backend_get_distro_upgrades:
+ */
+static void
+backend_get_distro_upgrades (PkBackend *backend)
+{
+	pk_backend_spawn_helper (spawn, "get-distro-upgrade.py", "get-distro-upgrades", NULL);
+}
+
 static gboolean
 backend_get_files_thread (PkBackend *backend)
 {
@@ -543,7 +555,7 @@ backend_what_provides_thread (PkBackend *backend)
 
 		delete m_apt;
 	} else {
-		provides_text = pk_provides_enum_to_text (provides);
+		provides_text = pk_provides_enum_to_string (provides);
 		pk_backend_error_code (backend,
 				       PK_ERROR_ENUM_NOT_SUPPORTED,
 				       "Provides %s not supported",
@@ -562,7 +574,7 @@ static void
 backend_what_provides (PkBackend *backend,
 		       PkBitfield filters,
 		       PkProvidesEnum provide,
-		       const gchar *search)
+		       gchar **values)
 {
 	pk_backend_thread_create (backend, backend_what_provides_thread);
 }
@@ -729,9 +741,7 @@ backend_refresh_cache_thread (PkBackend *backend)
 	AcqPackageKitStatus Stat(m_apt, backend, _cancel);
 
 	// do the work
-	if (_config->FindB("APT::Get::Download",true) == true) {
-		ListUpdate(Stat, *m_apt->packageSourceList);
-	}
+	ListUpdate(Stat, *m_apt->packageSourceList);
 
 	// Rebuild the cache.
 	pkgCacheFile Cache;
@@ -845,12 +855,12 @@ backend_resolve (PkBackend *backend, PkBitfield filters, gchar **packages)
 }
 
 static gboolean
-backend_search_file_thread (PkBackend *backend)
+backend_search_files_thread (PkBackend *backend)
 {
-	const gchar *search;
+	gchar **values;
 	PkBitfield filters;
 
-	search = pk_backend_get_string (backend, "search");
+	values = pk_backend_get_strv (backend, "search");
 	filters = (PkBitfield) pk_backend_get_uint (backend, "filters");
 
 	pk_backend_set_allow_cancel (backend, true);
@@ -867,7 +877,7 @@ backend_search_file_thread (PkBackend *backend)
 		}
 
 		pk_backend_set_status (backend, PK_STATUS_ENUM_QUERY);
-		vector<string> packages = search_file (backend, search, _cancel);
+		vector<string> packages = search_files (backend, values, _cancel);
 		vector<pair<pkgCache::PkgIterator, pkgCache::VerIterator> > output;
 		for(vector<string>::iterator i = packages.begin();
 		    i != packages.end(); ++i)
@@ -894,36 +904,38 @@ backend_search_file_thread (PkBackend *backend)
 }
 
 /**
- * backend_search_file:
+ * backend_search_files:
  */
 static void
-backend_search_file (PkBackend *backend, PkBitfield filters, const gchar *search)
+backend_search_files (PkBackend *backend, PkBitfield filters, gchar **values)
 {
-	pk_backend_thread_create (backend, backend_search_file_thread);
+	pk_backend_thread_create (backend, backend_search_files_thread);
 }
 
 static gboolean
-backend_search_group_thread (PkBackend *backend)
+backend_search_groups_thread (PkBackend *backend)
 {
-	const gchar *group;
+	gchar **values;
 	PkBitfield filters;
+	vector<PkGroupEnum> groups;
 
-	group = pk_backend_get_string (backend, "search");
+	values = pk_backend_get_strv (backend, "search");
 	filters = (PkBitfield) pk_backend_get_uint (backend, "filters");
 	pk_backend_set_allow_cancel (backend, true);
 
-	if (group == NULL) {
-		pk_backend_error_code (backend,
-				       PK_ERROR_ENUM_GROUP_NOT_FOUND,
-				       group);
-		pk_backend_finished (backend);
-		return false;
+	int len = g_strv_length(values);
+	for (uint i = 0; i < len; i++) {
+		if (values[i] == NULL) {
+			pk_backend_error_code (backend,
+					       PK_ERROR_ENUM_GROUP_NOT_FOUND,
+					       values[i]);
+			pk_backend_finished (backend);
+			return false;
+		} else {
+			groups.push_back(pk_group_enum_from_string(values[i]));
+		}
 	}
 
-	pk_backend_set_percentage (backend, 0);
-
-	PkGroupEnum pkGroup = pk_group_enum_from_text (group);
-
 	aptcc *m_apt = new aptcc(backend, _cancel);
 	pk_backend_set_pointer(backend, "aptcc_obj", m_apt);
 	if (m_apt->init()) {
@@ -954,8 +966,13 @@ backend_search_group_thread (PkBackend *backend)
 			section = section.substr(found + 1);
 
 			// Don't insert virtual packages instead add what it provides
-			if (pkGroup == get_enum_group(section)) {
-				output.push_back(pair<pkgCache::PkgIterator, pkgCache::VerIterator>(pkg, ver));
+			for (vector<PkGroupEnum>::iterator i = groups.begin();
+			     i != groups.end();
+			     ++i) {
+				if (*i == get_enum_group(section)) {
+					output.push_back(pair<pkgCache::PkgIterator, pkgCache::VerIterator>(pkg, ver));
+					break;
+				}
 			}
 		}
 	}
@@ -971,27 +988,30 @@ backend_search_group_thread (PkBackend *backend)
 }
 
 /**
- * backend_search_group:
+ * backend_search_groups:
  */
 static void
-backend_search_group (PkBackend *backend, PkBitfield filters, const gchar *pkGroup)
+backend_search_groups (PkBackend *backend, PkBitfield filters, gchar **values)
 {
-	pk_backend_thread_create (backend, backend_search_group_thread);
+	pk_backend_thread_create (backend, backend_search_groups_thread);
 }
 
 static gboolean
 backend_search_package_thread (PkBackend *backend)
 {
-	const gchar *search;
+	gchar **values;
+	gchar *search;
 	PkBitfield filters;
 
-	search = pk_backend_get_string (backend, "search");
+	values = pk_backend_get_strv (backend, "search");
+	search = g_strjoinv("|", values);
 	filters = (PkBitfield) pk_backend_get_uint (backend, "filters");
 
 	pk_backend_set_percentage (backend, PK_BACKEND_PERCENTAGE_INVALID);
 	pk_backend_set_allow_cancel (backend, true);
 
-	matcher *m_matcher = new matcher(string(search));
+	matcher *m_matcher = new matcher(search);
+	g_free(search);
 	if (m_matcher->hasError()) {
 		egg_debug("Regex compilation error");
 		delete m_matcher;
@@ -1128,10 +1148,10 @@ backend_search_package_thread (PkBackend *backend)
 }
 
 /**
- * backend_search_name:
+ * backend_search_names:
  */
 static void
-backend_search_name (PkBackend *backend, PkBitfield filters, const gchar *search)
+backend_search_names (PkBackend *backend, PkBitfield filters, gchar **values)
 {
 	pk_backend_set_bool(backend, "search_details", false);
 	pk_backend_thread_create(backend, backend_search_package_thread);
@@ -1141,7 +1161,7 @@ backend_search_name (PkBackend *backend, PkBitfield filters, const gchar *search
  * backend_search_details:
  */
 static void
-backend_search_details (PkBackend *backend, PkBitfield filters, const gchar *search)
+backend_search_details (PkBackend *backend, PkBitfield filters, gchar **values)
 {
 	pk_backend_set_bool(backend, "search_details", true);
 	pk_backend_thread_create(backend, backend_search_package_thread);
@@ -1254,7 +1274,7 @@ backend_remove_packages (PkBackend *backend, gchar **package_ids, gboolean allow
  * backend_simulate_remove_packages:
  */
 static void
-backend_simulate_remove_packages (PkBackend *backend, gchar **packages)
+backend_simulate_remove_packages (PkBackend *backend, gchar **packages, gboolean autoremove)
 {
 	pk_backend_set_bool(backend, "simulate", true);
 	pk_backend_set_bool(backend, "remove", true);
@@ -1453,7 +1473,7 @@ extern "C" PK_BACKEND_OPTIONS (
 	NULL,						/* get_categories */
 	backend_get_depends,				/* get_depends */
 	backend_get_details,				/* get_details */
-	NULL,						/* get_distro_upgrades */
+	backend_get_distro_upgrades,			/* get_distro_upgrades */
 	backend_get_files,				/* get_files */
 	backend_get_packages,				/* get_packages */
 	backend_get_repo_list,				/* get_repo_list */
@@ -1470,9 +1490,9 @@ extern "C" PK_BACKEND_OPTIONS (
 	backend_resolve,				/* resolve */
 	NULL,						/* rollback */
 	backend_search_details,				/* search_details */
-	backend_search_file,				/* search_file */
-	backend_search_group,				/* search_group */
-	backend_search_name,				/* search_name */
+	backend_search_files,				/* search_files */
+	backend_search_groups,				/* search_groups */
+	backend_search_names,				/* search_names */
 	backend_install_update_packages,		/* update_packages */
 	backend_update_system,				/* update_system */
 	backend_what_provides,				/* what_provides */
