@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2007 Grzegorz Dąbrowski <gdx@o2.pl>
+ * Copyright (C) 2007 Grzegorz Dąbrowski grzegorz.dabrowski@gmail.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -21,8 +21,6 @@
 
 #include <gmodule.h>
 #include <glib.h>
-#include <string.h>
-#include <unistd.h>
 #include <pk-backend.h>
 #include <egg-debug.h>
 
@@ -88,7 +86,7 @@ add_packages_from_list (PkBackend *backend, GList *list, gboolean updates)
 
 	for (li = list; li != NULL; li = li->next) {
 		package = (PackageSearch*)li->data;
-		pkg_string = pk_package_id_build(package->package, package->version, package->arch, package->reponame);
+		pkg_string = pk_package_id_build (package->package, package->version, package->arch, package->reponame);
 		if (updates == TRUE)
 			info = PK_INFO_ENUM_NORMAL;
 		else if (package->installed)
@@ -96,7 +94,7 @@ add_packages_from_list (PkBackend *backend, GList *list, gboolean updates)
 		else
 			info = PK_INFO_ENUM_AVAILABLE;
 		pk_backend_package (backend, info, pkg_string, package->description);
-		g_free(pkg_string);
+		g_free (pkg_string);
 	}
 }
 
@@ -105,14 +103,18 @@ backend_find_packages_thread (PkBackend *backend)
 {
 	PkBitfield filters;
 	const gchar *search;
+	gchar **values;
 	guint mode;
 	GList *list = NULL;
 	sqlite3 *db = NULL;
 	gint filter_box = 0;
 
+
 	filters = pk_backend_get_uint (backend, "filters");
 	mode = pk_backend_get_uint (backend, "mode");
-	search = pk_backend_get_string (backend, "search");
+	values = pk_backend_get_strv (backend, "search");
+	/* FIXME: support multiple packages */
+	search = values[0];
 
 	pk_backend_set_status (backend, PK_STATUS_ENUM_QUERY);
 
@@ -140,7 +142,7 @@ backend_find_packages_thread (PkBackend *backend)
 
 	pk_backend_set_percentage (backend, PK_BACKEND_PERCENTAGE_INVALID);
 
-	db = db_open();
+	db = db_open ();
 
 	if (mode == SEARCH_TYPE_FILE) {
 		list = box_db_repos_search_file_with_filter (db, search, filter_box);
@@ -155,21 +157,57 @@ backend_find_packages_thread (PkBackend *backend)
 		     pk_bitfield_contain (filters, PK_FILTER_ENUM_NOT_INSTALLED)) ||
 		    (!pk_bitfield_contain (filters, PK_FILTER_ENUM_INSTALLED) &&
 		     !pk_bitfield_contain (filters, PK_FILTER_ENUM_NOT_INSTALLED))) {
-			list = box_db_repos_packages_search_all(db, (gchar *)search, filter_box);
+			list = box_db_repos_packages_search_all (db, (gchar *)search, filter_box);
 		} else if (pk_bitfield_contain (filters, PK_FILTER_ENUM_INSTALLED)) {
-			list = box_db_repos_packages_search_installed(db, (gchar *)search, filter_box);
+			list = box_db_repos_packages_search_installed (db, (gchar *)search, filter_box);
 		} else if (pk_bitfield_contain (filters, PK_FILTER_ENUM_NOT_INSTALLED)) {
-			list = box_db_repos_packages_search_available(db, (gchar *)search, filter_box);
+			list = box_db_repos_packages_search_available (db, (gchar *)search, filter_box);
 		}
 		add_packages_from_list (backend, list, FALSE);
 		box_db_repos_package_list_free (list);
 	}
 
+	db_close (db);
+	pk_backend_finished (backend);
+	return TRUE;
+}
+
+
+static gboolean
+backend_get_packages_thread (PkBackend *backend)
+{
+	PkBitfield filters;
+	GList *list = NULL;
+	sqlite3 *db = NULL;
+
+	filters = pk_backend_get_uint (backend, "filters");
+
+	pk_backend_set_status (backend, PK_STATUS_ENUM_QUERY);
+
+	pk_backend_set_percentage (backend, PK_BACKEND_PERCENTAGE_INVALID);
+
+	db = db_open();
+
+	if ((pk_bitfield_contain (filters, PK_FILTER_ENUM_INSTALLED) &&
+	     pk_bitfield_contain (filters, PK_FILTER_ENUM_NOT_INSTALLED)) ||
+	    (!pk_bitfield_contain (filters, PK_FILTER_ENUM_INSTALLED) &&
+	     !pk_bitfield_contain (filters, PK_FILTER_ENUM_NOT_INSTALLED))) {
+		list = box_db_repos_packages_search_all (db, NULL, 0);
+	} else if (pk_bitfield_contain (filters, PK_FILTER_ENUM_INSTALLED)) {
+		list = box_db_repos_packages_search_installed (db, NULL, 0);
+	} else if (pk_bitfield_contain (filters, PK_FILTER_ENUM_NOT_INSTALLED)) {
+		list = box_db_repos_packages_search_available (db, NULL, 0);
+	}
+
+	add_packages_from_list (backend, list, FALSE);
+	box_db_repos_package_list_free (list);
+
 	db_close(db);
 	pk_backend_finished (backend);
 	return TRUE;
 }
 
+
 static gboolean
 backend_get_updates_thread (PkBackend *backend)
 {
@@ -204,22 +242,19 @@ backend_update_system_thread (PkBackend *backend)
 static gboolean
 backend_install_packages_thread (PkBackend *backend)
 {
-	gboolean result;
-	PkPackageId *pi;
+	gboolean result = TRUE;
 	gchar **package_ids;
+	size_t i;
 
 	pk_backend_set_status (backend, PK_STATUS_ENUM_QUERY);
 
 	/* FIXME: support only_trusted */
 
-	package_id = pk_backend_get_string (backend, "package_id");
-	pi = pk_package_id_new_from_string (package_id);
-	if (pi == NULL) {
-		pk_backend_error_code (backend, PK_ERROR_ENUM_PACKAGE_ID_INVALID, "invalid package id");
-		pk_package_id_free (pi);
-		return FALSE;
-	}
-	result = box_package_install(pi->name, ROOT_DIRECTORY, common_progress, backend, FALSE);
+	package_ids = pk_backend_get_strv (backend, "package_ids");
+        for (i = 0; i < g_strv_length (package_ids); i++) {
+		gchar **package_id_data = pk_package_id_split (package_ids[i]);
+		result = box_package_install (package_id_data[PK_PACKAGE_ID_NAME], ROOT_DIRECTORY, common_progress, backend, FALSE);
+        }
 
 	pk_backend_finished (backend);
 
@@ -231,8 +266,7 @@ backend_update_packages_thread (PkBackend *backend)
 {
 	gboolean result = TRUE;
 	gchar **package_ids;
-	PkPackageId *pi;
-	gint i;
+	size_t i;
 
 	pk_backend_set_status (backend, PK_STATUS_ENUM_QUERY);
 	/* FIXME: support only_trusted */
@@ -240,15 +274,7 @@ backend_update_packages_thread (PkBackend *backend)
 
 	for (i = 0; i < g_strv_length (package_ids); i++)
 	{
-		pi = pk_package_id_new_from_string (package_ids[i]);
-		if (pi == NULL) {
-			pk_backend_error_code (backend, PK_ERROR_ENUM_PACKAGE_ID_INVALID, "invalid package id");
-			pk_package_id_free (pi);
-			g_strfreev (package_ids);
-
-			return FALSE;
-		}
-		result |= box_package_install(pi->name, ROOT_DIRECTORY, common_progress, backend, FALSE);
+		result |= box_package_install (package_ids[i], ROOT_DIRECTORY, common_progress, backend, FALSE);
 	}
 
 	pk_backend_finished (backend);
@@ -264,7 +290,7 @@ backend_install_files_thread (PkBackend *backend)
 	pk_backend_set_status (backend, PK_STATUS_ENUM_QUERY);
 
 	full_paths = pk_backend_get_strv (backend, "full_paths");
-	result = box_package_install(full_paths[0], ROOT_DIRECTORY, common_progress, backend, FALSE);
+	result = box_package_install (full_paths[0], ROOT_DIRECTORY, common_progress, backend, FALSE);
 
 	pk_backend_finished (backend);
 
@@ -274,43 +300,35 @@ backend_install_files_thread (PkBackend *backend)
 static gboolean
 backend_get_details_thread (PkBackend *backend)
 {
-	PkPackageId *pi;
 	PackageSearch *ps;
 	GList *list;
 	sqlite3 *db;
 	gchar **package_ids;
+	gchar **package_id_data;
 
-	package_id = pk_backend_get_string (backend, "package_id");
-	db = db_open();
+	package_ids = pk_backend_get_strv (backend, "package_ids");
+	/* FIXME: support multiple packages */
+	package_id_data = pk_package_id_split (package_ids[0]);
 
-	pi = pk_package_id_new_from_string (package_id);
-	if (pi == NULL) {
-		pk_backend_error_code (backend, PK_ERROR_ENUM_PACKAGE_ID_INVALID, "invalid package id");
-		pk_package_id_free (pi);
-		db_close (db);
-
-		return FALSE;
-	}
+	db = db_open ();
 
 	pk_backend_set_status (backend, PK_STATUS_ENUM_QUERY);
 
 	/* only one element is returned */
-	list = box_db_repos_packages_search_by_data(db, pi->name, pi->version);
+	list = box_db_repos_packages_search_by_data (db, package_id_data[PK_PACKAGE_ID_NAME], package_id_data[PK_PACKAGE_ID_VERSION]);
 
 	if (list == NULL) {
 		pk_backend_error_code (backend, PK_ERROR_ENUM_PACKAGE_ID_INVALID, "cannot find package by id");
-		pk_package_id_free (pi);
 		db_close (db);
 		return FALSE;
 	}
 	ps = (PackageSearch*) list->data;
 
-	pk_backend_details (backend, package_id, "unknown", PK_GROUP_ENUM_OTHER, ps->description, "", 0);
+	pk_backend_details (backend, package_ids[0], "unknown", PK_GROUP_ENUM_OTHER, ps->description, "", 0);
 
-	pk_package_id_free (pi);
 	box_db_repos_package_list_free (list);
 
-	db_close(db);
+	db_close (db);
 	pk_backend_finished (backend);
 	return TRUE;
 }
@@ -318,30 +336,22 @@ backend_get_details_thread (PkBackend *backend)
 static gboolean
 backend_get_files_thread (PkBackend *backend)
 {
-	PkPackageId *pi;
 	gchar *files;
 	sqlite3 *db;
 	gchar **package_ids;
+	gchar **package_id_data;
 
 	db = db_open();
-	package_id = pk_backend_get_string (backend, "package_id");
-
-	pi = pk_package_id_new_from_string (package_id);
-	if (pi == NULL) {
-		pk_backend_error_code (backend, PK_ERROR_ENUM_PACKAGE_ID_INVALID, "invalid package id");
-		pk_package_id_free (pi);
-		db_close (db);
-
-		return FALSE;
-	}
+	package_ids = pk_backend_get_strv (backend, "package_ids");
+	/* FIXME: support multiple packages */
+	package_id_data = pk_package_id_split (package_ids[0]);
 
 	pk_backend_set_status (backend, PK_STATUS_ENUM_QUERY);
 
-	files = box_db_repos_get_files_string (db, pi->name, pi->version);
-	pk_backend_files (backend, package_id, files);
+	files = box_db_repos_get_files_string (db, package_id_data[PK_PACKAGE_ID_NAME], package_id_data[PK_PACKAGE_ID_VERSION]);
+	pk_backend_files (backend, package_ids[0], files);
 
-	pk_package_id_free (pi);
-	db_close(db);
+	db_close (db);
 	g_free (files);
 
 	pk_backend_finished (backend);
@@ -351,34 +361,27 @@ backend_get_files_thread (PkBackend *backend)
 static gboolean
 backend_get_depends_requires_thread (PkBackend *backend)
 {
-	PkPackageId *pi;
 	GList *list = NULL;
 	sqlite3 *db;
 	gchar **package_ids;
 	int deps_type;
+        gchar **package_id_data;
 
 	db = db_open ();
-	package_id = pk_backend_get_string (backend, "package_id");
+	package_ids = pk_backend_get_strv (backend, "package_ids");
 	deps_type = pk_backend_get_uint (backend, "type");
-
-	pi = pk_package_id_new_from_string (package_id);
-	if (pi == NULL) {
-		pk_backend_error_code (backend, PK_ERROR_ENUM_PACKAGE_ID_INVALID, "invalid package id");
-		db_close (db);
-
-		return FALSE;
-	}
+	/* FIXME: support multiple packages */
+	package_id_data = pk_package_id_split (package_ids[0]);
 
 	pk_backend_set_status (backend, PK_STATUS_ENUM_QUERY);
 
 	if (deps_type == DEPS_TYPE_DEPENDS)
-		list = box_db_repos_get_depends(db, pi->name);
+		list = box_db_repos_get_depends (db, package_id_data[PK_PACKAGE_ID_NAME]);
 	else if (deps_type == DEPS_TYPE_REQUIRES)
-		list = box_db_repos_get_requires(db, pi->name);
+		list = box_db_repos_get_requires (db, package_id_data[PK_PACKAGE_ID_NAME]);
 
 	add_packages_from_list (backend, list, FALSE);
 	box_db_repos_package_list_free (list);
-	pk_package_id_free (pi);
 
 	db_close (db);
 
@@ -389,25 +392,20 @@ backend_get_depends_requires_thread (PkBackend *backend)
 static gboolean
 backend_remove_packages_thread (PkBackend *backend)
 {
-	PkPackageId *pi;
 	gchar **package_ids;
+	gchar **package_id_data;
 
 	package_ids = pk_backend_get_strv (backend, "package_ids");
-	pi = pk_package_id_new_from_string (package_ids[0]);
-	if (pi == NULL) {
-		pk_backend_error_code (backend, PK_ERROR_ENUM_PACKAGE_ID_INVALID, "invalid package id");
-
-		return FALSE;
-	}
+	/* FIXME: support multiple packages */
+	package_id_data = pk_package_id_split (package_ids[0]);
 
 	pk_backend_set_status (backend, PK_STATUS_ENUM_REMOVE);
 
-	if (!box_package_uninstall (pi->name, ROOT_DIRECTORY, common_progress, backend, FALSE))
+	if (!box_package_uninstall (package_id_data[PK_PACKAGE_ID_NAME], ROOT_DIRECTORY, common_progress, backend, FALSE))
 	{
 		pk_backend_error_code (backend, PK_ERROR_ENUM_DEP_RESOLUTION_FAILED, "Cannot uninstall");
 	}
 
-	pk_package_id_free (pi);
 	pk_backend_finished (backend);
 	return TRUE;
 }
@@ -453,6 +451,16 @@ backend_get_filters (PkBackend *backend)
 		-1);
 }
 
+/**
+ * backend_get_mime_types:
+ */
+static gchar *
+backend_get_mime_types (PkBackend *backend)
+{
+        return g_strdup ("application/x-box-package");
+}
+
+
 /**
  * backend_get_depends:
  */
@@ -460,6 +468,8 @@ static void
 backend_get_depends (PkBackend *backend, PkBitfield filters, gchar **package_ids, gboolean recursive)
 {
 	pk_backend_set_uint (backend, "type", DEPS_TYPE_DEPENDS);
+	pk_backend_set_strv (backend, "package_ids", package_ids);
+	/* TODO: param recursive */
 	pk_backend_thread_create (backend, backend_get_depends_requires_thread);
 }
 
@@ -469,6 +479,7 @@ backend_get_depends (PkBackend *backend, PkBitfield filters, gchar **package_ids
 static void
 backend_get_details (PkBackend *backend, gchar **package_ids)
 {
+	pk_backend_set_strv (backend, "package_ids", package_ids);
 	pk_backend_thread_create (backend, backend_get_details_thread);
 }
 
@@ -478,9 +489,20 @@ backend_get_details (PkBackend *backend, gchar **package_ids)
 static void
 backend_get_files (PkBackend *backend, gchar **package_ids)
 {
+	pk_backend_set_strv (backend, "package_ids", package_ids);
 	pk_backend_thread_create (backend, backend_get_files_thread);
 }
 
+/**
+ * backend_get_packages:
+ */
+static void
+backend_get_packages (PkBackend *backend, PkBitfield filters)
+{
+	pk_backend_set_uint (backend, "filters", filters);
+	pk_backend_thread_create (backend, backend_get_packages_thread);
+}
+
 /**
  * backend_get_requires:
  */
@@ -488,6 +510,8 @@ static void
 backend_get_requires (PkBackend *backend, PkBitfield filters, gchar **package_ids, gboolean recursive)
 {
 	pk_backend_set_uint (backend, "type", DEPS_TYPE_REQUIRES);
+	pk_backend_set_strv (backend, "package_ids", package_ids);
+	/* TODO: param recursive */
 	pk_backend_thread_create (backend, backend_get_depends_requires_thread);
 }
 
@@ -497,6 +521,7 @@ backend_get_requires (PkBackend *backend, PkBitfield filters, gchar **package_id
 static void
 backend_get_updates (PkBackend *backend, PkBitfield filters)
 {
+	/* TODO: filters */
 	pk_backend_thread_create (backend, backend_get_updates_thread);
 }
 
@@ -512,6 +537,7 @@ backend_install_packages (PkBackend *backend, gboolean only_trusted, gchar **pac
 		pk_backend_finished (backend);
 		return;
 	}
+	pk_backend_set_strv (backend, "package_ids", package_ids);
 
 	pk_backend_thread_create (backend, backend_install_packages_thread);
 }
@@ -520,8 +546,9 @@ backend_install_packages (PkBackend *backend, gboolean only_trusted, gchar **pac
  * backend_install_files:
  */
 static void
-backend_install_files (PkBackend *backend, gboolean only_trusted, gchar **files)
+backend_install_files (PkBackend *backend, gboolean only_trusted, gchar **full_paths)
 {
+	pk_backend_set_strv (backend, "full_paths", full_paths);
 	pk_backend_thread_create (backend, backend_install_files_thread);
 }
 
@@ -537,6 +564,7 @@ backend_refresh_cache (PkBackend *backend, gboolean force)
 		pk_backend_finished (backend);
 		return;
 	}
+	/* FIXME: support force */
 	pk_backend_thread_create (backend, backend_refresh_cache_thread);
 }
 
@@ -544,9 +572,10 @@ backend_refresh_cache (PkBackend *backend, gboolean force)
  * backend_remove_packages:
  */
 static void
-backend_remove_packages (PkBackend *backend, gchar **package_id, gboolean allow_deps, gboolean autoremove)
+backend_remove_packages (PkBackend *backend, gchar **package_ids, gboolean allow_deps, gboolean autoremove)
 {
 	pk_backend_set_uint (backend, "type", DEPS_ALLOW);
+	pk_backend_set_strv (backend, "package_ids", package_ids);
 	pk_backend_thread_create (backend, backend_remove_packages_thread);
 }
 
@@ -554,9 +583,10 @@ backend_remove_packages (PkBackend *backend, gchar **package_id, gboolean allow_
  * backend_resolve:
  */
 static void
-backend_resolve (PkBackend *backend, PkBitfield filters, const gchar *package)
+backend_resolve (PkBackend *backend, PkBitfield filters, gchar **packages)
 {
 	pk_backend_set_uint (backend, "mode", SEARCH_TYPE_RESOLVE);
+	pk_backend_set_strv (backend, "search", packages);
 	pk_backend_thread_create (backend, backend_find_packages_thread);
 }
 
@@ -564,19 +594,21 @@ backend_resolve (PkBackend *backend, PkBitfield filters, const gchar *package)
  * backend_search_details:
  */
 static void
-backend_search_details (PkBackend *backend, PkBitfield filters, const gchar *search)
+backend_search_details (PkBackend *backend, PkBitfield filters, gchar **values)
 {
 	pk_backend_set_uint (backend, "mode", SEARCH_TYPE_DETAILS);
+	pk_backend_set_strv (backend, "search", values);
 	pk_backend_thread_create (backend, backend_find_packages_thread);
 }
 
 /**
- * backend_search_file:
+ * backend_search_files:
  */
 static void
-backend_search_file (PkBackend *backend, PkBitfield filters, const gchar *search)
+backend_search_files (PkBackend *backend, PkBitfield filters, gchar **values)
 {
 	pk_backend_set_uint (backend, "mode", SEARCH_TYPE_FILE);
+	pk_backend_set_strv (backend, "search", values);
 	pk_backend_thread_create (backend, backend_find_packages_thread);
 }
 
@@ -584,9 +616,10 @@ backend_search_file (PkBackend *backend, PkBitfield filters, const gchar *search
  * backend_search_name:
  */
 static void
-backend_search_name (PkBackend *backend, PkBitfield filters, const gchar *search)
+backend_search_names (PkBackend *backend, PkBitfield filters, gchar **values)
 {
 	pk_backend_set_uint (backend, "mode", SEARCH_TYPE_NAME);
+	pk_backend_set_strv (backend, "search", values);
 	pk_backend_thread_create (backend, backend_find_packages_thread);
 }
 
@@ -673,7 +706,7 @@ PK_BACKEND_OPTIONS (
 	NULL,					/* get_groups */
 	backend_get_filters,			/* get_filters */
 	NULL,					/* get_roles */
-	NULL,					/* get_mime_types */
+	backend_get_mime_types,			/* get_mime_types */
 	NULL,					/* cancel */
 	NULL,					/* download_packages */
 	NULL,					/* get_categories */
@@ -681,7 +714,7 @@ PK_BACKEND_OPTIONS (
 	backend_get_details,			/* get_details */
 	NULL,					/* get_distro_upgrades */
 	backend_get_files,			/* get_files */
-	NULL,					/* get_packages */
+	backend_get_packages,			/* get_packages */
 	backend_get_repo_list,			/* get_repo_list */
 	backend_get_requires,			/* get_requires */
 	NULL,					/* get_update_detail */
@@ -696,9 +729,9 @@ PK_BACKEND_OPTIONS (
 	backend_resolve,			/* resolve */
 	NULL,					/* rollback */
 	backend_search_details,			/* search_details */
-	backend_search_file,			/* search_file */
-	NULL,					/* search_group */
-	backend_search_name,			/* search_name */
+	backend_search_files,			/* search_files */
+	NULL,					/* search_groups */
+	backend_search_names,			/* search_names */
 	backend_update_packages,		/* update_packages */
 	backend_update_system,			/* update_system */
 	NULL,					/* what_provides */
@@ -707,4 +740,3 @@ PK_BACKEND_OPTIONS (
 	NULL,					/* simulate_remove_packages */
 	NULL					/* simulate_update_packages */
 );
-
