@@ -17,7 +17,8 @@
 #
 # Copyright (C) 2007 Ken VanDine <ken@vandine.org>
 # Copyright (C) 2008 Richard Hughes <richard@hughsie.com>
-# Copyright (C) 2008 Andres Vargas <zodman@foresightlinux.org>
+# Copyright (C) 2009-2010 Andres Vargas <zodman@foresightlinux.org>
+#                         Scott Parkerson <scott.parkerson@gmail.com>
 # }}}
 #{{{ LIBS
 import sys
@@ -38,7 +39,7 @@ from packagekit.progress import PackagekitProgress
 from conaryCallback import UpdateCallback, GetUpdateCallback
 from conaryCallback import RemoveCallback, UpdateSystemCallback
 from conaryFilter import *
-from XMLCache import XMLCache as Cache
+from XMLCache import XMLCache
 from conaryInit import *
 
 from conary import conarycfg, conaryclient
@@ -127,6 +128,7 @@ class PackageKitConaryBackend(PackageKitBaseBackend):
         self.conary = conary
         self.callback = UpdateCallback(self, self.cfg)
         self.client.setUpdateCallback(self.callback)
+        self.xmlcache = XMLCache()
 
     def _freezeData(self, version, flavor):
         frzVersion = version.freeze()
@@ -167,8 +169,7 @@ class PackageKitConaryBackend(PackageKitBaseBackend):
     def get_package_id(self, name, versionObj, flavor):
 
         version = versionObj.trailingRevision()
-        cache = Cache()
-        pkg = cache.resolve(name)
+        pkg = self.xmlcache.resolve(name)
         #pkg["shortDesc"] = "."
         arch = self._get_arch(flavor)
         #data = versionObj.asString() + "#"
@@ -229,10 +230,9 @@ class PackageKitConaryBackend(PackageKitBaseBackend):
             log.info("where %s" % where)
             self.error(ERROR_UNKNOWN, "DORK---- search where not found")
 
-        cache = Cache()
         log.debug((searchlist, where))
         log.info("||||||||||||||||||||||||||||searching  on cache... ")
-        pkgList = cache.search(searchlist, where )
+        pkgList = self.xmlcache.search(searchlist, where )
         log.info("|||||||||||||||||||||||||||||1end searching on cache... ")
 
         if len(pkgList) > 0 :
@@ -251,30 +251,37 @@ class PackageKitConaryBackend(PackageKitBaseBackend):
 
     def _get_update(self, applyList, cache=True):
         from conary.conaryclient.update import NoNewTrovesError,DepResolutionFailure
+        self.allow_cancel(False)
         updJob = self.client.newUpdateJob()
-        try:
-            log.info("prepare updateJOb...............")
-            log.info(applyList)
-            suggMap = self.client.prepareUpdateJob(updJob, applyList)
-            log.info("end prepare updateJOB..............")
-        except NoNewTrovesError:
-            #self.error(ERROR_NO_PACKAGES_TO_UPDATE, "No new apps were found")
-            return updJob, None
-        except DepResolutionFailure as error :
-            log.info(error.getErrorMessage())
-            deps =  error.cannotResolve
-            
-            dep_package = [ str(i[0][0]).split(":")[0] for i in deps ]
-            log.info(dep_package)
-            self.error(ERROR_DEP_RESOLUTION_FAILED,  "This package depends of:  %s" % ", ".join(set(dep_package)))
-        if cache:
-            Cache().cacheUpdateJob(applyList, updJob)
-        return updJob, suggMap
+        jobPath = self.xmlcache.checkCachedUpdateJob(applyList)
+        if cache and jobPath:
+            try:
+                log.info("Using previously cached update job at %s" % (jobPath,))
+                updJob.thaw(jobPath)
+            except IOError, err:
+                log.error("Failed to read update job at %s (error=%s)" % (jobPath, str(err)))
+                updJob = None
+        else:
+            log.info("Creating a new update job")
+            try:
+                suggMap = self.client.prepareUpdateJob(updJob, applyList)
+                log.info("Successfully created a new update job")
+                if cache:
+                    self.xmlcache.cacheUpdateJob(applyList, updJob)
+            except NoNewTrovesError:
+                return updJob, None
+            except DepResolutionFailure as error :
+                log.info(error.getErrorMessage())
+                deps =  error.cannotResolve
+                dep_package = [ str(i[0][0]).split(":")[0] for i in deps ]
+                log.info(dep_package)
+                self.error(ERROR_DEP_RESOLUTION_FAILED,  "This package depends of:  %s" % ", ".join(set(dep_package)))
 
-    def _do_update(self, applyList):
-        log.info("========= _do_update ========")
-        jobPath = Cache().checkCachedUpdateJob(applyList)
-        log.info(jobPath)
+        return updJob
+
+    def _do_update(self, applyList, simulate=False):
+        updJob = None
+        jobPath = self.xmlcache.checkCachedUpdateJob(applyList)
         if jobPath:
             updJob = self.client.newUpdateJob()
             try:
@@ -285,10 +292,11 @@ class PackageKitConaryBackend(PackageKitBaseBackend):
             updJob,suggMap = self._get_update(applyList, cache=False)
         self.allow_cancel(False)
         try:
-            restartDir = self.client.applyUpdateJob(updJob)
+            # TODO we should really handle the restart case here
+            restartDir = self.client.applyUpdateJob(updJob, test=simulate)
         except errors.InternalConaryError:
             self.error(ERROR_NO_PACKAGES_TO_UPDATE,"get-updates first and then update sytem")
-        except trove.TroveIntegrityError: 
+        except trove.TroveIntegrityError:
             self.error(ERROR_NO_PACKAGES_TO_UPDATE,"run get-updates again")
         return updJob
 
@@ -299,12 +307,12 @@ class PackageKitConaryBackend(PackageKitBaseBackend):
             applyList = [(name, (None, None), (version, flavor), True)]
         return self._get_update(applyList)
 
-    def _do_package_update(self, name, version, flavor):
+    def _do_package_update(self, name, version, flavor, simulate):
         if name.startswith('-'):
             applyList = [(name, (version, flavor), (None, None), False)]
         else:
             applyList = [(name, (None, None), (version, flavor), True)]
-        return self._do_update(applyList)
+        return self._do_update(applyList, simulate)
 
     def _resolve_list(self, filters):
 
@@ -367,30 +375,32 @@ class PackageKitConaryBackend(PackageKitBaseBackend):
         self.status(STATUS_INFO)
         log.info("filters: %s package:%s " % (filters, package))
 
-        cache = Cache()
-        pkg_dict = cache.resolve( package[0] )
+        pkg_dict = self.xmlcache.resolve( package[0] )
         log.info(pkg_dict)
         if pkg_dict is None:
             return None
         log.info("doing a resolve")
-
         filter = ConaryFilter(filters)
         trove = None
     
         trove_installed = self.conary.query( pkg_dict.get("name") )
+        log.info("end of conary query")
         if trove_installed:
             pkg = self._convert_package( trove_installed[0], pkg_dict )
             log.info( pkg)
             filter.add_installed( [ pkg ] )
-            
+        log.info("filter installed end.. doing a rq")    
         trove_available = self.conary.request_query( pkg_dict.get("name") )
+        log.info("end of conary rquery")
         if trove_available:
             pkg = self._convert_package( trove_available[0], pkg_dict )
             filter.add_available(  [ pkg ] )
 
+        log.info("filter available end")    
         package_list = filter.post_process()
         log.info("package_list %s" % package_list)
         self._show_package_list(package_list)
+	log.info("end resolve ...................")
 
     def _show_package_list(self, lst):
         """ 
@@ -568,20 +578,19 @@ class PackageKitConaryBackend(PackageKitBaseBackend):
         self.percentage(None)
         self.status(STATUS_REFRESH_CACHE)
         self.percentage(None)
-        cache = Cache()
-        cache.refresh()
+        self.xmlcache.refresh()
 
-    def install_packages(self, only_trusted, package_ids):
+    def install_packages(self, only_trusted, package_ids, simulate=False):
         """
             alias of update_packages
         """
 
         # FIXME: use only_trusted
 
-        self.update_packages(only_trusted, package_ids)
+        self.update_packages(only_trusted, package_ids, simulate)
 
     @ExceptionHandler
-    def update_packages(self, only_trusted, package_ids):
+    def update_packages(self, only_trusted, package_ids, simulate=False):
         '''
         Implement the {backend}-{install, update}-packages functionality
         '''
@@ -606,11 +615,11 @@ class PackageKitConaryBackend(PackageKitBaseBackend):
                 log.info(">>> Prepare Update")
                 self._get_package_update(name, version, flavor)
                 log.info(">>> end Prepare Update")
-                self._do_package_update(name, version, flavor)
+                self._do_package_update(name, version, flavor, simulate)
 
         
     @ExceptionHandler
-    def remove_packages(self, allowDeps, autoremove, package_ids):
+    def remove_packages(self, allowDeps, autoremove, package_ids, simulate=False):
         '''
         Implement the {backend}-remove-packages functionality
         '''
@@ -636,7 +645,7 @@ class PackageKitConaryBackend(PackageKitBaseBackend):
                 if callback.error:
                     self.error(ERROR_DEP_RESOLUTION_FAILED,', '.join(callback.error))
                         
-                self._do_package_update(name, version, flavor)
+                self._do_package_update(name, version, flavor, simulate)
         self.client.setUpdateCallback(self.callback)
 
     def _get_metadata(self, package_id, field):
@@ -708,31 +717,21 @@ class PackageKitConaryBackend(PackageKitBaseBackend):
         self.allow_cancel(True)
         self.percentage(None)
         self.status(STATUS_INFO)
-        package_id = package_ids[0]
-        log.info(package_id)
-        name, version,arch,summary  = pkpackage.get_package_from_id(package_id)
-        cache = Cache()
-        pkgDict = cache.resolve(name)
-        #update = self._get_updated(pkg)
-        update = ""
-        obsolete = ""
-        cve_url = ""
-        if pkgDict:
-            if "url" in pkgDict:
-                vendor_url = pkgDict["url"]
-            else:
-                vendor_url = ""
-            if "longDesc" in pkgDict:
-                desc = pkgDict["longDesc"]
-            else:
-                desc = ""
-            reboot = self._get_restart(pkgDict.get("name"))
-            state = self._get_branch( pkgDict.get("label"))
-            bz_url = self._get_fits(pkgDict.get("label"), pkgDict.get("name"))
-            #
-            #def update_detail(self, package_id, updates, obsoletes, vendor_url, bugzilla_url, cve_url, restart, update_text, changelog, state, issued, updated):
-            self.update_detail(package_id, update, obsolete, vendor_url, bz_url, cve_url,
-                    reboot, desc, changelog="", state= state, issued="", updated = "")
+        for package_id in package_ids:
+            log.info(package_id)
+            name, version,arch,summary  = pkpackage.get_package_from_id(package_id)
+            pkgDict = self.xmlcache.resolve(name)
+            update = ""
+            obsolete = ""
+            cve_url = ""
+            if pkgDict:
+                vendor_url = pkgDict.get("url","")
+                desc = pkgDict.get("longDesc","")
+                reboot = self._get_restart(pkgDict.get("name"))
+                state = self._get_branch( pkgDict.get("label"))
+                bz_url = self._get_fits(pkgDict.get("label"), pkgDict.get("name"))
+                self.update_detail(package_id, update, obsolete, vendor_url, bz_url, cve_url,
+                        reboot, desc, changelog="", state= state, issued="", updated = "")
 
    # @ExceptionHandler
     def get_details(self, package_ids):
@@ -744,42 +743,24 @@ class PackageKitConaryBackend(PackageKitBaseBackend):
         self.status(STATUS_INFO)
 
         log.info("========== get_details =============")
-        log.info(package_ids[0])
-        package_id = package_ids[0]
-        #name, version, flavor, installed = self._findPackage(package_id)
-        
-        summary = package_id.split(";")
-        log.info("====== geting summary")
-        log.info(summary)
-        name,version,arch,data = pkpackage.get_package_from_id(package_id)
-        cache = Cache()
-        pkgDict = cache.resolve(name)
-        
-        if name and pkgDict:
-            shortDesc = ""
-            longDesc = ""
-            url = ""
-            categories  = None
-            license = ""
+        for package_id in package_ids:
+            name,version,arch,data = pkpackage.get_package_from_id(package_id)
+            pkgDict = self.xmlcache.resolve(name)
+            if name and pkgDict:
+                shortDesc = ""
+                longDesc = ""
+                url = ""
+                categories  = None
+                license = ""
 
-            if "shortDesc" in pkgDict:
-                shortDesc = pkgDict["shortDesc"] 
-            if "longDesc" in pkgDict:
-                longDesc = pkgDict["longDesc"]
-            if "url" in pkgDict:
-                url = pkgDict["url"] #+ ";%s" % pkgDict["url"].replace("http://","")
-            if "category" in pkgDict:
-                categories =  Cache().getGroup( pkgDict['category'])
-            if "licenses" in pkgDict:
-                license = self._get_license(pkgDict["licenses"])
-                log.info(license)
-            # Package size goes here, but I don't know how to find that for conary packages.
-            #
-            #LICENSE_UNKNOWN = "unknown"
-            pkg_id = package_id.split(";")
-            pkg_id[3] = pkgDict["label"]
-            package_id = ";".join(pkg_id)
-            self.details(package_id, license, categories, longDesc, url, 0)
+                shortDesc = pkgDict.get("shortDesc", "")
+                longDesc = pkgDict.get("longDesc", "")
+                url = pkgDict.get("url", "")
+                categories = self.xmlcache.getGroup(pkgDict.get("category",""))
+                license = self._get_license(pkgDict.get("licenses",""))
+                size = pkgDict.get("size", 0)
+                log.info("Details: %s, %s, %s, %s, %s, %d" % (package_id, license, categories, longDesc, url, size))
+                self.details(package_id, license, categories, longDesc, url, size)
 
     def _show_package(self, name, version, flavor, status):
         '''  Show info about package'''
@@ -865,13 +846,12 @@ class PackageKitConaryBackend(PackageKitBaseBackend):
         log.info("_get_update ....")
 
         self.status(STATUS_RUNNING)
-        updJob, suggMap = self._get_update(applyList)
+        updJob = self._get_update(applyList)
         log.info("_get_update ....end.")
 
-        jobLists = updJob.getJobs()
         log.info("getting JobLists...........")
         r = []
-        for num, job in enumerate(jobLists):
+        for num, job in enumerate(updJob.getJobs()):
             name = job[0][0]
 
             # On an erase display the old version/flavor information.
@@ -882,13 +862,12 @@ class PackageKitConaryBackend(PackageKitBaseBackend):
             flavor = job[0][2][1]
             if flavor is None:
                 flavor = job[0][1][1]
-            
+
             info = self._get_info(name)
             trove_info = ( ( name,version,flavor ), info) 
             r.append(trove_info)
-            #self._show_package(name, version, flavor, info)
-        
-        pkg_list = Cache().resolve_list([ name for (  ( name,version,flavor), info )  in r ])
+
+        pkg_list = self.xmlcache.resolve_list([ name for (  ( name,version,flavor), info )  in r ])
         log.info("generate the pkgs ")
         new_res = []
         for pkg in pkg_list:
@@ -947,6 +926,24 @@ class PackageKitConaryBackend(PackageKitBaseBackend):
         '''
         pass
 
+    def simulate_install_packages(self, package_ids):
+	'''
+	Simulate an install of one or more packages.
+        '''
+	return self.install_packages(False, package_ids, simulate=True)
+
+    def simulate_update_packages(self, package_ids):
+	'''
+	Simulate an update of one or more packages.
+        '''
+	return self.update_packages(False, package_ids, simulate=True)
+
+    def simulate_remove_packages(self, package_ids):
+	'''
+	Simulate an update of one or more packages.
+        '''
+	return self.remove_packages(False, False, package_ids, simulate=True)
+
 from pkConaryLog import pdb
 
 def main():
