@@ -27,10 +27,9 @@
 #include <pk-backend.h>
 #include <egg-debug.h>
 
+#include <stdio.h>
 #include <libopkg/opkg.h>
 
-static opkg_t *opkg;
-
 enum {
 	SEARCH_NAME,
 	SEARCH_DESCRIPTION,
@@ -62,7 +61,7 @@ opkg_unknown_error (PkBackend *backend, gint error_code, const gchar *failed_cmd
  * check an opkg package for known GUI dependancies
  */
 static gboolean
-opkg_is_gui_pkg (opkg_package_t *pkg)
+opkg_is_gui_pkg (pkg_t *pkg)
 {
 
   /* TODO: check appropriate tag */
@@ -84,7 +83,7 @@ opkg_is_gui_pkg (opkg_package_t *pkg)
  * check an opkg package to determine if it is a development package
  */
 static gboolean
-opkg_is_devel_pkg (opkg_package_t *pkg)
+opkg_is_devel_pkg (pkg_t *pkg)
 {
   if (g_strrstr (pkg->name, "-dev"))
       return TRUE;
@@ -105,7 +104,7 @@ opkg_is_devel_pkg (opkg_package_t *pkg)
  * returns true if the tag is present
  */
 static gboolean 
-opkg_check_tag (opkg_package_t *pkg, const gchar *tag)
+opkg_check_tag (pkg_t *pkg, const gchar *tag)
 {
 	if (pkg->tags && tag)
 		return (g_strrstr (pkg->tags, tag) != NULL);
@@ -118,7 +117,7 @@ handle_install_error (PkBackend *backend, int err)
 {
 	switch (err)
 	{
-	case OPKG_NO_ERROR:
+/*	case OPKG_NO_ERROR:
 		break;
 	case OPKG_PACKAGE_NOT_INSTALLED:
 		pk_backend_error_code (backend, PK_ERROR_ENUM_PACKAGE_NOT_INSTALLED, NULL);
@@ -140,7 +139,7 @@ handle_install_error (PkBackend *backend, int err)
 		break;
 	case OPKG_PACKAGE_NOT_AVAILABLE:
 		pk_backend_error_code (backend, PK_ERROR_ENUM_PACKAGE_NOT_FOUND, NULL);
-		break;
+		break;*/
 	default:
 		opkg_unknown_error (backend, err, "Update package");
 	}
@@ -152,7 +151,7 @@ handle_install_error (PkBackend *backend, int err)
 static void
 backend_initialize (PkBackend *backend)
 {
-	opkg = opkg_new ();
+	int opkg = opkg_new ();
 
 	if (!opkg) {
 		pk_backend_error_code (backend,
@@ -162,8 +161,8 @@ backend_initialize (PkBackend *backend)
 	}
 
 #ifdef OPKG_OFFLINE_ROOT
-	opkg_set_option (opkg, (char *) "offline_root", OPKG_OFFLINE_ROOT);
-	opkg_re_read_config_files (opkg);
+	opkg_set_option ((char *) "offline_root", OPKG_OFFLINE_ROOT);
+	opkg_re_read_config_files ();
 #endif
 
 }
@@ -174,22 +173,22 @@ backend_initialize (PkBackend *backend)
 static void
 backend_destroy (PkBackend *backend)
 {
-	opkg_free (opkg);
+	opkg_free ();
 }
 
 
 static void
-pk_opkg_progress_cb (opkg_t *_opkg, const opkg_progress_data_t *pdata, void *data)
+pk_opkg_progress_cb (const opkg_progress_data_t *pdata, void *data)
 {
-	PkBackend *backend = PK_BACKEND (data);
+	PkBackend *backend = (PkBackend*) data;
 	if (!backend)
 		return;
 
 	pk_backend_set_percentage (backend, pdata->percentage);
-	if (pdata->package)
+	if (pdata->pkg)
 	{
 		gchar *uid;
-		opkg_package_t *pkg = pdata->package;
+		pkg_t *pkg = pdata->pkg;
 		gint status = PK_INFO_ENUM_UNKNOWN;
 
 		uid = g_strdup_printf ("%s;%s;%s;",
@@ -225,12 +224,12 @@ backend_refresh_cache_thread (PkBackend *backend)
 {
 	int ret;
 
-	ret = opkg_update_package_lists (opkg, pk_opkg_progress_cb, backend);
+	ret = opkg_update_package_lists (pk_opkg_progress_cb, backend);
 
 	if (ret) {
-		if (ret == OPKG_DOWNLOAD_FAILED)
-			pk_backend_error_code (backend, PK_ERROR_ENUM_REPO_NOT_AVAILABLE, NULL);
-		else
+//		if (ret == OPKG_DOWNLOAD_FAILED)
+//			pk_backend_error_code (backend, PK_ERROR_ENUM_REPO_NOT_AVAILABLE, NULL);
+//		else
 			opkg_unknown_error (backend, ret, "Refreshing cache");
 	}
 	pk_backend_finished (backend);
@@ -256,7 +255,7 @@ backend_refresh_cache (PkBackend *backend, gboolean force)
  */
 
 static void
-pk_opkg_package_list_cb (opkg_t *_opkg, opkg_package_t *pkg, void *data)
+pk_opkg_package_list_cb (pkg_t *pkg, void *data)
 {
 	SearchParams *params = (SearchParams*) data;
 	gchar *uid;
@@ -298,7 +297,7 @@ pk_opkg_package_list_cb (opkg_t *_opkg, opkg_package_t *pkg, void *data)
 	uid = g_strdup_printf ("%s;%s;%s;",
 		pkg->name, pkg->version, pkg->architecture);
 
-	if (pkg->installed)
+	if (pkg->state_status == SS_INSTALLED)
 		status = PK_INFO_ENUM_INSTALLED;
 	else
 		status = PK_INFO_ENUM_AVAILABLE;
@@ -318,10 +317,10 @@ pk_opkg_package_list_cb (opkg_t *_opkg, opkg_package_t *pkg, void *data)
                 opkg_is_gui_pkg (pkg))
 		goto end_handle;
 	if (pk_bitfield_contain(filters, PK_FILTER_ENUM_INSTALLED) && 
-                (!pkg->installed))
+                (pkg->state_status != SS_INSTALLED))
 		goto end_handle;
 	if (pk_bitfield_contain(filters, PK_FILTER_ENUM_NOT_INSTALLED) && 
-                (pkg->installed))
+                (pkg->state_status == SS_INSTALLED))
 		goto end_handle;
 
 	pk_backend_package (params->backend, status, uid, pkg->description);
@@ -338,7 +337,7 @@ backend_search_thread (PkBackend *backend)
 
 	params = pk_backend_get_pointer (backend, "search-params");
 
-	opkg_list_packages (opkg, pk_opkg_package_list_cb, params);
+	opkg_list_packages (pk_opkg_package_list_cb, params);
 
 	pk_backend_finished (params->backend);
 
@@ -349,7 +348,7 @@ backend_search_thread (PkBackend *backend)
 }
 
 static void
-backend_search_name (PkBackend *backend, PkBitfield filters, const gchar *search)
+backend_search_name (PkBackend *backend, PkBitfield filters, gchar **search)
 {
 	SearchParams *params;
 
@@ -360,7 +359,7 @@ backend_search_name (PkBackend *backend, PkBitfield filters, const gchar *search
 	params = g_new0 (SearchParams, 1);
 	params->filters = filters;
 	params->search_type = SEARCH_NAME;
-	params->needle = g_utf8_strdown (search, -1);
+	params->needle = g_utf8_strdown (search[0], -1);
 	params->backend = backend;
 
 	pk_backend_set_pointer (backend, "search-params", params);
@@ -371,7 +370,7 @@ backend_search_name (PkBackend *backend, PkBitfield filters, const gchar *search
  * backend_search_description:
  */
 static void
-backend_search_description (PkBackend *backend, PkBitfield filters, const gchar *search)
+backend_search_description (PkBackend *backend, PkBitfield filters, gchar **search)
 {
 	SearchParams *params;
 
@@ -382,7 +381,7 @@ backend_search_description (PkBackend *backend, PkBitfield filters, const gchar
 	params = g_new0 (SearchParams, 1);
 	params->filters = filters;
 	params->search_type = SEARCH_DESCRIPTION;
-	params->needle = g_utf8_strdown (search, -1);
+	params->needle = g_utf8_strdown (search[0], -1);
 	params->backend = backend;
 
 	pk_backend_set_pointer (backend, "search-params", params);
@@ -390,7 +389,7 @@ backend_search_description (PkBackend *backend, PkBitfield filters, const gchar
 }
 
 static void
-backend_search_group (PkBackend *backend, PkBitfield filters, const gchar *search)
+backend_search_group (PkBackend *backend, PkBitfield filters, gchar **search)
 {
 	SearchParams *params;
 
@@ -401,7 +400,7 @@ backend_search_group (PkBackend *backend, PkBitfield filters, const gchar *searc
 	params = g_new0 (SearchParams, 1);
 	params->filters = filters;
 	params->search_type = SEARCH_TAG;
-	params->needle = g_strdup_printf ("group::%s", search);
+	params->needle = g_strdup_printf ("group::%s", search[0]);
 	params->backend = backend;
 
 	pk_backend_set_pointer (backend, "search-params", params);
@@ -412,9 +411,9 @@ backend_search_group (PkBackend *backend, PkBitfield filters, const gchar *searc
 static gboolean
 backend_install_packages_thread (PkBackend *backend)
 {
-	PkPackageId *pi;
 	gint err, i;
 	gchar **package_ids;
+	gchar **parts;
 
 	package_ids = pk_backend_get_strv (backend, "pkids");
 
@@ -424,13 +423,13 @@ backend_install_packages_thread (PkBackend *backend)
 	{
 		pk_backend_package (backend, PK_INFO_ENUM_INSTALLING, package_ids[i], NULL);
 
-		pi = pk_package_id_new_from_string (package_ids[i]);
+		parts = pk_package_id_split (package_ids[i]);
 
-		err = opkg_install_package (opkg, pi->name, pk_opkg_progress_cb, backend);
+		err = opkg_install_package (parts[PK_PACKAGE_ID_NAME], pk_opkg_progress_cb, backend);
 		if (err)
 			handle_install_error (backend, err);
 
-		pk_package_id_free (pi);
+		g_strfreev (parts);
 		if (err != 0)
 			break;
 	}
@@ -453,9 +452,9 @@ backend_install_packages (PkBackend *backend, gboolean only_trusted, gchar **pac
 static gboolean
 backend_remove_packages_thread (PkBackend *backend)
 {
-	PkPackageId *pi;
 	gint err, i;
 	gchar **package_ids;
+	gchar **parts;
 	gboolean allow_deps;
 	gboolean autoremove;
 	gpointer *data;
@@ -467,29 +466,30 @@ backend_remove_packages_thread (PkBackend *backend)
 	autoremove = GPOINTER_TO_INT (data[2]);
 	g_free (data);
 
-	opkg_set_option (opkg, (char *)"autoremove", &autoremove);
-	opkg_set_option (opkg, (char *)"force_removal_of_dependent_packages", &allow_deps);
+	opkg_set_option ((char *)"autoremove", &autoremove);
+	opkg_set_option ((char *)"force_removal_of_dependent_packages", &allow_deps);
 
 	err = 0;
 
 	for (i = 0; package_ids[i]; i++)
 	{
-		pi = pk_package_id_new_from_string (package_ids[i]);
 		pk_backend_package (backend, PK_INFO_ENUM_REMOVING, package_ids[i], NULL);
 
-		err = opkg_remove_package (opkg, pi->name, pk_opkg_progress_cb, backend);
+		parts = pk_package_id_split (package_ids[i]);
+
+		err = opkg_remove_package (parts[PK_PACKAGE_ID_NAME], pk_opkg_progress_cb, backend);
 
 		switch (err)
 		{
-		case OPKG_NO_ERROR:
-			break;
-		case OPKG_PACKAGE_NOT_INSTALLED:
-			pk_backend_error_code (backend, PK_ERROR_ENUM_PACKAGE_NOT_INSTALLED, NULL);
-			break;
+		//case OPKG_NO_ERROR:
+		//	break;
+		//case OPKG_PACKAGE_NOT_INSTALLED:
+		//	pk_backend_error_code (backend, PK_ERROR_ENUM_PACKAGE_NOT_INSTALLED, NULL);
+		//	break;
 		default:
 			opkg_unknown_error (backend, err, "Remove");
 		}
-		pk_package_id_free (pi);
+		g_strfreev (parts);
 
 		if (err != 0)
 			break;
@@ -540,7 +540,7 @@ backend_update_system_thread (PkBackend *backend)
 	gint err;
 
 	/* FIXME: support only_trusted */
-	err = opkg_upgrade_all (opkg, pk_opkg_progress_cb, backend);
+	err = opkg_upgrade_all (pk_opkg_progress_cb, backend);
 
 	if (err)
 		opkg_unknown_error (backend, err, "Upgrading system");
@@ -564,29 +564,28 @@ backend_update_system (PkBackend *backend, gboolean only_trusted)
 static gboolean
 backend_update_package_thread (PkBackend *backend)
 {
-	PkPackageId *pi;
+        gchar **parts;
 	gint err = 0;
 	const gchar *package_id;
 
 	/* FIXME: support only_trusted */
 	package_id = pk_backend_get_string (backend, "pkgid");
-	pi = pk_package_id_new_from_string (package_id);
+	parts = pk_package_id_split (package_id);
 
-	if (!pi->name || !pi->version)
+	if (!parts)
 	{
 		pk_backend_error_code (backend, PK_ERROR_ENUM_PACKAGE_NOT_FOUND,
 				"Package not found");
-		pk_package_id_free (pi);
 		pk_backend_finished (backend);
 		return FALSE;
 	}
 
-	err = opkg_upgrade_package (opkg, pi->name, pk_opkg_progress_cb, backend);
+	err = opkg_upgrade_package (parts[PK_PACKAGE_ID_NAME], pk_opkg_progress_cb, backend);
 	if (err)
 		handle_install_error (backend, err);
 
 
-	pk_package_id_free (pi);
+	g_strfreev (parts);
 	pk_backend_finished (backend);
 	return (err != 0);
 }
@@ -610,13 +609,13 @@ backend_update_packages (PkBackend *backend, gboolean only_trusted, gchar **pack
  */
 
 static void
-pk_opkg_list_upgradable_cb (opkg_t *_opkg, opkg_package_t *pkg, void *data)
+pk_opkg_list_upgradable_cb (pkg_t *pkg, void *data)
 {
-	PkBackend *backend = PK_BACKEND (data);
+	PkBackend *backend = (PkBackend*) data;
 	gchar *uid;
 	gint status;
 
-	if (pkg->installed)
+	if (pkg->state_status == SS_INSTALLED)
 		status = PK_INFO_ENUM_INSTALLED;
 	else
 		status = PK_INFO_ENUM_AVAILABLE;
@@ -631,7 +630,7 @@ pk_opkg_list_upgradable_cb (opkg_t *_opkg, opkg_package_t *pkg, void *data)
 static gboolean
 backend_get_updates_thread (PkBackend *backend)
 {
-	opkg_list_upgradable_packages (opkg, pk_opkg_list_upgradable_cb, backend);
+	opkg_list_upgradable_packages (pk_opkg_list_upgradable_cb, backend);
 	pk_backend_finished (backend);
 	return TRUE;
 }
@@ -639,7 +638,7 @@ backend_get_updates_thread (PkBackend *backend)
 static void
 backend_get_updates (PkBackend *backend, PkBitfield filters)
 {
-	pk_backend_set_status (backend, PK_STATUS_ENUM_UPDATE);
+	pk_backend_set_status (backend, PK_STATUS_ENUM_QUERY);
 	pk_backend_set_percentage (backend, PK_BACKEND_PERCENTAGE_INVALID);
 
 	pk_backend_thread_create (backend, backend_get_updates_thread);
@@ -668,16 +667,18 @@ backend_get_groups (PkBackend *backend)
 static gboolean
 backend_get_details_thread (PkBackend *backend)
 {
-	PkPackageId *pi;
 	gchar **package_ids;
+        gchar **parts;
 	int group_index;
 	PkGroupEnum group = 0;
-	opkg_package_t *pkg;
+	pkg_t *pkg;
 	gchar *newid;
 
         package_ids = pk_backend_get_strv(backend, "package_ids");
-	pi = pk_package_id_new_from_string (package_ids[0]);
-	if (pi == NULL)
+	parts = pk_package_id_split (package_ids[0]);
+
+
+	if (!parts)
 	{
 		pk_backend_error_code (backend, PK_ERROR_ENUM_PACKAGE_ID_INVALID, "invalid package id");
 		pk_backend_finished (backend);
@@ -685,8 +686,8 @@ backend_get_details_thread (PkBackend *backend)
 	}
 
 
-	pkg = opkg_find_package (opkg, pi->name, pi->version, pi->arch, pi->data);
-	pk_package_id_free (pi);
+	pkg = opkg_find_package (parts[PK_PACKAGE_ID_NAME], parts[PK_PACKAGE_ID_VERSION], parts[PK_PACKAGE_ID_ARCH], parts[PK_PACKAGE_ID_DATA]);
+	g_strfreev (parts);
 
 	if (!pkg)
 	{
@@ -695,20 +696,19 @@ backend_get_details_thread (PkBackend *backend)
 		return FALSE;
 	}
 
-	newid = g_strdup_printf ("%s;%s;%s;%s", pkg->name, pkg->version, pkg->architecture, pkg->repository);
+	newid = g_strdup_printf ("%s;%s;%s;%s", pkg->name, pkg->version, pkg->architecture, pkg->src->name);
 
 	if (pkg->tags) {
 		for (group_index = 0; group < PK_GROUP_ENUM_LAST; group_index++) {
 			group = 1 << group_index;
 			if (!(group & backend_get_groups(backend))) continue;
-			if (opkg_check_tag(pkg, (const gchar *)pk_group_enum_to_text(group))) 
+			if (opkg_check_tag(pkg, (const gchar *)pk_group_enum_to_string(group))) 
 				break;
 		}
 	}
 
-	pk_backend_details (backend, newid, NULL, group, pkg->description, pkg->url, pkg->size);
+	pk_backend_details (backend, newid, NULL, group, pkg->description, NULL, pkg->size);
 	g_free (newid);
-	opkg_package_free(pkg);
 	pk_backend_finished (backend);
 	return TRUE;
 }
