@@ -28,6 +28,8 @@ from packagekit.package import PackagekitPackage
 from packagekit.enums import *
 import re
 import sys
+import codecs
+import locale
 
 # TODO: move Groups to a separate class (including the lookup table)
 # TODO: move Filter to a separate class (and use "PackagekitFilter")
@@ -36,6 +38,11 @@ import sys
 pkprogress = PackagekitProgress()
 pkpackage = PackagekitPackage()
 
+try:
+    ENCODING = locale.getpreferredencoding()
+except locale.Error:
+    ENCODING = "ascii"
+
 def needs_cache(func):
     """ Load smart's channels, and save the cache when done. """
     def cache_wrap(obj, *args, **kwargs):
@@ -148,6 +155,9 @@ class PackageKitSmartBackend(PackageKitBaseBackend):
         PackageKitBaseBackend.__init__(self, args)
         self._cacheloaded = False
 
+        writer = codecs.getwriter(ENCODING)
+        sys.stdout = writer(sys.stdout, errors="replace")
+
         self.ctrl = smart.init()
         smart.iface.object = PackageKitSmartInterface(self.ctrl, self)
         smart.initPlugins()
@@ -461,22 +471,23 @@ class PackageKitSmartBackend(PackageKitBaseBackend):
         self._show_package_list()
 
     @needs_cache
-    def search_name(self, filters, packagename):
-        globbed = "*%s*" % packagename
-        self.status(STATUS_QUERY)
-        self.allow_cancel(True)
-        ratio, results, suggestions = self.ctrl.search(globbed)
+    def search_name(self, filters, packagenames):
+        for packagename in packagenames:
+            globbed = "*%s*" % packagename
+            self.status(STATUS_QUERY)
+            self.allow_cancel(True)
+            ratio, results, suggestions = self.ctrl.search(globbed)
 
-        packages = self._process_search_results(results)
+            packages = self._process_search_results(results)
 
-        for package in packages:
-            if self._package_passes_filters(package, filters):
-                self._add_package(package)
+            for package in packages:
+                if self._package_passes_filters(package, filters):
+                    self._add_package(package)
         self._post_process_package_list(filters)
         self._show_package_list()
 
     @needs_cache
-    def search_file(self, filters, searchstring):
+    def search_file(self, filters, searchstrings):
         self.status(STATUS_QUERY)
         self.allow_cancel(True)
         packages = self.ctrl.getCache().getPackages()
@@ -490,36 +501,43 @@ class PackageKitSmartBackend(PackageKitBaseBackend):
                     paths = info.getPathList()
                     if len(paths) > 0:
                         break
-                if searchstring in paths:
-                    self._add_package(package)
+                for searchstring in searchstrings:
+                    if searchstring in paths:
+                        self._add_package(package)
         self._post_process_package_list(filters)
         self._show_package_list()
 
     @needs_cache
-    def search_group(self, filters, searchstring):
+    def search_group(self, filters, searchstrings):
         self.status(STATUS_QUERY)
         self.allow_cancel(True)
-        filter_desktops = searchstring.find("desktop") != -1
+        filter_desktops = False
+        for searchstring in searchstrings:
+            if searchstring.find("desktop") != -1:
+                filter_desktops = True
         packages = self.ctrl.getCache().getPackages()
         for package in packages:
             if self._package_passes_filters(package, filters):
                 info = package.loaders.keys()[0].getInfo(package)
                 group = self._get_group(info, filter_desktops)
-                if searchstring in group:
-                    self._add_package(package)
+                for searchstring in searchstrings:
+                    if searchstring in group:
+                        self._add_package(package)
         self._post_process_package_list(filters)
         self._show_package_list()
 
     @needs_cache
-    def search_details(self, filters, searchstring):
+    def search_details(self, filters, searchstrings):
         self.status(STATUS_QUERY)
         self.allow_cancel(True)
         packages = self.ctrl.getCache().getPackages()
         for package in packages:
             if self._package_passes_filters(package, filters):
                 info = package.loaders.keys()[0].getInfo(package)
-                if searchstring in info.getDescription():
-                    self._add_package(package)
+                desc = info.getDescription()
+                for searchstring in searchstrings:
+                    if searchstring in desc:
+                        self._add_package(package)
 
     @needs_cache
     def get_packages(self, filters):
@@ -889,7 +907,8 @@ class PackageKitSmartBackend(PackageKitBaseBackend):
             name = channel.get("name", alias)
             parsed = smart.channel.parseChannelData(channel)
             enabled = True
-            if channel.has_key('disabled') and channel['disabled'] == 'yes':
+            if channel.has_key('disabled') and (channel['disabled'] == 'yes'
+                                            or channel['disabled'] == True):
                 enabled = False
             channel['alias'] = alias
             if self._channel_passes_filters(channel, filters):
@@ -1157,8 +1176,7 @@ class PackageKitSmartBackend(PackageKitBaseBackend):
 
         return packages
 
-    def _channel_passes_filters(self, channel, filters):
-        filterlist = filters.split(';')
+    def _channel_passes_filters(self, channel, filterlist):
         if FILTER_NOT_DEVELOPMENT in filterlist:
             if channel['type'] == 'rpm-md':
                 repo = channel['alias']
@@ -1203,8 +1221,7 @@ class PackageKitSmartBackend(PackageKitBaseBackend):
         else:
             return None
 
-    def _package_passes_filters(self, package, filters):
-        filterlist = filters.split(';')
+    def _package_passes_filters(self, package, filterlist):
         if FILTER_NOT_INSTALLED in filterlist and package.installed:
             return False
         elif FILTER_INSTALLED in filterlist and not package.installed:
@@ -1303,8 +1320,7 @@ class PackageKitSmartBackend(PackageKitBaseBackend):
             newest[key] = (package, status)
         return newest.values()
 
-    def _post_process_package_list(self, filters):
-        filterlist = filters.split(';')
+    def _post_process_package_list(self, filterlist):
         if FILTER_BASENAME in filterlist:
             self._package_list = self._do_basename_filtering(self._package_list)
         if FILTER_NEWEST in filterlist:
