@@ -38,6 +38,7 @@
 #define PK_EXIT_CODE_SYNTAX_INVALID	3
 #define PK_EXIT_CODE_FILE_NOT_FOUND	4
 #define PK_EXIT_CODE_NOTHING_USEFUL	5
+#define PK_EXIT_CODE_CANNOT_SETUP	6
 
 static GMainLoop *loop = NULL;
 static PkBitfield roles;
@@ -170,7 +171,7 @@ pk_console_transaction_cb (PkTransactionPast *item, gpointer user_data)
 		      "data", &data,
 		      NULL);
 
-	role_text = pk_role_enum_to_text (role);
+	role_text = pk_role_enum_to_string (role);
 	/* TRANSLATORS: this is an atomic transaction */
 	g_print ("%s: %s\n", _("Transaction"), tid);
 	/* TRANSLATORS: this is the time the transaction was started in system timezone */
@@ -247,7 +248,7 @@ pk_console_distro_upgrade_cb (PkDistroUpgrade *item, gpointer user_data)
 	/* TRANSLATORS: this is the distro, e.g. Fedora 10 */
 	g_print ("%s: %s\n", _("Distribution"), name);
 	/* TRANSLATORS: this is type of update, stable or testing */
-	g_print (" %s: %s\n", _("Type"), pk_update_state_enum_to_text (state));
+	g_print (" %s: %s\n", _("Type"), pk_update_state_enum_to_string (state));
 	/* TRANSLATORS: this is any summary text describing the upgrade */
 	g_print (" %s: %s\n", _("Summary"), summary);
 
@@ -366,7 +367,7 @@ pk_console_update_detail_cb (PkUpdateDetail *item, gpointer data)
 	}
 	if (restart != PK_RESTART_ENUM_NONE) {
 		/* TRANSLATORS: details about the update, if the package requires a restart */
-		g_print (" %s: %s\n", _("Restart"), pk_restart_enum_to_text (restart));
+		g_print (" %s: %s\n", _("Restart"), pk_restart_enum_to_string (restart));
 	}
 	if (update_text != NULL) {
 		/* TRANSLATORS: details about the update, any description of the update */
@@ -378,7 +379,7 @@ pk_console_update_detail_cb (PkUpdateDetail *item, gpointer data)
 	}
 	if (state != PK_UPDATE_STATE_ENUM_UNKNOWN) {
 		/* TRANSLATORS: details about the update, the ongoing state of the update */
-		g_print (" %s: %s\n", _("State"), pk_update_state_enum_to_text (state));
+		g_print (" %s: %s\n", _("State"), pk_update_state_enum_to_string (state));
 	}
 	if (issued != NULL) {
 		/* TRANSLATORS: details about the update, date the update was issued */
@@ -507,7 +508,7 @@ pk_console_details_cb (PkDetails *item, gpointer data)
 	g_print ("%s\n", _("Package description"));
 	g_print ("  package:     %s\n", package);
 	g_print ("  license:     %s\n", license);
-	g_print ("  group:       %s\n", pk_group_enum_to_text (group));
+	g_print ("  group:       %s\n", pk_group_enum_to_string (group));
 	g_print ("  description: %s\n", description);
 	g_print ("  size:        %lu bytes\n", (long unsigned int) size);
 	g_print ("  url:         %s\n", url);
@@ -535,7 +536,7 @@ pk_console_message_cb (PkMessage *item, gpointer data)
 		      NULL);
 
 	/* TRANSLATORS: This a message (like a little note that may be of interest) from the transaction */
-	g_print ("%s %s: %s\n", _("Message:"), pk_message_enum_to_text (type), details);
+	g_print ("%s %s: %s\n", _("Message:"), pk_message_enum_to_string (type), details);
 	g_free (details);
 }
 
@@ -667,6 +668,7 @@ pk_console_finished_cb (GObject *object, GAsyncResult *res, gpointer data)
 	PkResults *results;
 	GError *error = NULL;
 	GPtrArray *array;
+	PkPackageSack *sack;
 	PkRestartEnum restart;
 	PkRoleEnum role;
 
@@ -679,7 +681,7 @@ pk_console_finished_cb (GObject *object, GAsyncResult *res, gpointer data)
 	}
 
 	/* get the results */
-	results = pk_client_generic_finish (PK_CLIENT(task), res, &error);
+	results = pk_task_generic_finish (PK_TASK(task), res, &error);
 	if (results == NULL) {
 		/* TRANSLATORS: we failed to get any results, which is pretty fatal in my book */
 		g_print ("%s: %s\n", _("Fatal error"), error->message);
@@ -691,7 +693,7 @@ pk_console_finished_cb (GObject *object, GAsyncResult *res, gpointer data)
 	error_code = pk_results_get_error_code (results);
 	if (error_code != NULL) {
 		/* TRANSLATORS: the transaction failed in a way we could not expect */
-		g_print ("%s: %s, %s\n", _("The transaction failed"), pk_error_enum_to_text (pk_error_get_code (error_code)), pk_error_get_details (error_code));
+		g_print ("%s: %s, %s\n", _("The transaction failed"), pk_error_enum_to_string (pk_error_get_code (error_code)), pk_error_get_details (error_code));
 
 		/* special case */
 		if (pk_error_get_code (error_code) == PK_ERROR_ENUM_NO_PACKAGES_TO_UPDATE)
@@ -702,8 +704,12 @@ pk_console_finished_cb (GObject *object, GAsyncResult *res, gpointer data)
 	/* get the role */
 	g_object_get (G_OBJECT(results), "role", &role, NULL);
 
+	/* get the sack */
+	sack = pk_results_get_package_sack (results);
+	pk_package_sack_sort (sack, PK_PACKAGE_SACK_SORT_TYPE_NAME);
+	array = pk_package_sack_get_array (sack);
+
 	/* package */
-	array = pk_results_get_package_array (results);
 	if (!is_console ||
 	    (role != PK_ROLE_ENUM_INSTALL_PACKAGES &&
 	     role != PK_ROLE_ENUM_UPDATE_PACKAGES &&
@@ -723,6 +729,7 @@ pk_console_finished_cb (GObject *object, GAsyncResult *res, gpointer data)
 	}
 
 	g_ptr_array_unref (array);
+	g_object_unref (sack);
 
 	/* transaction */
 	array = pk_results_get_transaction_array (results);
@@ -739,8 +746,10 @@ pk_console_finished_cb (GObject *object, GAsyncResult *res, gpointer data)
 	g_ptr_array_foreach (array, (GFunc) pk_console_distro_upgrade_cb, NULL);
 
 	/* special case */
-	if (array->len == 0 && role == PK_ROLE_ENUM_GET_DISTRO_UPGRADES)
+	if (array->len == 0 && role == PK_ROLE_ENUM_GET_DISTRO_UPGRADES) {
+		g_print ("%s\n", _("There are no upgrades available at this time."));
 		retval = PK_EXIT_CODE_NOTHING_USEFUL;
+	}
 
 	g_ptr_array_unref (array);
 
@@ -831,8 +840,19 @@ static gboolean
 pk_console_install_packages (gchar **packages, GError **error)
 {
 	gboolean ret = TRUE;
-	gchar **package_ids;
+	gchar **package_ids = NULL;
 	GError *error_local = NULL;
+	guint i;
+
+	/* test to see if we've been given files, not packages */
+	for (i=0; packages[i] != NULL; i++) {
+		ret = !g_file_test (packages[i], G_FILE_TEST_EXISTS);
+		if (!ret) {
+			/* TRANSLATORS: The user used 'pkcon install dave.rpm' rather than 'pkcon install-local dave.rpm' */
+			*error = g_error_new (1, 0, _("Expected package name, actually got file. Try using 'pkcon install-local %s' instead."), packages[i]);
+			goto out;
+		}
+	}
 
 	package_ids = pk_console_resolve_packages (PK_CLIENT(task), pk_bitfield_value (PK_FILTER_ENUM_NOT_INSTALLED), packages, &error_local);
 	if (package_ids == NULL) {
@@ -890,7 +910,7 @@ pk_console_download_packages (gchar **packages, const gchar *directory, GError *
 	gchar **package_ids;
 	GError *error_local = NULL;
 
-	package_ids = pk_console_resolve_packages (PK_CLIENT(task), pk_bitfield_value (PK_FILTER_ENUM_NONE), packages, &error_local);
+	package_ids = pk_console_resolve_packages (PK_CLIENT(task), pk_bitfield_value (PK_FILTER_ENUM_NOT_INSTALLED), packages, &error_local);
 	if (package_ids == NULL) {
 		/* TRANSLATORS: There was an error getting the list of files for the package. The detailed error follows */
 		*error = g_error_new (1, 0, _("This tool could not find the package: %s"), error_local->message);
@@ -900,9 +920,9 @@ pk_console_download_packages (gchar **packages, const gchar *directory, GError *
 	}
 
 	/* do the async action */
-	pk_client_download_packages_async (PK_CLIENT(task), package_ids, directory, cancellable,
-				           (PkProgressCallback) pk_console_progress_cb, NULL,
-				           (GAsyncReadyCallback) pk_console_finished_cb, NULL);
+	pk_task_download_packages_async (PK_TASK (task),package_ids, directory, cancellable,
+				         (PkProgressCallback) pk_console_progress_cb, NULL,
+				         (GAsyncReadyCallback) pk_console_finished_cb, NULL);
 out:
 	g_strfreev (package_ids);
 	return ret;
@@ -956,9 +976,9 @@ pk_console_get_requires (PkBitfield filters, gchar **packages, GError **error)
 	}
 
 	/* do the async action */
-	pk_client_get_requires_async (PK_CLIENT(task), filters, package_ids, TRUE, cancellable,
-				      (PkProgressCallback) pk_console_progress_cb, NULL,
-				      (GAsyncReadyCallback) pk_console_finished_cb, NULL);
+	pk_task_get_requires_async (PK_TASK (task),filters, package_ids, TRUE, cancellable,
+				    (PkProgressCallback) pk_console_progress_cb, NULL,
+				    (GAsyncReadyCallback) pk_console_finished_cb, NULL);
 out:
 	g_strfreev (package_ids);
 	return ret;
@@ -984,9 +1004,9 @@ pk_console_get_depends (PkBitfield filters, gchar **packages, GError **error)
 	}
 
 	/* do the async action */
-	pk_client_get_depends_async (PK_CLIENT(task), filters, package_ids, FALSE, cancellable,
-				     (PkProgressCallback) pk_console_progress_cb, NULL,
-				     (GAsyncReadyCallback) pk_console_finished_cb, NULL);
+	pk_task_get_depends_async (PK_TASK (task),filters, package_ids, FALSE, cancellable,
+				   (PkProgressCallback) pk_console_progress_cb, NULL,
+				   (GAsyncReadyCallback) pk_console_finished_cb, NULL);
 out:
 	g_strfreev (package_ids);
 	return ret;
@@ -1012,9 +1032,9 @@ pk_console_get_details (gchar **packages, GError **error)
 	}
 
 	/* do the async action */
-	pk_client_get_details_async (PK_CLIENT(task), package_ids, cancellable,
-				     (PkProgressCallback) pk_console_progress_cb, NULL,
-				     (GAsyncReadyCallback) pk_console_finished_cb, NULL);
+	pk_task_get_details_async (PK_TASK (task),package_ids, cancellable,
+				   (PkProgressCallback) pk_console_progress_cb, NULL,
+				   (GAsyncReadyCallback) pk_console_finished_cb, NULL);
 out:
 	g_strfreev (package_ids);
 	return ret;
@@ -1040,9 +1060,9 @@ pk_console_get_files (gchar **packages, GError **error)
 	}
 
 	/* do the async action */
-	pk_client_get_files_async (PK_CLIENT(task), package_ids, cancellable,
-				   (PkProgressCallback) pk_console_progress_cb, NULL,
-				   (GAsyncReadyCallback) pk_console_finished_cb, NULL);
+	pk_task_get_files_async (PK_TASK (task),package_ids, cancellable,
+				 (PkProgressCallback) pk_console_progress_cb, NULL,
+				 (GAsyncReadyCallback) pk_console_finished_cb, NULL);
 out:
 	g_strfreev (package_ids);
 	return ret;
@@ -1068,9 +1088,9 @@ pk_console_get_update_detail (gchar **packages, GError **error)
 	}
 
 	/* do the async action */
-	pk_client_get_update_detail_async (PK_CLIENT(task), package_ids, cancellable,
-					   (PkProgressCallback) pk_console_progress_cb, NULL,
-					   (GAsyncReadyCallback) pk_console_finished_cb, NULL);
+	pk_task_get_update_detail_async (PK_TASK (task),package_ids, cancellable,
+					 (PkProgressCallback) pk_console_progress_cb, NULL,
+					 (GAsyncReadyCallback) pk_console_finished_cb, NULL);
 out:
 	g_strfreev (package_ids);
 	return ret;
@@ -1154,7 +1174,7 @@ pk_console_get_summary (void)
 	    pk_bitfield_contain (roles, PK_ROLE_ENUM_UPDATE_PACKAGES))
 		g_string_append_printf (string, "  %s\n", "update <package>");
 	if (pk_bitfield_contain (roles, PK_ROLE_ENUM_REFRESH_CACHE))
-		g_string_append_printf (string, "  %s\n", "refresh");
+		g_string_append_printf (string, "  %s\n", "refresh [--force]");
 	if (pk_bitfield_contain (roles, PK_ROLE_ENUM_RESOLVE))
 		g_string_append_printf (string, "  %s\n", "resolve [package]");
 	if (pk_bitfield_contain (roles, PK_ROLE_ENUM_GET_UPDATES))
@@ -1222,6 +1242,7 @@ main (int argc, char *argv[])
 {
 	gboolean ret;
 	GError *error = NULL;
+	GError *error_local = NULL;
 	gboolean background = FALSE;
 	gboolean noninteractive = FALSE;
 	gboolean plain = FALSE;
@@ -1229,8 +1250,11 @@ main (int argc, char *argv[])
 	GOptionContext *context;
 	gchar *options_help;
 	gchar *filter = NULL;
+	gchar *root = NULL;
 	gchar *summary = NULL;
 	const gchar *mode;
+	const gchar *http_proxy;
+	const gchar *ftp_proxy;
 	const gchar *value = NULL;
 	const gchar *details = NULL;
 	const gchar *parameter = NULL;
@@ -1245,6 +1269,9 @@ main (int argc, char *argv[])
 		{ "filter", '\0', 0, G_OPTION_ARG_STRING, &filter,
 			/* TRANSLATORS: command line argument, use a filter to narrow down results */
 			_("Set the filter, e.g. installed"), NULL},
+		{ "root", '\0', 0, G_OPTION_ARG_STRING, &root,
+			/* TRANSLATORS: command line argument, use a non-standard install prefix */
+			_("Set the install root, e.g. '/' or '/mnt/ltsp'"), NULL},
 		{ "nowait", 'n', 0, G_OPTION_ARG_NONE, &nowait,
 			/* TRANSLATORS: command line argument, work asynchronously */
 			_("Exit without waiting for actions to complete"), NULL},
@@ -1329,11 +1356,39 @@ main (int argc, char *argv[])
 	g_object_set (task,
 		      "background", background,
 		      "simulate", !noninteractive,
+		      "interactive", !noninteractive,
 		      NULL);
 
+	/* set the proxy */
+	http_proxy = g_getenv ("http_proxy");
+	ftp_proxy = g_getenv ("ftp_proxy");
+	if (http_proxy != NULL ||
+	    ftp_proxy != NULL) {
+		ret = pk_control_set_proxy (control, http_proxy, ftp_proxy, NULL, &error_local);
+		if (!ret) {
+			/* TRANSLATORS: The user specified an incorrect filter */
+			error = g_error_new (1, 0, "%s: %s", _("The proxy could not be set"), error_local->message);
+			g_error_free (error_local);
+			retval = PK_EXIT_CODE_CANNOT_SETUP;
+			goto out;
+		}
+	}
+
+	/* set the install root if set */
+	if (root != NULL) {
+		ret = pk_control_set_root (control, root, NULL, &error_local);
+		if (!ret) {
+			/* TRANSLATORS: The user specified an incorrect filter */
+			error = g_error_new (1, 0, "%s: %s", _("The install root could not be set"), error_local->message);
+			g_error_free (error_local);
+			retval = PK_EXIT_CODE_CANNOT_SETUP;
+			goto out;
+		}
+	}
+
 	/* check filter */
 	if (filter != NULL) {
-		filters = pk_filter_bitfield_from_text (filter);
+		filters = pk_filter_bitfield_from_string (filter);
 		if (filters == 0) {
 			/* TRANSLATORS: The user specified an incorrect filter */
 			error = g_error_new (1, 0, "%s: %s", _("The filter specified was invalid"), filter);
@@ -1367,9 +1422,9 @@ main (int argc, char *argv[])
 				goto out;
 			}
 			/* fire off an async request */
-			pk_client_search_names_async (PK_CLIENT(task), filters, argv+3, cancellable,
-						     (PkProgressCallback) pk_console_progress_cb, NULL,
-						     (GAsyncReadyCallback) pk_console_finished_cb, NULL);
+			pk_task_search_names_async (PK_TASK (task),filters, argv+3, cancellable,
+						    (PkProgressCallback) pk_console_progress_cb, NULL,
+						    (GAsyncReadyCallback) pk_console_finished_cb, NULL);
 
 		} else if (strcmp (value, "details") == 0) {
 			if (details == NULL) {
@@ -1379,9 +1434,9 @@ main (int argc, char *argv[])
 				goto out;
 			}
 			/* fire off an async request */
-			pk_client_search_details_async (PK_CLIENT(task), filters, argv+3, cancellable,
-						        (PkProgressCallback) pk_console_progress_cb, NULL,
-						        (GAsyncReadyCallback) pk_console_finished_cb, NULL);
+			pk_task_search_details_async (PK_TASK (task),filters, argv+3, cancellable,
+						      (PkProgressCallback) pk_console_progress_cb, NULL,
+						      (GAsyncReadyCallback) pk_console_finished_cb, NULL);
 
 		} else if (strcmp (value, "group") == 0) {
 			if (details == NULL) {
@@ -1391,9 +1446,9 @@ main (int argc, char *argv[])
 				goto out;
 			}
 			/* fire off an async request */
-			pk_client_search_groups_async (PK_CLIENT(task), filters, argv+3, cancellable,
-						      (PkProgressCallback) pk_console_progress_cb, NULL,
-						      (GAsyncReadyCallback) pk_console_finished_cb, NULL);
+			pk_task_search_groups_async (PK_TASK (task),filters, argv+3, cancellable,
+						     (PkProgressCallback) pk_console_progress_cb, NULL,
+						     (GAsyncReadyCallback) pk_console_finished_cb, NULL);
 
 		} else if (strcmp (value, "file") == 0) {
 			if (details == NULL) {
@@ -1403,9 +1458,9 @@ main (int argc, char *argv[])
 				goto out;
 			}
 			/* fire off an async request */
-			pk_client_search_files_async (PK_CLIENT(task), filters, argv+3, cancellable,
-						     (PkProgressCallback) pk_console_progress_cb, NULL,
-						     (GAsyncReadyCallback) pk_console_finished_cb, NULL);
+			pk_task_search_files_async (PK_TASK (task),filters, argv+3, cancellable,
+						    (PkProgressCallback) pk_console_progress_cb, NULL,
+						    (GAsyncReadyCallback) pk_console_finished_cb, NULL);
 		} else {
 			/* TRANSLATORS: the search type was provided, but invalid */
 			error = g_error_new (1, 0, "%s", _("Invalid search type"));
@@ -1486,9 +1541,9 @@ main (int argc, char *argv[])
 			retval = PK_EXIT_CODE_SYNTAX_INVALID;
 			goto out;
 		}
-		pk_client_rollback_async (PK_CLIENT(task), value, cancellable,
-					  (PkProgressCallback) pk_console_progress_cb, NULL,
-					  (GAsyncReadyCallback) pk_console_finished_cb, NULL);
+		pk_task_rollback_async (PK_TASK (task),value, cancellable,
+					(PkProgressCallback) pk_console_progress_cb, NULL,
+					(GAsyncReadyCallback) pk_console_finished_cb, NULL);
 
 	} else if (strcmp (mode, "update") == 0) {
 		if (value == NULL) {
@@ -1507,9 +1562,9 @@ main (int argc, char *argv[])
 			retval = PK_EXIT_CODE_SYNTAX_INVALID;
 			goto out;
 		}
-		pk_client_resolve_async (PK_CLIENT(task), filters, argv+2, cancellable,
-				         (PkProgressCallback) pk_console_progress_cb, NULL,
-					 (GAsyncReadyCallback) pk_console_finished_cb, NULL);
+		pk_task_resolve_async (PK_TASK (task),filters, argv+2, cancellable,
+				       (PkProgressCallback) pk_console_progress_cb, NULL,
+				       (GAsyncReadyCallback) pk_console_finished_cb, NULL);
 
 	} else if (strcmp (mode, "repo-enable") == 0) {
 		if (value == NULL) {
@@ -1518,9 +1573,9 @@ main (int argc, char *argv[])
 			retval = PK_EXIT_CODE_SYNTAX_INVALID;
 			goto out;
 		}
-		pk_client_repo_enable_async (PK_CLIENT(task), value, TRUE, cancellable,
-					     (PkProgressCallback) pk_console_progress_cb, NULL,
-					     (GAsyncReadyCallback) pk_console_finished_cb, NULL);
+		pk_task_repo_enable_async (PK_TASK (task),value, TRUE, cancellable,
+					   (PkProgressCallback) pk_console_progress_cb, NULL,
+					   (GAsyncReadyCallback) pk_console_finished_cb, NULL);
 
 	} else if (strcmp (mode, "repo-disable") == 0) {
 		if (value == NULL) {
@@ -1529,9 +1584,9 @@ main (int argc, char *argv[])
 			retval = PK_EXIT_CODE_SYNTAX_INVALID;
 			goto out;
 		}
-		pk_client_repo_enable_async (PK_CLIENT(task), value, FALSE, cancellable,
-					     (PkProgressCallback) pk_console_progress_cb, NULL,
-					     (GAsyncReadyCallback) pk_console_finished_cb, NULL);
+		pk_task_repo_enable_async (PK_TASK (task),value, FALSE, cancellable,
+					   (PkProgressCallback) pk_console_progress_cb, NULL,
+					   (GAsyncReadyCallback) pk_console_finished_cb, NULL);
 
 	} else if (strcmp (mode, "repo-set-data") == 0) {
 		if (value == NULL || details == NULL || parameter == NULL) {
@@ -1545,9 +1600,9 @@ main (int argc, char *argv[])
 					       (GAsyncReadyCallback) pk_console_finished_cb, NULL);
 
 	} else if (strcmp (mode, "repo-list") == 0) {
-		pk_client_get_repo_list_async (PK_CLIENT(task), filters, cancellable,
-					       (PkProgressCallback) pk_console_progress_cb, NULL,
-					       (GAsyncReadyCallback) pk_console_finished_cb, NULL);
+		pk_task_get_repo_list_async (PK_TASK (task),filters, cancellable,
+					     (PkProgressCallback) pk_console_progress_cb, NULL,
+					     (GAsyncReadyCallback) pk_console_finished_cb, NULL);
 
 	} else if (strcmp (mode, "get-time") == 0) {
 		PkRoleEnum role;
@@ -1557,7 +1612,7 @@ main (int argc, char *argv[])
 			retval = PK_EXIT_CODE_SYNTAX_INVALID;
 			goto out;
 		}
-		role = pk_role_enum_from_text (value);
+		role = pk_role_enum_from_string (value);
 		if (role == PK_ROLE_ENUM_UNKNOWN) {
 			/* TRANSLATORS: The user specified an invalid action */
 			error = g_error_new (1, 0, "%s", _("A correct role is required"));
@@ -1607,9 +1662,9 @@ main (int argc, char *argv[])
 			retval = PK_EXIT_CODE_SYNTAX_INVALID;
 			goto out;
 		}
-		pk_client_what_provides_async (PK_CLIENT(task), filters, PK_PROVIDES_ENUM_ANY, argv+2, cancellable,
-					       (PkProgressCallback) pk_console_progress_cb, NULL,
-					       (GAsyncReadyCallback) pk_console_finished_cb, NULL);
+		pk_task_what_provides_async (PK_TASK (task),filters, PK_PROVIDES_ENUM_ANY, argv+2, cancellable,
+					     (PkProgressCallback) pk_console_progress_cb, NULL,
+					     (GAsyncReadyCallback) pk_console_finished_cb, NULL);
 
 	} else if (strcmp (mode, "get-details") == 0) {
 		if (value == NULL) {
@@ -1630,22 +1685,22 @@ main (int argc, char *argv[])
 		nowait = !pk_console_get_files (argv+2, &error);
 
 	} else if (strcmp (mode, "get-updates") == 0) {
-		pk_client_get_updates_async (PK_CLIENT(task), filters, cancellable,
-					     (PkProgressCallback) pk_console_progress_cb, NULL,
-					     (GAsyncReadyCallback) pk_console_finished_cb, NULL);
+		pk_task_get_updates_async (PK_TASK (task),filters, cancellable,
+					   (PkProgressCallback) pk_console_progress_cb, NULL,
+					   (GAsyncReadyCallback) pk_console_finished_cb, NULL);
 
 	} else if (strcmp (mode, "get-categories") == 0) {
-		pk_client_get_categories_async (PK_CLIENT(task), cancellable,
-						(PkProgressCallback) pk_console_progress_cb, NULL,
-						(GAsyncReadyCallback) pk_console_finished_cb, NULL);
-
-	} else if (strcmp (mode, "get-packages") == 0) {
-		pk_client_get_packages_async (PK_CLIENT(task), filters, cancellable,
+		pk_task_get_categories_async (PK_TASK (task),cancellable,
 					      (PkProgressCallback) pk_console_progress_cb, NULL,
 					      (GAsyncReadyCallback) pk_console_finished_cb, NULL);
 
+	} else if (strcmp (mode, "get-packages") == 0) {
+		pk_task_get_packages_async (PK_TASK (task),filters, cancellable,
+					    (PkProgressCallback) pk_console_progress_cb, NULL,
+					    (GAsyncReadyCallback) pk_console_finished_cb, NULL);
+
 	} else if (strcmp (mode, "get-roles") == 0) {
-		text = pk_role_bitfield_to_text (roles);
+		text = pk_role_bitfield_to_string (roles);
 		g_strdelimit (text, ";", '\n');
 		g_print ("%s\n", text);
 		g_free (text);
@@ -1655,7 +1710,7 @@ main (int argc, char *argv[])
 		g_object_get (control,
 			      "filters", &filters,
 			      NULL);
-		text = pk_filter_bitfield_to_text (filters);
+		text = pk_filter_bitfield_to_string (filters);
 		g_strdelimit (text, ";", '\n');
 		g_print ("%s\n", text);
 		g_free (text);
@@ -1665,7 +1720,7 @@ main (int argc, char *argv[])
 		g_object_get (control,
 			      "groups", &groups,
 			      NULL);
-		text = pk_group_bitfield_to_text (groups);
+		text = pk_group_bitfield_to_string (groups);
 		g_strdelimit (text, ";", '\n');
 		g_print ("%s\n", text);
 		g_free (text);
@@ -1677,9 +1732,10 @@ main (int argc, char *argv[])
 						      (GAsyncReadyCallback) pk_console_finished_cb, NULL);
 
 	} else if (strcmp (mode, "refresh") == 0) {
-		pk_client_refresh_cache_async (PK_CLIENT(task), FALSE, cancellable,
-					       (PkProgressCallback) pk_console_progress_cb, NULL,
-					       (GAsyncReadyCallback) pk_console_finished_cb, NULL);
+		gboolean force = value && !strcmp (value, "--force");
+		pk_task_refresh_cache_async (PK_TASK (task), force, cancellable,
+					     (PkProgressCallback) pk_console_progress_cb, NULL,
+					     (GAsyncReadyCallback) pk_console_finished_cb, NULL);
 
 	} else {
 		/* TRANSLATORS: The user tried to use an unsupported option on the command line */
@@ -1700,6 +1756,7 @@ out:
 
 	g_free (options_help);
 	g_free (filter);
+	g_free (root);
 	g_free (summary);
 	g_object_unref (progressbar);
 	g_object_unref (control);
@@ -1709,67 +1766,3 @@ out_last:
 	return retval;
 }
 
-/***************************************************************************
- ***                          MAKE CHECK TESTS                           ***
- ***************************************************************************/
-#ifdef EGG_TEST
-#include "egg-test.h"
-
-void
-egg_test_console (EggTest *test)
-{
-	gchar *text_safe;
-
-	if (!egg_test_start (test, "PkConsole"))
-		return;
-
-	/************************************************************
-	 ****************         Padding          ******************
-	 ************************************************************/
-	egg_test_title (test, "pad smaller");
-	text_safe = pk_strpad ("richard", 10);
-	if (g_strcmp0 (text_safe, "richard   ") == 0)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "failed the padd '%s'", text_safe);
-	g_free (text_safe);
-
-	/************************************************************/
-	egg_test_title (test, "pad NULL");
-	text_safe = pk_strpad (NULL, 10);
-	if (g_strcmp0 (text_safe, "          ") == 0)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "failed the padd '%s'", text_safe);
-	g_free (text_safe);
-
-	/************************************************************/
-	egg_test_title (test, "pad nothing");
-	text_safe = pk_strpad ("", 10);
-	if (g_strcmp0 (text_safe, "          ") == 0)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "failed the padd '%s'", text_safe);
-	g_free (text_safe);
-
-	/************************************************************/
-	egg_test_title (test, "pad over");
-	text_safe = pk_strpad ("richardhughes", 10);
-	if (g_strcmp0 (text_safe, "richardhughes") == 0)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "failed the padd '%s'", text_safe);
-	g_free (text_safe);
-
-	/************************************************************/
-	egg_test_title (test, "pad zero");
-	text_safe = pk_strpad ("rich", 0);
-	if (g_strcmp0 (text_safe, "rich") == 0)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "failed the padd '%s'", text_safe);
-	g_free (text_safe);
-	egg_test_end (test);
-}
-#endif
-
