@@ -51,7 +51,9 @@ typedef struct {
 	PkCnfPolicy	 single_install;
 	PkCnfPolicy	 multiple_install;
 	gboolean	 software_source_search;
+	gboolean	 similar_name_search;
 	gchar		**locations;
+	guint		 max_search_time;
 } PkCnfPolicyConfig;
 
 static PkTask *task = NULL;
@@ -386,13 +388,24 @@ pk_cnf_progress_cb (PkProgress *progress, PkProgressType type, gpointer data)
 		g_print ("\n * %s.. ", text);
 }
 
+/**
+ * pk_cnf_cancel_cb:
+ */
+static gboolean
+pk_cnf_cancel_cb (GCancellable *_cancellable)
+{
+	egg_warning ("Cancelling request");
+	g_cancellable_cancel (cancellable);
+	return FALSE;
+}
+
 /**
  * pk_cnf_find_available:
  *
  * Find software we could install
  **/
 static gchar **
-pk_cnf_find_available (const gchar *cmd)
+pk_cnf_find_available (const gchar *cmd, guint max_search_time)
 {
 	PkPackage *item;
 	gchar **package_ids = NULL;
@@ -405,6 +418,7 @@ pk_cnf_find_available (const gchar *cmd)
 	PkBitfield filters;
 	PkResults *results = NULL;
 	PkError *error_code = NULL;
+	guint cancel_id;
 
 	/* create new array of full paths */
 	len = g_strv_length ((gchar **)prefixes);
@@ -412,6 +426,14 @@ pk_cnf_find_available (const gchar *cmd)
 	for (i=0; prefixes[i] != NULL; i++)
 		values[i] = g_build_filename (prefixes[i], cmd, NULL);
 
+	/* only allow searching for a limited amount of time */
+	cancel_id = g_timeout_add (max_search_time,
+				   (GSourceFunc) pk_cnf_cancel_cb,
+				   cancellable);
+#if GLIB_CHECK_VERSION(2,25,8)
+	g_source_set_name_by_id (cancel_id, "[PkCommandNotFount] cancel");
+#endif
+
 	/* do search */
 	filters = pk_bitfield_from_enums (PK_FILTER_ENUM_NOT_INSTALLED,
 					  PK_FILTER_ENUM_NEWEST,
@@ -429,7 +451,7 @@ pk_cnf_find_available (const gchar *cmd)
 	error_code = pk_results_get_error_code (results);
 	if (error_code != NULL) {
 		/* TRANSLATORS: the transaction failed in a way we could not expect */
-		g_printerr ("%s: %s, %s\n", _("The transaction failed"), pk_error_enum_to_text (pk_error_get_code (error_code)), pk_error_get_details (error_code));
+		g_printerr ("%s: %s, %s\n", _("The transaction failed"), pk_error_enum_to_string (pk_error_get_code (error_code)), pk_error_get_details (error_code));
 		goto out;
 	}
 
@@ -441,6 +463,8 @@ pk_cnf_find_available (const gchar *cmd)
 		package_ids[i] = g_strdup (pk_package_get_id (item));
 	}
 out:
+	if (cancel_id > 0)
+		g_source_remove (cancel_id);
 	if (error_code != NULL)
 		g_object_unref (error_code);
 	if (results != NULL)
@@ -512,6 +536,7 @@ pk_cnf_get_config (void)
 	config->single_install = PK_CNF_POLICY_UNKNOWN;
 	config->multiple_install = PK_CNF_POLICY_UNKNOWN;
 	config->software_source_search = FALSE;
+	config->similar_name_search = FALSE;
 	config->locations = NULL;
 
 	/* load file */
@@ -530,13 +555,19 @@ pk_cnf_get_config (void)
 	config->single_install = pk_cnf_get_policy_from_file (file, "SingleInstall");
 	config->multiple_install = pk_cnf_get_policy_from_file (file, "MultipleInstall");
 	config->software_source_search = g_key_file_get_boolean (file, "CommandNotFound", "SoftwareSourceSearch", NULL);
+	config->similar_name_search = g_key_file_get_boolean (file, "CommandNotFound", "SimilarNameSearch", NULL);
 	config->locations = g_key_file_get_string_list (file, "CommandNotFound", "SearchLocations", NULL, NULL);
+	config->max_search_time = g_key_file_get_integer (file, "CommandNotFound", "MaxSearchTime", NULL);
 
 	/* fallback */
 	if (config->locations == NULL) {
 		egg_warning ("not found SearchLocations, using fallback");
 		config->locations = g_strsplit ("/usr/bin;/usr/sbin", ";", -1);
 	}
+	if (config->max_search_time == 0) {
+		egg_warning ("not found MaxSearchTime, using fallback");
+		config->max_search_time = 2000;
+	}
 out:
 	g_free (path);
 	g_key_file_free (file);
@@ -600,7 +631,7 @@ pk_cnf_install_package_id (const gchar *package_id)
 	error_code = pk_results_get_error_code (results);
 	if (error_code != NULL) {
 		/* TRANSLATORS: the transaction failed in a way we could not expect */
-		g_printerr ("%s: %s, %s\n", _("The transaction failed"), pk_error_enum_to_text (pk_error_get_code (error_code)), pk_error_get_details (error_code));
+		g_printerr ("%s: %s, %s\n", _("The transaction failed"), pk_error_enum_to_string (pk_error_get_code (error_code)), pk_error_get_details (error_code));
 		goto out;
 	}
 
@@ -692,14 +723,22 @@ main (int argc, char *argv[])
 		goto out;
 	}
 
-	/* generate swizzles */
-	array = pk_cnf_find_alternatives (argv[1], len);
-
 	/* TRANSLATORS: the prefix of all the output telling the user why it's not executing */
 	g_printerr ("%s ", _("Command not found."));
 
+	/* user is not allowing CNF to do anything useful */
+	if (!config->software_source_search &&
+	    !config->similar_name_search) {
+		retval = EXIT_COMMAND_NOT_FOUND;
+		goto out;
+	}
+
+	/* generate swizzles */
+	if (config->similar_name_search)
+		array = pk_cnf_find_alternatives (argv[1], len);
+
 	/* one exact possibility */
-	if (array->len == 1) {
+	if (array != NULL && array->len == 1) {
 		possible = g_ptr_array_index (array, 0);
 		if (config->single_match == PK_CNF_POLICY_WARN) {
 			/* TRANSLATORS: tell the user what we think the command is */
@@ -724,7 +763,7 @@ main (int argc, char *argv[])
 		goto out;
 
 	/* multiple choice */
-	} else if (array->len > 1) {
+	} else if (array != NULL && array->len > 1) {
 		if (config->multiple_match == PK_CNF_POLICY_WARN) {
 			/* TRANSLATORS: show the user a list of commands that they could have meant */
 			g_printerr ("%s:\n", _("Similar commands are:"));
@@ -753,7 +792,7 @@ main (int argc, char *argv[])
 
 	/* only search using PackageKit if configured to do so */
 	} else if (config->software_source_search) {
-		package_ids = pk_cnf_find_available (argv[1]);
+		package_ids = pk_cnf_find_available (argv[1], config->max_search_time);
 		if (package_ids == NULL)
 			goto out;
 		len = g_strv_length (package_ids);
