@@ -2,21 +2,21 @@
  *
  * Copyright (C) 2008-2009 Richard Hughes <richard@hughsie.com>
  *
- * Licensed under the GNU General Public License Version 2
+ * Licensed under the GNU Lesser General Public License Version 2.1
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
  */
 
 #include "config.h"
@@ -50,11 +50,17 @@ pk_console_get_number (const gchar *question, guint maxnum)
 	g_print ("%s", question);
 
 	do {
+		char buffer[64];
+
+		/* swallow the \n at end of line too */
+		if (!fgets (buffer, 64, stdin))
+			break;
+
 		/* get a number */
-		retval = scanf("%u", &answer);
+		retval = sscanf(buffer, "%u", &answer);
 
 		/* positive */
-		if (retval == 1 && answer > 0 && answer <= (gint) maxnum)
+		if (answer > 0 && answer <= (gint) maxnum)
 			break;
 		g_print (_("Please enter a number from 1 to %i: "), maxnum);
 	} while (TRUE);
@@ -80,7 +86,7 @@ pk_console_getchar_unbuffered (void)
 	memcpy (&new_opts, &org_opts, sizeof(new_opts));
 	new_opts.c_lflag &= ~(ICANON | ECHO | ECHOE | ECHOK | ECHONL | ECHOPRT | ECHOKE | ICRNL);
 	tcsetattr (STDIN_FILENO, TCSANOW, &new_opts);
-	c = getchar ();
+	c = getc (stdin);
 
 	/* restore old settings */
 	res = tcsetattr (STDIN_FILENO, TCSANOW, &org_opts);
@@ -135,9 +141,11 @@ pk_console_get_prompt (const gchar *question, gboolean defaultyes)
 gchar *
 pk_console_resolve_package (PkClient *client, PkBitfield filter, const gchar *package_name, GError **error)
 {
+	const gchar *package_id_tmp;
 	gchar *package_id = NULL;
 	gboolean valid;
 	gchar **tmp;
+	gchar **split = NULL;
 	PkResults *results;
 	GPtrArray *array = NULL;
 	guint i;
@@ -160,13 +168,13 @@ pk_console_resolve_package (PkClient *client, PkBitfield filter, const gchar *pa
 	/* get the packages returned */
 	array = pk_results_get_package_array (results);
 	if (array == NULL) {
-		*error = g_error_new (1, 0, "did not get package struct for %s", package_name);
+		g_set_error (error, 1, 0, "did not get package struct for %s", package_name);
 		goto out;
 	}
 
 	/* nothing found */
 	if (array->len == 0) {
-		*error = g_error_new (1, 0, "could not find %s", package_name);
+		g_set_error (error, 1, 0, "could not find %s", package_name);
 		goto out;
 	}
 
@@ -183,13 +191,11 @@ pk_console_resolve_package (PkClient *client, PkBitfield filter, const gchar *pa
 	g_print ("%s\n", _("More than one package matches:"));
 	for (i=0; i<array->len; i++) {
 		package = g_ptr_array_index (array, i);
-		g_object_get (package,
-			      "package-id", &package_id,
-			      NULL);
-		printable = pk_package_id_to_printable (package_id);
-		g_print ("%i. %s\n", i+1, printable);
+		package_id_tmp = pk_package_get_id (package);
+		split = pk_package_id_split (package_id_tmp);
+		printable = pk_package_id_to_printable (package_id_tmp);
+		g_print ("%i. %s [%s]\n", i+1, printable, split[PK_PACKAGE_ID_DATA]);
 		g_free (printable);
-		g_free (package_id);
 	}
 
 	/* TRANSLATORS: This finds out which package in the list to use */
@@ -204,6 +210,7 @@ out:
 	if (array != NULL)
 		g_ptr_array_unref (array);
 	g_strfreev (tmp);
+	g_strfreev (split);
 	return package_id;
 }
 
@@ -394,7 +401,7 @@ pk_status_enum_to_localised_text (PkStatusEnum status)
 		text = _("Copying files");
 		break;
 	default:
-		egg_warning ("status unrecognised: %s", pk_status_enum_to_text (status));
+		egg_warning ("status unrecognised: %s", pk_status_enum_to_string (status));
 	}
 	return text;
 }
@@ -446,7 +453,7 @@ pk_info_enum_to_localised_text (PkInfoEnum info)
 		text = _("Available");
 		break;
 	default:
-		egg_warning ("info unrecognised: %s", pk_info_enum_to_text (info));
+		egg_warning ("info unrecognised: %s", pk_info_enum_to_string (info));
 	}
 	return text;
 }
@@ -684,47 +691,7 @@ pk_role_enum_to_localised_present (PkRoleEnum role)
 		text = _("Simulating update");
 		break;
 	default:
-		egg_warning ("role unrecognised: %s", pk_role_enum_to_text (role));
+		egg_warning ("role unrecognised: %s", pk_role_enum_to_string (role));
 	}
 	return text;
 }
-
-/***************************************************************************
- ***                          MAKE CHECK TESTS                           ***
- ***************************************************************************/
-#ifdef EGG_TEST
-#include "egg-test.h"
-
-void
-pk_console_test (gpointer user_data)
-{
-	EggTest *test = (EggTest *) user_data;
-	gboolean ret;
-
-	if (!egg_test_start (test, "PkConsole"))
-		return;
-
-	/************************************************************/
-	egg_test_title (test, "get prompt 1");
-	ret = pk_console_get_prompt ("press enter", TRUE);
-	egg_test_assert (test, ret);
-
-	/************************************************************/
-	egg_test_title (test, "get prompt 2");
-	ret = pk_console_get_prompt ("press enter", TRUE);
-	egg_test_assert (test, ret);
-
-	/************************************************************/
-	egg_test_title (test, "get prompt 3");
-	ret = pk_console_get_prompt ("press Y", TRUE);
-	egg_test_assert (test, ret);
-
-	/************************************************************/
-	egg_test_title (test, "get prompt 3");
-	ret = pk_console_get_prompt ("press N", TRUE);
-	egg_test_assert (test, !ret);
-
-	egg_test_end (test);
-}
-#endif
-
