@@ -2,21 +2,21 @@
  *
  * Copyright (C) 2008 Richard Hughes <richard@hughsie.com>
  *
- * Licensed under the GNU General Public License Version 2
+ * Licensed under the GNU Lesser General Public License Version 2.1
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
  */
 
 /**
@@ -132,6 +132,8 @@ typedef struct {
  * We are a GObject that sets errors
  *
  * Return value: Our personal error quark.
+ *
+ * Since: 0.5.2
  **/
 GQuark
 pk_control_error_quark (void)
@@ -264,6 +266,8 @@ pk_control_call_destroy_cb (PkControlState *state)
  * @user_data: the data to pass to @callback
  *
  * Gets a transacton ID from the daemon.
+ *
+ * Since: 0.5.2
  **/
 void
 pk_control_get_tid_async (PkControl *control, GCancellable *cancellable, GAsyncReadyCallback callback, gpointer user_data)
@@ -308,6 +312,8 @@ pk_control_get_tid_async (PkControl *control, GCancellable *cancellable, GAsyncR
  * Gets the result from the asynchronous function.
  *
  * Return value: the ID, or %NULL if unset, free with g_free()
+ *
+ * Since: 0.5.2
  **/
 gchar *
 pk_control_get_tid_finish (PkControl *control, GAsyncResult *res, GError **error)
@@ -554,6 +560,8 @@ out:
  * @user_data: the data to pass to @callback
  *
  * Gets the debugging state from the daemon.
+ *
+ * Since: 0.5.2
  **/
 void
 pk_control_get_daemon_state_async (PkControl *control, GCancellable *cancellable, GAsyncReadyCallback callback, gpointer user_data)
@@ -598,6 +606,8 @@ pk_control_get_daemon_state_async (PkControl *control, GCancellable *cancellable
  * Gets the result from the asynchronous function.
  *
  * Return value: the ID, or %NULL if unset, free with g_free()
+ *
+ * Since: 0.5.2
  **/
 gchar *
 pk_control_get_daemon_state_finish (PkControl *control, GAsyncResult *res, GError **error)
@@ -697,6 +707,8 @@ out:
  * @user_data: the data to pass to @callback
  *
  * Set a proxy on the PK daemon
+ *
+ * Since: 0.5.2
  **/
 void
 pk_control_set_proxy_async (PkControl *control, const gchar *proxy_http, const gchar *proxy_ftp, GCancellable *cancellable,
@@ -744,6 +756,8 @@ pk_control_set_proxy_async (PkControl *control, const gchar *proxy_http, const g
  * Gets the result from the asynchronous function.
  *
  * Return value: %TRUE if we set the proxy successfully
+ *
+ * Since: 0.5.2
  **/
 gboolean
 pk_control_set_proxy_finish (PkControl *control, GAsyncResult *res, GError **error)
@@ -768,6 +782,154 @@ pk_control_set_proxy_finish (PkControl *control, GAsyncResult *res, GError **err
 
 /***************************************************************************************************/
 
+/**
+ * pk_control_set_root_state_finish:
+ **/
+static void
+pk_control_set_root_state_finish (PkControlState *state, const GError *error)
+{
+	/* get result */
+	if (state->ret) {
+		g_simple_async_result_set_op_res_gboolean (state->res, state->ret);
+	} else {
+		g_simple_async_result_set_from_error (state->res, error);
+	}
+
+	/* remove from list */
+	g_ptr_array_remove (state->control->priv->calls, state);
+	if (state->call != NULL)
+		egg_warning ("state array remove %p (%p)", state, state->call);
+	else
+		egg_debug ("state array remove %p", state);
+
+	/* complete */
+	g_simple_async_result_complete_in_idle (state->res);
+
+	/* deallocate */
+	if (state->cancellable != NULL) {
+		g_cancellable_disconnect (state->cancellable, state->cancellable_id);
+		g_object_unref (state->cancellable);
+	}
+	g_object_unref (state->res);
+	g_object_unref (state->control);
+	g_slice_free (PkControlState, state);
+}
+
+/**
+ * pk_control_set_root_cb:
+ **/
+static void
+pk_control_set_root_cb (DBusGProxy *root, DBusGProxyCall *call, PkControlState *state)
+{
+	GError *error = NULL;
+	gchar *tid = NULL;
+	gboolean ret;
+
+	/* finished this call */
+	state->call = NULL;
+
+	/* get the result */
+	ret = dbus_g_proxy_end_call (root, call, &error,
+				     G_TYPE_INVALID);
+	if (!ret) {
+		egg_warning ("failed to set root: %s", error->message);
+		pk_control_set_root_state_finish (state, error);
+		g_error_free (error);
+		goto out;
+	}
+
+	/* save data */
+	state->ret = TRUE;
+
+	/* we're done */
+	pk_control_set_root_state_finish (state, NULL);
+out:
+	g_free (tid);
+}
+
+/**
+ * pk_control_set_root_async:
+ * @control: a valid #PkControl instance
+ * @root: an install root string such as "/mnt/ltsp"
+ * @cancellable: a #GCancellable or %NULL
+ * @callback: the function to run on completion
+ * @user_data: the data to pass to @callback
+ *
+ * Set the install root for the backend used by PackageKit
+ *
+ * Since: 0.6.4
+ **/
+void
+pk_control_set_root_async (PkControl *control, const gchar *root, GCancellable *cancellable,
+			   GAsyncReadyCallback callback, gpointer user_data)
+{
+	GSimpleAsyncResult *res;
+	PkControlState *state;
+
+	g_return_if_fail (PK_IS_CONTROL (control));
+	g_return_if_fail (callback != NULL);
+
+	res = g_simple_async_result_new (G_OBJECT (control), callback, user_data, pk_control_set_root_async);
+
+	/* save state */
+	state = g_slice_new0 (PkControlState);
+	state->res = g_object_ref (res);
+	state->control = g_object_ref (control);
+	if (cancellable != NULL) {
+		state->cancellable = g_object_ref (cancellable);
+		state->cancellable_id = g_cancellable_connect (cancellable, G_CALLBACK (pk_control_cancellable_cancel_cb), state, NULL);
+	}
+
+	/* call D-Bus set_root async */
+	state->call = dbus_g_proxy_begin_call (control->priv->proxy, "SetRoot",
+					       (DBusGProxyCallNotify) pk_control_set_root_cb, state, NULL,
+					       G_TYPE_STRING, root,
+					       G_TYPE_INVALID);
+	if (state->call == NULL)
+		egg_error ("failed to setup call, maybe OOM or no connection");
+
+	/* track state */
+	g_ptr_array_add (control->priv->calls, state);
+	egg_debug ("state array add %p (%p)", state, state->call);
+
+	g_object_unref (res);
+}
+
+/**
+ * pk_control_set_root_finish:
+ * @control: a valid #PkControl instance
+ * @res: the #GAsyncResult
+ * @error: A #GError or %NULL
+ *
+ * Gets the result from the asynchronous function.
+ *
+ * Return value: %TRUE if we set the root successfully
+ *
+ * Since: 0.6.4
+ **/
+gboolean
+pk_control_set_root_finish (PkControl *control, GAsyncResult *res, GError **error)
+{
+	GSimpleAsyncResult *simple;
+	gpointer source_tag;
+
+	g_return_val_if_fail (PK_IS_CONTROL (control), FALSE);
+	g_return_val_if_fail (G_IS_SIMPLE_ASYNC_RESULT (res), FALSE);
+	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+	simple = G_SIMPLE_ASYNC_RESULT (res);
+	source_tag = g_simple_async_result_get_source_tag (simple);
+
+	g_return_val_if_fail (source_tag == pk_control_set_root_async, FALSE);
+
+	if (g_simple_async_result_propagate_error (simple, error))
+		return FALSE;
+
+	return g_simple_async_result_get_op_res_gboolean (simple);
+}
+
+/***************************************************************************************************/
+
 /**
  * pk_control_get_transaction_list_state_finish:
  **/
@@ -845,6 +1007,8 @@ out:
  * @user_data: the data to pass to @callback
  *
  * Gets the transactions currently running in the daemon.
+ *
+ * Since: 0.5.2
  **/
 void
 pk_control_get_transaction_list_async (PkControl *control, GCancellable *cancellable, GAsyncReadyCallback callback, gpointer user_data)
@@ -889,6 +1053,8 @@ pk_control_get_transaction_list_async (PkControl *control, GCancellable *cancell
  * Gets the result from the asynchronous function.
  *
  * Return value: A GStrv list of transaction ID's, free with g_strfreev()
+ *
+ * Since: 0.5.2
  **/
 gchar **
 pk_control_get_transaction_list_finish (PkControl *control, GAsyncResult *res, GError **error)
@@ -997,6 +1163,8 @@ out:
  *
  * We may want to know how long it has been since we refreshed the cache or
  * retrieved the update list.
+ *
+ * Since: 0.5.2
  **/
 void
 pk_control_get_time_since_action_async (PkControl *control, PkRoleEnum role, GCancellable *cancellable, GAsyncReadyCallback callback, gpointer user_data)
@@ -1020,7 +1188,7 @@ pk_control_get_time_since_action_async (PkControl *control, PkRoleEnum role, GCa
 	}
 
 	/* call D-Bus get_time_since_action async */
-	role_text = pk_role_enum_to_text (role);
+	role_text = pk_role_enum_to_string (role);
 	state->call = dbus_g_proxy_begin_call (control->priv->proxy, "GetTimeSinceAction",
 					       (DBusGProxyCallNotify) pk_control_get_time_since_action_cb, state, NULL,
 					       G_TYPE_STRING, role_text,
@@ -1044,6 +1212,8 @@ pk_control_get_time_since_action_async (PkControl *control, PkRoleEnum role, GCa
  * Gets the result from the asynchronous function.
  *
  * Return value: %TRUE if the daemon serviced the request
+ *
+ * Since: 0.5.2
  **/
 guint
 pk_control_get_time_since_action_finish (PkControl *control, GAsyncResult *res, GError **error)
@@ -1068,157 +1238,6 @@ pk_control_get_time_since_action_finish (PkControl *control, GAsyncResult *res,
 
 /***************************************************************************************************/
 
-/**
- * pk_control_get_network_state_state_finish:
- **/
-static void
-pk_control_get_network_state_state_finish (PkControlState *state, const GError *error)
-{
-	/* get result */
-	if (state->network != PK_NETWORK_ENUM_UNKNOWN) {
-		g_simple_async_result_set_op_res_gssize (state->res, state->network);
-	} else {
-		g_simple_async_result_set_from_error (state->res, error);
-	}
-
-	/* remove from list */
-	g_ptr_array_remove (state->control->priv->calls, state);
-	if (state->call != NULL)
-		egg_warning ("state array remove %p (%p)", state, state->call);
-	else
-		egg_debug ("state array remove %p", state);
-
-	/* complete */
-	g_simple_async_result_complete_in_idle (state->res);
-
-	/* deallocate */
-	if (state->cancellable != NULL) {
-		g_cancellable_disconnect (state->cancellable, state->cancellable_id);
-		g_object_unref (state->cancellable);
-	}
-	g_object_unref (state->res);
-	g_object_unref (state->control);
-	g_slice_free (PkControlState, state);
-}
-
-/**
- * pk_control_get_network_state_cb:
- **/
-static void
-pk_control_get_network_state_cb (DBusGProxy *proxy, DBusGProxyCall *call, PkControlState *state)
-{
-	GError *error = NULL;
-	gboolean ret;
-	gchar *network_state = NULL;
-
-	/* finished this call */
-	state->call = NULL;
-
-	/* get the result */
-	ret = dbus_g_proxy_end_call (proxy, call, &error,
-				     G_TYPE_STRING, &network_state,
-				     G_TYPE_INVALID);
-	if (!ret) {
-		/* fix up the D-Bus error */
-		pk_control_fixup_dbus_error (error);
-		egg_warning ("failed: %s", error->message);
-		pk_control_get_network_state_state_finish (state, error);
-		g_error_free (error);
-		goto out;
-	}
-
-	/* save data */
-	state->network = pk_network_enum_from_text (network_state);
-	if (state->network == PK_NETWORK_ENUM_UNKNOWN) {
-		error = g_error_new (PK_CONTROL_ERROR, PK_CONTROL_ERROR_FAILED, "could not get state");
-		pk_control_get_network_state_state_finish (state, error);
-		g_error_free (error);
-		goto out;
-	}
-
-	/* we're done */
-	pk_control_get_network_state_state_finish (state, NULL);
-out:
-	g_free (network_state);
-	return;
-}
-
-/**
- * pk_control_get_network_state_async:
- * @control: a valid #PkControl instance
- * @cancellable: a #GCancellable or %NULL
- * @callback: the function to run on completion
- * @user_data: the data to pass to @callback
- *
- * Gets the network state.
- **/
-void
-pk_control_get_network_state_async (PkControl *control, GCancellable *cancellable, GAsyncReadyCallback callback, gpointer user_data)
-{
-	GSimpleAsyncResult *res;
-	PkControlState *state;
-
-	g_return_if_fail (PK_IS_CONTROL (control));
-	g_return_if_fail (callback != NULL);
-
-	res = g_simple_async_result_new (G_OBJECT (control), callback, user_data, pk_control_get_network_state_async);
-
-	/* save state */
-	state = g_slice_new0 (PkControlState);
-	state->res = g_object_ref (res);
-	state->control = g_object_ref (control);
-	state->network = PK_NETWORK_ENUM_UNKNOWN;
-	if (cancellable != NULL) {
-		state->cancellable = g_object_ref (cancellable);
-		state->cancellable_id = g_cancellable_connect (cancellable, G_CALLBACK (pk_control_cancellable_cancel_cb), state, NULL);
-	}
-
-	/* call D-Bus async */
-	state->call = dbus_g_proxy_begin_call (control->priv->proxy, "GetNetworkState",
-					       (DBusGProxyCallNotify) pk_control_get_network_state_cb, state, NULL,
-					       G_TYPE_INVALID);
-	if (state->call == NULL)
-		egg_error ("failed to setup call, maybe OOM or no connection");
-
-	/* track state */
-	g_ptr_array_add (control->priv->calls, state);
-	egg_debug ("state array add %p (%p)", state, state->call);
-
-	g_object_unref (res);
-}
-
-/**
- * pk_control_get_network_state_finish:
- * @control: a valid #PkControl instance
- * @res: the #GAsyncResult
- * @error: A #GError or %NULL
- *
- * Gets the result from the asynchronous function.
- *
- * Return value: an enumerated network state
- **/
-PkNetworkEnum
-pk_control_get_network_state_finish (PkControl *control, GAsyncResult *res, GError **error)
-{
-	GSimpleAsyncResult *simple;
-	gpointer source_tag;
-
-	g_return_val_if_fail (PK_IS_CONTROL (control), PK_NETWORK_ENUM_UNKNOWN);
-	g_return_val_if_fail (G_IS_SIMPLE_ASYNC_RESULT (res), PK_NETWORK_ENUM_UNKNOWN);
-
-	simple = G_SIMPLE_ASYNC_RESULT (res);
-	source_tag = g_simple_async_result_get_source_tag (simple);
-
-	g_return_val_if_fail (source_tag == pk_control_get_network_state_async, PK_NETWORK_ENUM_UNKNOWN);
-
-	if (g_simple_async_result_propagate_error (simple, error))
-		return PK_NETWORK_ENUM_UNKNOWN;
-
-	return (PkNetworkEnum) g_simple_async_result_get_op_res_gssize (simple);
-}
-
-/***************************************************************************************************/
-
 /**
  * pk_control_can_authorize_state_finish:
  **/
@@ -1279,7 +1298,7 @@ pk_control_can_authorize_cb (DBusGProxy *proxy, DBusGProxyCall *call, PkControlS
 	}
 
 	/* save data */
-	state->authorize = pk_authorize_type_enum_from_text (authorize_state);
+	state->authorize = pk_authorize_type_enum_from_string (authorize_state);
 	if (state->authorize == PK_AUTHORIZE_ENUM_UNKNOWN) {
 		error = g_error_new (PK_CONTROL_ERROR, PK_CONTROL_ERROR_FAILED, "could not get state");
 		pk_control_can_authorize_state_finish (state, error);
@@ -1304,6 +1323,8 @@ out:
  *
  * We may want to know before we run a method if we are going to be denied,
  * accepted or challenged for authentication.
+ *
+ * Since: 0.5.2
  **/
 void
 pk_control_can_authorize_async (PkControl *control, const gchar *action_id, GCancellable *cancellable, GAsyncReadyCallback callback, gpointer user_data)
@@ -1350,6 +1371,8 @@ pk_control_can_authorize_async (PkControl *control, const gchar *action_id, GCan
  * Gets the result from the asynchronous function.
  *
  * Return value: the %PkAuthorizeEnum or %PK_AUTHORIZE_ENUM_UNKNOWN if the method failed
+ *
+ * Since: 0.5.2
  **/
 PkAuthorizeEnum
 pk_control_can_authorize_finish (PkControl *control, GAsyncResult *res, GError **error)
@@ -1640,15 +1663,15 @@ pk_control_get_properties_collect_cb (const char *key, const GValue *value, PkCo
 	} else if (g_strcmp0 (key, "MimeTypes") == 0) {
 		pk_control_set_mime_types (control, g_value_get_string (value));
 	} else if (g_strcmp0 (key, "Roles") == 0) {
-		pk_control_set_roles (control, pk_role_bitfield_from_text (g_value_get_string (value)));
+		pk_control_set_roles (control, pk_role_bitfield_from_string (g_value_get_string (value)));
 	} else if (g_strcmp0 (key, "Groups") == 0) {
-		pk_control_set_groups (control, pk_group_bitfield_from_text (g_value_get_string (value)));
+		pk_control_set_groups (control, pk_group_bitfield_from_string (g_value_get_string (value)));
 	} else if (g_strcmp0 (key, "Filters") == 0) {
-		pk_control_set_filters (control, pk_filter_bitfield_from_text (g_value_get_string (value)));
+		pk_control_set_filters (control, pk_filter_bitfield_from_string (g_value_get_string (value)));
 	} else if (g_strcmp0 (key, "Locked") == 0) {
 		pk_control_set_locked (control, g_value_get_boolean (value));
 	} else if (g_strcmp0 (key, "NetworkState") == 0) {
-		pk_control_set_network_state (control, pk_network_enum_from_text (g_value_get_string (value)));
+		pk_control_set_network_state (control, pk_network_enum_from_string (g_value_get_string (value)));
 	} else if (g_strcmp0 (key, "DistroId") == 0) {
 		pk_control_set_distro_id (control, g_value_get_string (value));
 	} else {
@@ -1705,6 +1728,8 @@ out:
  * @user_data: the data to pass to @callback
  *
  * Gets global properties from the daemon.
+ *
+ * Since: 0.5.2
  **/
 void
 pk_control_get_properties_async (PkControl *control, GCancellable *cancellable,
@@ -1751,6 +1776,8 @@ pk_control_get_properties_async (PkControl *control, GCancellable *cancellable,
  * Gets the result from the asynchronous function.
  *
  * Return value: %TRUE if we set the proxy successfully
+ *
+ * Since: 0.5.2
  **/
 gboolean
 pk_control_get_properties_finish (PkControl *control, GAsyncResult *res, GError **error)
@@ -1827,7 +1854,12 @@ pk_control_transaction_list_changed_cb (DBusGProxy *proxy, gchar **transaction_i
 
 	/* we have to do this idle as the transaction list will change when not yet finished */
 	egg_debug ("emit transaction-list-changed (when idle)");
-	control->priv->transaction_list_changed_id = g_idle_add ((GSourceFunc) pk_control_transaction_list_changed_idle_cb, store);
+	control->priv->transaction_list_changed_id =
+		g_idle_add ((GSourceFunc) pk_control_transaction_list_changed_idle_cb, store);
+#if GLIB_CHECK_VERSION(2,25,8)
+	g_source_set_name_by_id (control->priv->transaction_list_changed_id,
+				 "[PkControl] transaction-list-changed");
+#endif
 }
 
 /**
@@ -1865,7 +1897,12 @@ pk_control_restart_schedule_cb (DBusGProxy *proxy, PkControl *control)
 
 	/* we have to do this idle as the transaction list will change when not yet finished */
 	egg_debug ("emit restart-schedule (when idle)");
-	store->control->priv->restart_schedule_id = g_idle_add ((GSourceFunc) pk_control_restart_schedule_idle_cb, store);
+	store->control->priv->restart_schedule_id =
+		g_idle_add ((GSourceFunc) pk_control_restart_schedule_idle_cb, store);
+#if GLIB_CHECK_VERSION(2,25,8)
+	g_source_set_name_by_id (store->control->priv->restart_schedule_id,
+				 "[PkControl] restart-schedule");
+#endif
 }
 
 /**
@@ -1903,7 +1940,12 @@ pk_control_updates_changed_cb (DBusGProxy *proxy, PkControl *control)
 
 	/* we have to do this idle as the transaction list will change when not yet finished */
 	egg_debug ("emit updates-changed (when idle)");
-	control->priv->updates_changed_id = g_idle_add ((GSourceFunc) pk_control_updates_changed_idle_cb, store);
+	control->priv->updates_changed_id =
+		g_idle_add ((GSourceFunc) pk_control_updates_changed_idle_cb, store);
+#if GLIB_CHECK_VERSION(2,25,8)
+	g_source_set_name_by_id (control->priv->updates_changed_id,
+				 "[PkControl] updates-changed");
+#endif
 }
 
 /**
@@ -1941,7 +1983,12 @@ pk_control_repo_list_changed_cb (DBusGProxy *proxy, PkControl *control)
 
 	/* we have to do this idle as the transaction list will change when not yet finished */
 	egg_debug ("emit repo-list-changed (when idle)");
-	control->priv->repo_list_changed_id = g_idle_add ((GSourceFunc) pk_control_repo_list_changed_idle_cb, store);
+	control->priv->repo_list_changed_id =
+		g_idle_add ((GSourceFunc) pk_control_repo_list_changed_idle_cb, store);
+#if GLIB_CHECK_VERSION(2,25,8)
+	g_source_set_name_by_id (control->priv->repo_list_changed_id,
+				 "[PkControl] repo-list-changed");
+#endif
 }
 
 /**
@@ -2149,6 +2196,8 @@ pk_control_class_init (PkControlClass *klass)
 
 	/**
 	 * PkControl:version-major:
+	 *
+	 * Since: 0.5.2
 	 */
 	pspec = g_param_spec_uint ("version-major", NULL, NULL,
 				   0, G_MAXUINT, 0,
@@ -2157,6 +2206,8 @@ pk_control_class_init (PkControlClass *klass)
 
 	/**
 	 * PkControl:version-minor:
+	 *
+	 * Since: 0.5.2
 	 */
 	pspec = g_param_spec_uint ("version-minor", NULL, NULL,
 				   0, G_MAXUINT, 0,
@@ -2165,6 +2216,8 @@ pk_control_class_init (PkControlClass *klass)
 
 	/**
 	 * PkControl:version-micro:
+	 *
+	 * Since: 0.5.2
 	 */
 	pspec = g_param_spec_uint ("version-micro", NULL, NULL,
 				   0, G_MAXUINT, 0,
@@ -2173,6 +2226,8 @@ pk_control_class_init (PkControlClass *klass)
 
 	/**
 	 * PkControl:backend-name:
+	 *
+	 * Since: 0.5.2
 	 */
 	pspec = g_param_spec_string ("backend-name", NULL, NULL,
 				     NULL,
@@ -2181,6 +2236,8 @@ pk_control_class_init (PkControlClass *klass)
 
 	/**
 	 * PkControl:backend-description:
+	 *
+	 * Since: 0.5.2
 	 */
 	pspec = g_param_spec_string ("backend-description", NULL, NULL,
 				     NULL,
@@ -2189,6 +2246,8 @@ pk_control_class_init (PkControlClass *klass)
 
 	/**
 	 * PkControl:backend-author:
+	 *
+	 * Since: 0.5.2
 	 */
 	pspec = g_param_spec_string ("backend-author", NULL, NULL,
 				     NULL,
@@ -2197,6 +2256,8 @@ pk_control_class_init (PkControlClass *klass)
 
 	/**
 	 * PkControl:roles:
+	 *
+	 * Since: 0.5.2
 	 */
 	pspec = g_param_spec_uint64 ("roles", NULL, NULL,
 				     0, G_MAXUINT64, 0,
@@ -2205,6 +2266,8 @@ pk_control_class_init (PkControlClass *klass)
 
 	/**
 	 * PkControl:groups:
+	 *
+	 * Since: 0.5.2
 	 */
 	pspec = g_param_spec_uint64 ("groups", NULL, NULL,
 				     0, G_MAXUINT64, 0,
@@ -2213,6 +2276,8 @@ pk_control_class_init (PkControlClass *klass)
 
 	/**
 	 * PkControl:filters:
+	 *
+	 * Since: 0.5.2
 	 */
 	pspec = g_param_spec_uint64 ("filters", NULL, NULL,
 				     0, G_MAXUINT64, 0,
@@ -2221,6 +2286,8 @@ pk_control_class_init (PkControlClass *klass)
 
 	/**
 	 * PkControl:mime-types:
+	 *
+	 * Since: 0.5.2
 	 */
 	pspec = g_param_spec_string ("mime-types", NULL, NULL,
 				     NULL,
@@ -2229,6 +2296,8 @@ pk_control_class_init (PkControlClass *klass)
 
 	/**
 	 * PkControl:locked:
+	 *
+	 * Since: 0.5.3
 	 */
 	pspec = g_param_spec_boolean ("locked", NULL, NULL,
 				      FALSE,
@@ -2237,6 +2306,8 @@ pk_control_class_init (PkControlClass *klass)
 
 	/**
 	 * PkControl:network-state:
+	 *
+	 * Since: 0.5.3
 	 */
 	pspec = g_param_spec_uint ("network-state", NULL, NULL,
 				   0, G_MAXUINT, PK_NETWORK_ENUM_LAST,
@@ -2245,6 +2316,8 @@ pk_control_class_init (PkControlClass *klass)
 
 	/**
 	 * PkControl:distro-id:
+	 *
+	 * Since: 0.5.5
 	 */
 	pspec = g_param_spec_string ("distro-id", NULL, NULL,
 				     NULL,
@@ -2253,6 +2326,8 @@ pk_control_class_init (PkControlClass *klass)
 
 	/**
 	 * PkControl:connected:
+	 *
+	 * Since: 0.5.3
 	 */
 	pspec = g_param_spec_boolean ("connected", NULL, NULL,
 				      FALSE,
@@ -2465,6 +2540,8 @@ pk_control_finalize (GObject *object)
  * pk_control_new:
  *
  * Return value: a new PkControl object.
+ *
+ * Since: 0.5.2
  **/
 PkControl *
 pk_control_new (void)
@@ -2477,285 +2554,3 @@ pk_control_new (void)
 	}
 	return PK_CONTROL (pk_control_object);
 }
-
-/***************************************************************************
- ***                          MAKE CHECK TESTS                           ***
- ***************************************************************************/
-#ifdef EGG_TEST
-#include "egg-test.h"
-#include "pk-control-sync.h"
-
-static guint _refcount = 0;
-
-static void
-pk_control_test_get_tid_cb (GObject *object, GAsyncResult *res, EggTest *test)
-{
-	PkControl *control = PK_CONTROL (object);
-	GError *error = NULL;
-	gchar *tid;
-
-	/* get the result */
-	tid = pk_control_get_tid_finish (control, res, &error);
-	if (tid == NULL) {
-		egg_test_failed (test, "failed to get transaction: %s", error->message);
-		g_error_free (error);
-		return;
-	}
-
-	egg_debug ("tid = %s", tid);
-	g_free (tid);
-	if (--_refcount == 0)
-		egg_test_loop_quit (test);
-}
-
-static void
-pk_control_test_get_properties_cb (GObject *object, GAsyncResult *res, EggTest *test)
-{
-	PkControl *control = PK_CONTROL (object);
-	GError *error = NULL;
-	gboolean ret;
-	PkBitfield roles;
-	PkBitfield filters;
-	PkBitfield groups;
-	gchar *text;
-
-	/* get the result */
-	ret = pk_control_get_properties_finish (control, res, &error);
-	if (!ret) {
-		egg_test_failed (test, "failed to get properties: %s", error->message);
-		g_error_free (error);
-		return;
-	}
-
-	/* get values */
-	g_object_get (control,
-		      "mime-types", &text,
-		      "roles", &roles,
-		      "filters", &filters,
-		      "groups", &groups,
-		      NULL);
-
-	/* check mime_types */
-	if (g_strcmp0 (text, "application/x-rpm;application/x-deb") != 0) {
-		egg_test_failed (test, "data incorrect: %s", text);
-		return;
-	}
-	g_free (text);
-
-	/* check roles */
-	text = pk_role_bitfield_to_text (roles);
-	if (g_strcmp0 (text, "cancel;get-depends;get-details;get-files;get-packages;get-repo-list;"
-			     "get-requires;get-update-detail;get-updates;install-files;install-packages;install-signature;"
-			     "refresh-cache;remove-packages;repo-enable;repo-set-data;resolve;rollback;"
-			     "search-details;search-file;search-group;search-name;update-packages;update-system;"
-			     "what-provides;download-packages;get-distro-upgrades;simulate-install-packages;"
-			     "simulate-remove-packages;simulate-update-packages") != 0) {
-		egg_test_failed (test, "data incorrect: %s", text);
-		return;
-	}
-	g_free (text);
-
-	/* check filters */
-	text = pk_filter_bitfield_to_text (filters);
-	if (g_strcmp0 (text, "installed;devel;gui") != 0) {
-		egg_test_failed (test, "data incorrect: %s", text);
-		return;
-	}
-	g_free (text);
-
-	/* check groups */
-	text = pk_group_bitfield_to_text (groups);
-	if (g_strcmp0 (text, "accessibility;games;system") != 0) {
-		egg_test_failed (test, "data incorrect: %s", text);
-		return;
-	}
-	egg_debug ("groups = %s", text);
-
-	g_free (text);
-
-	if (--_refcount == 0)
-		egg_test_loop_quit (test);
-}
-
-static void
-pk_control_test_get_time_since_action_cb (GObject *object, GAsyncResult *res, EggTest *test)
-{
-	PkControl *control = PK_CONTROL (object);
-	GError *error = NULL;
-	guint seconds;
-
-	/* get the result */
-	seconds = pk_control_get_time_since_action_finish (control, res, &error);
-	if (seconds == 0) {
-		egg_test_failed (test, "failed to get time: %s", error->message);
-		g_error_free (error);
-		return;
-	}
-
-	egg_test_loop_quit (test);
-}
-
-static void
-pk_control_test_get_network_state_cb (GObject *object, GAsyncResult *res, EggTest *test)
-{
-	PkControl *control = PK_CONTROL (object);
-	GError *error = NULL;
-	PkNetworkEnum network;
-
-	/* get the result */
-	network = pk_control_get_network_state_finish (control, res, &error);
-	if (network == PK_NETWORK_ENUM_UNKNOWN) {
-		egg_test_failed (test, "failed to get network state: %s", error->message);
-		g_error_free (error);
-		return;
-	}
-
-	egg_test_loop_quit (test);
-}
-
-static void
-pk_control_test_can_authorize_cb (GObject *object, GAsyncResult *res, EggTest *test)
-{
-	PkControl *control = PK_CONTROL (object);
-	GError *error = NULL;
-	PkAuthorizeEnum auth;
-
-	/* get the result */
-	auth = pk_control_can_authorize_finish (control, res, &error);
-	if (auth == PK_AUTHORIZE_ENUM_UNKNOWN) {
-		egg_test_failed (test, "failed to get auth: %s", error->message);
-		g_error_free (error);
-		return;
-	}
-
-	egg_test_loop_quit (test);
-}
-
-void
-pk_control_test (gpointer user_data)
-{
-	EggTest *test = (EggTest *) user_data;
-	PkControl *control;
-	guint version;
-	GError *error = NULL;
-	gboolean ret;
-	gchar *text;
-	PkBitfield roles;
-	guint i;
-	const guint LOOP_SIZE = 5;
-
-	if (!egg_test_start (test, "PkControl"))
-		return;
-
-	/************************************************************/
-	egg_test_title (test, "get control");
-	control = pk_control_new ();
-	egg_test_assert (test, control != NULL);
-
-	/************************************************************/
-	egg_test_title (test, "get TID async");
-	_refcount = 1;
-	pk_control_get_tid_async (control, NULL, (GAsyncReadyCallback) pk_control_test_get_tid_cb, test);
-	egg_test_loop_wait (test, 5000);
-	egg_test_success (test, "got tid in %i", egg_test_elapsed (test));
-
-	/************************************************************/
-	egg_test_title (test, "get multiple TIDs async");
-	_refcount = LOOP_SIZE;
-	for (i=0; i<_refcount; i++) {
-		egg_debug ("getting #%i", i+1);
-		pk_control_get_tid_async (control, NULL, (GAsyncReadyCallback) pk_control_test_get_tid_cb, test);
-	}
-	egg_test_loop_wait (test, 5000);
-	egg_test_success (test, "got %i tids in %i", LOOP_SIZE, egg_test_elapsed (test));
-
-	/************************************************************/
-	egg_test_title (test, "get properties async");
-	_refcount = 1;
-	pk_control_get_properties_async (control, NULL, (GAsyncReadyCallback) pk_control_test_get_properties_cb, test);
-	egg_test_loop_wait (test, 5000);
-	egg_test_success (test, "got properties types in %i", egg_test_elapsed (test));
-
-	/************************************************************/
-	egg_test_title (test, "get properties async (again, to test caching)");
-	_refcount = 1;
-	pk_control_get_properties_async (control, NULL, (GAsyncReadyCallback) pk_control_test_get_properties_cb, test);
-	egg_test_loop_wait (test, 5000);
-	egg_test_success (test, "got properties in %i", egg_test_elapsed (test));
-
-	/************************************************************/
-	egg_test_title (test, "do multiple requests async");
-	_refcount = LOOP_SIZE * 4;
-	for (i=0; i<_refcount; i++) {
-		egg_debug ("getting #%i", i+1);
-		pk_control_get_tid_async (control, NULL, (GAsyncReadyCallback) pk_control_test_get_tid_cb, test);
-		pk_control_get_properties_async (control, NULL, (GAsyncReadyCallback) pk_control_test_get_properties_cb, test);
-		pk_control_get_tid_async (control, NULL, (GAsyncReadyCallback) pk_control_test_get_tid_cb, test);
-		pk_control_get_properties_async (control, NULL, (GAsyncReadyCallback) pk_control_test_get_properties_cb, test);
-	}
-	egg_test_loop_wait (test, 5000);
-	egg_test_success (test, "got %i 2*properties and 2*tids in %i", LOOP_SIZE, egg_test_elapsed (test));
-
-	/************************************************************/
-	egg_test_title (test, "get time since async");
-	pk_control_get_time_since_action_async (control, PK_ROLE_ENUM_GET_UPDATES, NULL, (GAsyncReadyCallback) pk_control_test_get_time_since_action_cb, test);
-	egg_test_loop_wait (test, 5000);
-	egg_test_success (test, "got get time since in %i", egg_test_elapsed (test));
-
-	/************************************************************/
-	egg_test_title (test, "get network state async");
-	pk_control_get_network_state_async (control, NULL, (GAsyncReadyCallback) pk_control_test_get_network_state_cb, test);
-	egg_test_loop_wait (test, 5000);
-	egg_test_success (test, "get network state in %i", egg_test_elapsed (test));
-
-	/************************************************************/
-	egg_test_title (test, "get auth state async");
-	pk_control_can_authorize_async (control, "org.freedesktop.packagekit.system-update", NULL,
-					(GAsyncReadyCallback) pk_control_test_can_authorize_cb, test);
-	egg_test_loop_wait (test, 5000);
-	egg_test_success (test, "get auth state in %i", egg_test_elapsed (test));
-
-	/************************************************************/
-	egg_test_title (test, "version major");
-	g_object_get (control, "version-major", &version, NULL);
-	egg_test_assert (test, (version == PK_MAJOR_VERSION));
-
-	/************************************************************/
-	egg_test_title (test, "version minor");
-	g_object_get (control, "version-minor", &version, NULL);
-	egg_test_assert (test, (version == PK_MINOR_VERSION));
-
-	/************************************************************/
-	egg_test_title (test, "version micro");
-	g_object_get (control, "version-micro", &version, NULL);
-	egg_test_assert (test, (version == PK_MICRO_VERSION));
-
-	/************************************************************/
-	egg_test_title (test, "get properties sync");
-	ret = pk_control_get_properties (control, NULL, &error);
-	if (!ret)
-		egg_test_failed (test, "failed to get properties: %s", error->message);
-
-	/* get data */
-	g_object_get (control,
-		      "roles", &roles,
-		      NULL);
-
-	/* check data */
-	text = pk_role_bitfield_to_text (roles);
-	if (g_strcmp0 (text, "cancel;get-depends;get-details;get-files;get-packages;get-repo-list;"
-			     "get-requires;get-update-detail;get-updates;install-files;install-packages;install-signature;"
-			     "refresh-cache;remove-packages;repo-enable;repo-set-data;resolve;rollback;"
-			     "search-details;search-file;search-group;search-name;update-packages;update-system;"
-			     "what-provides;download-packages;get-distro-upgrades;simulate-install-packages;"
-			     "simulate-remove-packages;simulate-update-packages") != 0) {
-		egg_test_failed (test, "data incorrect: %s", text);
-	}
-	egg_test_success (test, "got correct roles");
-	g_free (text);
-
-	g_object_unref (control);
-	egg_test_end (test);
-}
-#endif
-
