@@ -2,21 +2,21 @@
  *
  * Copyright (C) 2007-2008 Richard Hughes <richard@hughsie.com>
  *
- * Licensed under the GNU General Public License Version 2
+ * Licensed under the GNU Lesser General Public License Version 2.1
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
  */
 
 #if !defined (__PACKAGEKIT_H_INSIDE__) && !defined (PK_COMPILATION)
@@ -30,41 +30,41 @@
 
 G_BEGIN_DECLS
 
-/* These are compatibility enums so we can use the new names in client tools in the future */
-#define pk_sig_type_enum_from_string		pk_sig_type_enum_from_text
-#define pk_sig_type_enum_to_string		pk_sig_type_enum_to_text
-#define pk_distro_upgrade_enum_from_string	pk_distro_upgrade_enum_from_text
-#define pk_distro_upgrade_enum_to_string	pk_distro_upgrade_enum_to_text
-#define pk_provides_enum_from_string		pk_provides_enum_from_text
-#define pk_provides_enum_to_string		pk_provides_enum_to_text
-#define pk_info_enum_from_string		pk_info_enum_from_text
-#define pk_info_enum_to_string			pk_info_enum_to_text
-#define pk_exit_enum_from_string		pk_exit_enum_from_text
-#define pk_exit_enum_to_string			pk_exit_enum_to_text
-#define pk_network_enum_from_string		pk_network_enum_from_text
-#define pk_network_enum_to_string		pk_network_enum_to_text
-#define pk_status_enum_from_string		pk_status_enum_from_text
-#define pk_status_enum_to_string		pk_status_enum_to_text
-#define pk_role_enum_from_string		pk_role_enum_from_text
-#define pk_role_enum_to_string			pk_role_enum_to_text
-#define pk_error_enum_from_string		pk_error_enum_from_text
-#define pk_error_enum_to_string			pk_error_enum_to_text
-#define pk_restart_enum_from_string		pk_restart_enum_from_text
-#define pk_restart_enum_to_string		pk_restart_enum_to_text
-#define pk_message_enum_from_string		pk_message_enum_from_text
-#define pk_message_enum_to_string		pk_message_enum_to_text
-#define pk_group_enum_from_string		pk_group_enum_from_text
-#define pk_group_enum_to_string			pk_group_enum_to_text
-#define pk_update_state_enum_from_string	pk_update_state_enum_from_text
-#define pk_update_state_enum_to_string		pk_update_state_enum_to_text
-#define pk_filter_enum_from_string		pk_filter_enum_from_text
-#define pk_filter_enum_to_string		pk_filter_enum_to_text
-#define pk_license_enum_from_string		pk_license_enum_from_text
-#define pk_license_enum_to_string		pk_license_enum_to_text
-#define pk_media_type_enum_from_string		pk_media_type_enum_from_text
-#define pk_media_type_enum_to_string		pk_media_type_enum_to_text
-#define pk_authorize_type_enum_from_string	pk_authorize_type_enum_from_text
-#define pk_authorize_type_enum_to_string	pk_authorize_type_enum_to_text
+/* These are compatibility enums so we can break API in the future */
+#define pk_sig_type_enum_from_text		pk_sig_type_enum_from_string
+#define pk_sig_type_enum_to_text		pk_sig_type_enum_to_string
+#define pk_distro_upgrade_enum_from_text	pk_distro_upgrade_enum_from_string
+#define pk_distro_upgrade_enum_to_text		pk_distro_upgrade_enum_to_string
+#define pk_provides_enum_from_text		pk_provides_enum_from_string
+#define pk_provides_enum_to_text		pk_provides_enum_to_string
+#define pk_info_enum_from_text			pk_info_enum_from_string
+#define pk_info_enum_to_text			pk_info_enum_to_string
+#define pk_exit_enum_from_text			pk_exit_enum_from_string
+#define pk_exit_enum_to_text			pk_exit_enum_to_string
+#define pk_network_enum_from_text		pk_network_enum_from_string
+#define pk_network_enum_to_text			pk_network_enum_to_string
+#define pk_status_enum_from_text		pk_status_enum_from_string
+#define pk_status_enum_to_text			pk_status_enum_to_string
+#define pk_role_enum_from_text			pk_role_enum_from_string
+#define pk_role_enum_to_text			pk_role_enum_to_string
+#define pk_error_enum_from_text			pk_error_enum_from_string
+#define pk_error_enum_to_text			pk_error_enum_to_string
+#define pk_restart_enum_from_text		pk_restart_enum_from_string
+#define pk_restart_enum_to_text			pk_restart_enum_to_string
+#define pk_message_enum_from_text		pk_message_enum_from_string
+#define pk_message_enum_to_text			pk_message_enum_to_string
+#define pk_group_enum_from_text			pk_group_enum_from_string
+#define pk_group_enum_to_text			pk_group_enum_to_string
+#define pk_update_state_enum_from_text		pk_update_state_enum_from_string
+#define pk_update_state_enum_to_text		pk_update_state_enum_to_string
+#define pk_filter_enum_from_text		pk_filter_enum_from_string
+#define pk_filter_enum_to_text			pk_filter_enum_to_string
+#define pk_license_enum_from_text		pk_license_enum_from_string
+#define pk_license_enum_to_text			pk_license_enum_to_string
+#define pk_media_type_enum_from_text		pk_media_type_enum_from_string
+#define pk_media_type_enum_to_text		pk_media_type_enum_to_string
+#define pk_authorize_type_enum_from_text	pk_authorize_type_enum_from_string
+#define pk_authorize_type_enum_to_text		pk_authorize_type_enum_to_string
 
 /**
  * PkEnumMatch:
@@ -288,6 +288,7 @@ typedef enum {
 	PK_MESSAGE_ENUM_AUTOREMOVE_IGNORED,
 	PK_MESSAGE_ENUM_REPO_METADATA_DOWNLOAD_FAILED,
 	PK_MESSAGE_ENUM_REPO_FOR_DEVELOPERS_ONLY,
+	PK_MESSAGE_ENUM_OTHER_UPDATES_HELD_BACK,
 	PK_MESSAGE_ENUM_LAST
 } PkMessageEnum;
 
@@ -359,6 +360,8 @@ typedef enum {
 	PK_ERROR_ENUM_PACKAGE_FAILED_TO_REMOVE,
 	PK_ERROR_ENUM_UPDATE_FAILED_DUE_TO_RUNNING_PROCESS,
 	PK_ERROR_ENUM_PACKAGE_DATABASE_CHANGED,
+	PK_ERROR_ENUM_PROVIDE_TYPE_NOT_SUPPORTED,
+	PK_ERROR_ENUM_INSTALL_ROOT_INVALID,
 	PK_ERROR_ENUM_LAST
 } PkErrorEnum;
 
@@ -715,56 +718,56 @@ const gchar	*pk_enum_find_string			(const PkEnumMatch *table,
 							 guint		 value)
 							 G_GNUC_WARN_UNUSED_RESULT;
 
-PkSigTypeEnum	 pk_sig_type_enum_from_text		(const gchar	*sig_type);
-const gchar	*pk_sig_type_enum_to_text		(PkSigTypeEnum	 sig_type);
+PkSigTypeEnum	 pk_sig_type_enum_from_string		(const gchar	*sig_type);
+const gchar	*pk_sig_type_enum_to_string		(PkSigTypeEnum	 sig_type);
 
-PkInfoEnum	 pk_info_enum_from_text			(const gchar	*info);
-const gchar	*pk_info_enum_to_text			(PkInfoEnum	 info);
+PkInfoEnum	 pk_info_enum_from_string			(const gchar	*info);
+const gchar	*pk_info_enum_to_string			(PkInfoEnum	 info);
 
-PkUpdateStateEnum  pk_update_state_enum_from_text	(const gchar	*update_state);
-const gchar	*pk_update_state_enum_to_text		(PkUpdateStateEnum update_state);
+PkUpdateStateEnum  pk_update_state_enum_from_string	(const gchar	*update_state);
+const gchar	*pk_update_state_enum_to_string		(PkUpdateStateEnum update_state);
 
-PkExitEnum	 pk_exit_enum_from_text			(const gchar	*exit);
-const gchar	*pk_exit_enum_to_text			(PkExitEnum	 exit);
+PkExitEnum	 pk_exit_enum_from_string			(const gchar	*exit);
+const gchar	*pk_exit_enum_to_string			(PkExitEnum	 exit);
 
-PkNetworkEnum	 pk_network_enum_from_text		(const gchar	*network);
-const gchar	*pk_network_enum_to_text		(PkNetworkEnum	 network);
+PkNetworkEnum	 pk_network_enum_from_string		(const gchar	*network);
+const gchar	*pk_network_enum_to_string		(PkNetworkEnum	 network);
 
-PkStatusEnum	 pk_status_enum_from_text		(const gchar	*status);
-const gchar	*pk_status_enum_to_text			(PkStatusEnum	 status);
+PkStatusEnum	 pk_status_enum_from_string		(const gchar	*status);
+const gchar	*pk_status_enum_to_string			(PkStatusEnum	 status);
 
-PkRoleEnum	 pk_role_enum_from_text			(const gchar	*role);
-const gchar	*pk_role_enum_to_text			(PkRoleEnum	 role);
+PkRoleEnum	 pk_role_enum_from_string			(const gchar	*role);
+const gchar	*pk_role_enum_to_string			(PkRoleEnum	 role);
 
-PkErrorEnum	 pk_error_enum_from_text		(const gchar	*code);
-const gchar	*pk_error_enum_to_text			(PkErrorEnum code);
+PkErrorEnum	 pk_error_enum_from_string		(const gchar	*code);
+const gchar	*pk_error_enum_to_string			(PkErrorEnum code);
 
-PkRestartEnum	 pk_restart_enum_from_text		(const gchar	*restart);
-const gchar	*pk_restart_enum_to_text		(PkRestartEnum	 restart);
+PkRestartEnum	 pk_restart_enum_from_string		(const gchar	*restart);
+const gchar	*pk_restart_enum_to_string		(PkRestartEnum	 restart);
 
-PkMessageEnum	 pk_message_enum_from_text		(const gchar	*message);
-const gchar	*pk_message_enum_to_text		(PkMessageEnum	 message);
+PkMessageEnum	 pk_message_enum_from_string		(const gchar	*message);
+const gchar	*pk_message_enum_to_string		(PkMessageEnum	 message);
 
-PkGroupEnum	 pk_group_enum_from_text		(const gchar	*group);
-const gchar	*pk_group_enum_to_text			(PkGroupEnum	 group);
+PkGroupEnum	 pk_group_enum_from_string		(const gchar	*group);
+const gchar	*pk_group_enum_to_string			(PkGroupEnum	 group);
 
-PkFilterEnum	 pk_filter_enum_from_text		(const gchar	*filter);
-const gchar	*pk_filter_enum_to_text			(PkFilterEnum	 filter);
+PkFilterEnum	 pk_filter_enum_from_string		(const gchar	*filter);
+const gchar	*pk_filter_enum_to_string			(PkFilterEnum	 filter);
 
-PkProvidesEnum	 pk_provides_enum_from_text		(const gchar	*provides);
-const gchar	*pk_provides_enum_to_text		(PkProvidesEnum	 provides);
+PkProvidesEnum	 pk_provides_enum_from_string		(const gchar	*provides);
+const gchar	*pk_provides_enum_to_string		(PkProvidesEnum	 provides);
 
-PkLicenseEnum	 pk_license_enum_from_text		(const gchar	*license);
-const gchar	*pk_license_enum_to_text		(PkLicenseEnum	 license);
+PkLicenseEnum	 pk_license_enum_from_string		(const gchar	*license);
+const gchar	*pk_license_enum_to_string		(PkLicenseEnum	 license);
 
-PkDistroUpgradeEnum pk_distro_upgrade_enum_from_text	(const gchar	*upgrade);
-const gchar	*pk_distro_upgrade_enum_to_text		(PkDistroUpgradeEnum upgrade);
+PkDistroUpgradeEnum pk_distro_upgrade_enum_from_string	(const gchar	*upgrade);
+const gchar	*pk_distro_upgrade_enum_to_string		(PkDistroUpgradeEnum upgrade);
 
-PkMediaTypeEnum  pk_media_type_enum_from_text		(const gchar	*media_type);
-const gchar	*pk_media_type_enum_to_text		(PkMediaTypeEnum media_type);
+PkMediaTypeEnum  pk_media_type_enum_from_string		(const gchar	*media_type);
+const gchar	*pk_media_type_enum_to_string		(PkMediaTypeEnum media_type);
 
-PkAuthorizeEnum  pk_authorize_type_enum_from_text	(const gchar	*auth_type);
-const gchar	*pk_authorize_type_enum_to_text		(PkAuthorizeEnum auth_type);
+PkAuthorizeEnum  pk_authorize_type_enum_from_string	(const gchar	*auth_type);
+const gchar	*pk_authorize_type_enum_to_string		(PkAuthorizeEnum auth_type);
 
 G_END_DECLS
 
