@@ -2,21 +2,21 @@
  *
  * Copyright (C) 2009 Richard Hughes <richard@hughsie.com>
  *
- * Licensed under the GNU General Public License Version 2
+ * Licensed under the GNU Lesser General Public License Version 2.1
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
  */
 
 /**
@@ -38,6 +38,7 @@
 #include <packagekit-glib2/pk-common.h>
 #include <packagekit-glib2/pk-enum.h>
 #include <packagekit-glib2/pk-results.h>
+#include <packagekit-glib2/pk-package-id.h>
 
 #include "egg-debug.h"
 
@@ -63,6 +64,21 @@ enum {
 
 G_DEFINE_TYPE (PkPackageSack, pk_package_sack, G_TYPE_OBJECT)
 
+/**
+ * pk_package_sack_clear:
+ * @sack: a valid #PkPackageSack instance
+ *
+ * Empty all the packages from the sack
+ *
+ * Since: 0.5.2
+ **/
+void
+pk_package_sack_clear (PkPackageSack *sack)
+{
+	g_return_if_fail (PK_IS_PACKAGE_SACK (sack));
+	g_ptr_array_set_size (sack->priv->array, 0);
+}
+
 /**
  * pk_package_sack_get_size:
  * @sack: a valid #PkPackageSack instance
@@ -70,6 +86,8 @@ G_DEFINE_TYPE (PkPackageSack, pk_package_sack, G_TYPE_OBJECT)
  * Gets the number of packages in the sack
  *
  * Return value: the number of packages in the sack
+ *
+ * Since: 0.5.2
  **/
 guint
 pk_package_sack_get_size (PkPackageSack *sack)
@@ -79,19 +97,6 @@ pk_package_sack_get_size (PkPackageSack *sack)
 	return sack->priv->array->len;
 }
 
-/**
- * pk_package_sack_clear:
- * @sack: a valid #PkPackageSack instance
- *
- * Empty all the packages from the sack
- **/
-void
-pk_package_sack_clear (PkPackageSack *sack)
-{
-	g_return_if_fail (PK_IS_PACKAGE_SACK (sack));
-	g_ptr_array_set_size (sack->priv->array, 0);
-}
-
 /**
  * pk_package_sack_get_ids:
  * @sack: a valid #PkPackageSack instance
@@ -99,6 +104,8 @@ pk_package_sack_clear (PkPackageSack *sack)
  * Returns all the Package IDs in the sack
  *
  * Return value: the number of packages in the sack, free with g_strfreev()
+ *
+ * Since: 0.5.3
  **/
 gchar **
 pk_package_sack_get_ids (PkPackageSack *sack)
@@ -120,29 +127,93 @@ pk_package_sack_get_ids (PkPackageSack *sack)
 }
 
 /**
- * pk_package_sack_get_index:
+ * pk_package_sack_get_array:
  * @sack: a valid #PkPackageSack instance
- * @i: the instance to get
  *
- * Gets a packages from the sack
+ * Gets the package array from the sack
  *
- * Return value: a %PkPackage instance
+ * Return value: a #GPtrArray, free with g_ptr_array_unref()
+ *
+ * Since: 0.6.1
  **/
-PkPackage *
-pk_package_sack_get_index (PkPackageSack *sack, guint i)
+GPtrArray *
+pk_package_sack_get_array (PkPackageSack *sack)
 {
-	PkPackage *package = NULL;
+	g_return_val_if_fail (PK_IS_PACKAGE_SACK (sack), NULL);
+	return g_ptr_array_ref (sack->priv->array);
+}
+
+/**
+ * pk_package_sack_filter_by_info:
+ * @sack: a valid #PkPackageSack instance
+ * @info: a %PkInfoEnum value to match
+ *
+ * Returns a new package sack which only matches packages that match the
+ * specified info enum value.
+ *
+ * Return value: a new #PkPackageSack, free with g_object_unref()
+ *
+ * Since: 0.6.2
+ **/
+PkPackageSack *
+pk_package_sack_filter_by_info (PkPackageSack *sack, PkInfoEnum info)
+{
+	PkPackageSack *results;
+	PkPackage *package;
+	PkInfoEnum info_tmp;
+	guint i;
+	PkPackageSackPrivate *priv = sack->priv;
 
 	g_return_val_if_fail (PK_IS_PACKAGE_SACK (sack), NULL);
 
-	/* index invalid */
-	if (i >= sack->priv->array->len)
-		goto out;
+	/* create new sack */
+	results = pk_package_sack_new ();
 
-	/* get object */
-	package = g_object_ref (g_ptr_array_index (sack->priv->array, i));
-out:
-	return package;
+	/* add each that matches the info enum */
+	for (i = 0; i < priv->array->len; i++) {
+		package = g_ptr_array_index (priv->array, i);
+		info_tmp = pk_package_get_info (package);
+		if (info_tmp == info)
+			pk_package_sack_add_package (results, package);
+	}
+
+	return results;
+}
+
+/**
+ * pk_package_sack_filter:
+ * @sack: a valid #PkPackageSack instance
+ * @filter_cb: a #PkPackageSackFilterFunc, which returns %TRUE for the #PkPackage's to add
+ * @user_data: user data to pass to @filter_cb
+ *
+ * Returns a new package sack which only matches packages that return %TRUE
+ * from the filter function.
+ *
+ * Return value: a new #PkPackageSack, free with g_object_unref()
+ *
+ * Since: 0.6.3
+ **/
+PkPackageSack *
+pk_package_sack_filter (PkPackageSack *sack, PkPackageSackFilterFunc filter_cb, gpointer user_data)
+{
+	PkPackageSack *results;
+	PkPackage *package;
+	guint i;
+	PkPackageSackPrivate *priv = sack->priv;
+
+	g_return_val_if_fail (PK_IS_PACKAGE_SACK (sack), NULL);
+	g_return_val_if_fail (filter_cb != NULL, NULL);
+
+	/* create new sack */
+	results = pk_package_sack_new ();
+
+	/* add each that matches the info enum */
+	for (i = 0; i < priv->array->len; i++) {
+		package = g_ptr_array_index (priv->array, i);
+		if (filter_cb (package, user_data))
+			pk_package_sack_add_package (results, package);
+	}
+	return results;
 }
 
 /**
@@ -153,6 +224,8 @@ out:
  * Adds a package to the sack.
  *
  * Return value: %TRUE if the package was added to the sack
+ *
+ * Since: 0.5.2
  **/
 gboolean
 pk_package_sack_add_package (PkPackageSack *sack, PkPackage *package)
@@ -175,6 +248,8 @@ pk_package_sack_add_package (PkPackageSack *sack, PkPackage *package)
  * Adds a package reference to the sack.
  *
  * Return value: %TRUE if the package was added to the sack
+ *
+ * Since: 0.5.2
  **/
 gboolean
 pk_package_sack_add_package_by_id (PkPackageSack *sack, const gchar *package_id, GError **error)
@@ -207,6 +282,8 @@ out:
  * Removes a package reference from the sack. The pointers have to match exactly.
  *
  * Return value: %TRUE if the package was removed from the sack
+ *
+ * Since: 0.5.2
  **/
 gboolean
 pk_package_sack_remove_package (PkPackageSack *sack, PkPackage *package)
@@ -231,6 +308,8 @@ pk_package_sack_remove_package (PkPackageSack *sack, PkPackage *package)
  * the search is stopped.
  *
  * Return value: %TRUE if the package was removed from the sack
+ *
+ * Since: 0.5.2
  **/
 gboolean
 pk_package_sack_remove_package_by_id (PkPackageSack *sack, const gchar *package_id)
@@ -258,6 +337,44 @@ pk_package_sack_remove_package_by_id (PkPackageSack *sack, const gchar *package_
 	return ret;
 }
 
+/**
+ * pk_package_sack_remove_by_filter:
+ * @sack: a valid #PkPackageSack instance
+ * @filter_cb: a #PkPackageSackFilterFunc, which returns %TRUE for the #PkPackage's to retain
+ * @user_data: user data to pass to @filter_cb
+ *
+ * Removes from the package sack any packages that return %FALSE from the filter
+ * function.
+ *
+ * Return value: %TRUE if a package was removed from the sack
+ *
+ * Since: 0.6.3
+ **/
+gboolean
+pk_package_sack_remove_by_filter (PkPackageSack *sack, PkPackageSackFilterFunc filter_cb, gpointer user_data)
+{
+	gboolean ret = FALSE;
+	PkPackage *package;
+	gint i;
+	PkPackageSackPrivate *priv = sack->priv;
+
+	g_return_val_if_fail (PK_IS_PACKAGE_SACK (sack), FALSE);
+	g_return_val_if_fail (filter_cb != NULL, FALSE);
+
+	/* add each that matches the info enum */
+	for (i = 0; i < (gint) priv->array->len; i++) {
+		package = g_ptr_array_index (priv->array, i);
+		if (!filter_cb (package, user_data)) {
+			ret = TRUE;
+			pk_package_sack_remove_package (sack, package);
+
+			/* ensure we pick up subsequent matches */
+			i--;
+		}
+	}
+	return ret;
+}
+
 /**
  * pk_package_sack_find_by_id:
  * @sack: a valid #PkPackageSack instance
@@ -267,6 +384,8 @@ pk_package_sack_remove_package_by_id (PkPackageSack *sack, const gchar *package_
  * the search is stopped.
  *
  * Return value: the #PkPackage object, or %NULL if unfound. Free with g_object_unref()
+ *
+ * Since: 0.5.2
  **/
 PkPackage *
 pk_package_sack_find_by_id (PkPackageSack *sack, const gchar *package_id)
@@ -293,6 +412,28 @@ pk_package_sack_find_by_id (PkPackageSack *sack, const gchar *package_id)
 	return package;
 }
 
+/**
+ * pk_package_sack_sort_compare_name_func:
+ **/
+static gint
+pk_package_sack_sort_compare_name_func (PkPackage **a, PkPackage **b)
+{
+	const gchar *package_id1;
+	const gchar *package_id2;
+	gchar **split1;
+	gchar **split2;
+	gint retval;
+
+	package_id1 = pk_package_get_id (*a);
+	package_id2 = pk_package_get_id (*b);
+	split1 = pk_package_id_split (package_id1);
+	split2 = pk_package_id_split (package_id2);
+	retval = g_strcmp0 (split1[PK_PACKAGE_ID_NAME], split2[PK_PACKAGE_ID_NAME]);
+	g_strfreev (split1);
+	g_strfreev (split2);
+	return retval;
+}
+
 /**
  * pk_package_sack_sort_compare_package_id_func:
  **/
@@ -312,17 +453,11 @@ pk_package_sack_sort_compare_package_id_func (PkPackage **a, PkPackage **b)
 static gint
 pk_package_sack_sort_compare_summary_func (PkPackage **a, PkPackage **b)
 {
-	gint retval;
-	gchar *summary1;
-	gchar *summary2;
-
-	g_object_get (*a, "summary", &summary1, NULL);
-	g_object_get (*b, "summary", &summary2, NULL);
-	retval = g_strcmp0 (summary1, summary2);
-
-	g_free (summary1);
-	g_free (summary2);
-	return retval;
+	const gchar *summary1;
+	const gchar *summary2;
+	summary1 = pk_package_get_summary (*a);
+	summary2 = pk_package_get_summary (*b);
+	return g_strcmp0 (summary1, summary2);
 }
 
 /**
@@ -331,12 +466,10 @@ pk_package_sack_sort_compare_summary_func (PkPackage **a, PkPackage **b)
 static gint
 pk_package_sack_sort_compare_info_func (PkPackage **a, PkPackage **b)
 {
-	PkInfoEnum *info1;
-	PkInfoEnum *info2;
-
-	g_object_get (*a, "info", &info1, NULL);
-	g_object_get (*b, "info", &info2, NULL);
-
+	PkInfoEnum info1;
+	PkInfoEnum info2;
+	info1 = pk_package_get_info (*a);
+	info2 = pk_package_get_info (*b);
 	if (info1 == info2)
 		return 0;
 	else if (info1 > info2)
@@ -347,40 +480,24 @@ pk_package_sack_sort_compare_info_func (PkPackage **a, PkPackage **b)
 /**
  * pk_package_sack_sort_package_id:
  * @sack: a valid #PkPackageSack instance
+ * @type: the type of sorting, e.g. #PK_PACKAGE_SACK_SORT_TYPE_NAME
  *
- * Sorts by Package ID
+ * Sorts the package sack
+ *
+ * Since: 0.6.1
  **/
 void
-pk_package_sack_sort_package_id (PkPackageSack *sack)
+pk_package_sack_sort (PkPackageSack *sack, PkPackageSackSortType type)
 {
 	g_return_if_fail (PK_IS_PACKAGE_SACK (sack));
-	g_ptr_array_sort (sack->priv->array, (GCompareFunc) pk_package_sack_sort_compare_package_id_func);
-}
-
-/**
- * pk_package_sack_sort_summary:
- * @sack: a valid #PkPackageSack instance
- *
- * Sorts by summary
- **/
-void
-pk_package_sack_sort_summary (PkPackageSack *sack)
-{
-	g_return_if_fail (PK_IS_PACKAGE_SACK (sack));
-	g_ptr_array_sort (sack->priv->array, (GCompareFunc) pk_package_sack_sort_compare_summary_func);
-}
-
-/**
- * pk_package_sack_sort_info:
- * @sack: a valid #PkPackageSack instance
- *
- * Sorts by PkInfoEnum
- **/
-void
-pk_package_sack_sort_info (PkPackageSack *sack)
-{
-	g_return_if_fail (PK_IS_PACKAGE_SACK (sack));
-	g_ptr_array_sort (sack->priv->array, (GCompareFunc) pk_package_sack_sort_compare_info_func);
+	if (type == PK_PACKAGE_SACK_SORT_TYPE_NAME)
+		g_ptr_array_sort (sack->priv->array, (GCompareFunc) pk_package_sack_sort_compare_name_func);
+	else if (type == PK_PACKAGE_SACK_SORT_TYPE_PACKAGE_ID)
+		g_ptr_array_sort (sack->priv->array, (GCompareFunc) pk_package_sack_sort_compare_package_id_func);
+	else if (type == PK_PACKAGE_SACK_SORT_TYPE_SUMMARY)
+		g_ptr_array_sort (sack->priv->array, (GCompareFunc) pk_package_sack_sort_compare_summary_func);
+	else if (type == PK_PACKAGE_SACK_SORT_TYPE_INFO)
+		g_ptr_array_sort (sack->priv->array, (GCompareFunc) pk_package_sack_sort_compare_info_func);
 }
 
 /**
@@ -390,6 +507,8 @@ pk_package_sack_sort_info (PkPackageSack *sack)
  * Gets the total size of the package sack in bytes.
  *
  * Return value: the size in bytes
+ *
+ * Since: 0.5.2
  **/
 guint64
 pk_package_sack_get_total_bytes (PkPackageSack *sack)
@@ -472,10 +591,10 @@ pk_package_sack_merge_bool_state_finish (PkPackageSackState *state, const GError
 }
 
 /**
- * pk_package_sack_merge_resolve_cb:
+ * pk_package_sack_resolve_cb:
  **/
 static void
-pk_package_sack_merge_resolve_cb (GObject *source_object, GAsyncResult *res, PkPackageSackState *state)
+pk_package_sack_resolve_cb (GObject *source_object, GAsyncResult *res, PkPackageSackState *state)
 {
 	PkClient *client = PK_CLIENT (source_object);
 	GError *error = NULL;
@@ -547,7 +666,7 @@ out:
 }
 
 /**
- * pk_package_sack_merge_resolve_async:
+ * pk_package_sack_resolve_async:
  * @sack: a valid #PkPackageSack instance
  * @cancellable: a #GCancellable or %NULL
  * @progress_callback: the function to run when the progress changes
@@ -556,9 +675,11 @@ out:
  * @user_data: the data to pass to @callback
  *
  * Merges in details about packages using resolve.
+ *
+ * Since: 0.5.2
  **/
 void
-pk_package_sack_merge_resolve_async (PkPackageSack *sack, GCancellable *cancellable,
+pk_package_sack_resolve_async (PkPackageSack *sack, GCancellable *cancellable,
 				     PkProgressCallback progress_callback, gpointer progress_user_data,
 				     GAsyncReadyCallback callback, gpointer user_data)
 {
@@ -569,7 +690,7 @@ pk_package_sack_merge_resolve_async (PkPackageSack *sack, GCancellable *cancella
 	g_return_if_fail (PK_IS_PACKAGE_SACK (sack));
 	g_return_if_fail (callback != NULL);
 
-	res = g_simple_async_result_new (G_OBJECT (sack), callback, user_data, pk_package_sack_merge_resolve_async);
+	res = g_simple_async_result_new (G_OBJECT (sack), callback, user_data, pk_package_sack_resolve_async);
 
 	/* save state */
 	state = g_slice_new0 (PkPackageSackState);
@@ -584,7 +705,7 @@ pk_package_sack_merge_resolve_async (PkPackageSack *sack, GCancellable *cancella
 	package_ids = pk_package_sack_get_package_ids (sack);
 	pk_client_resolve_async (sack->priv->client, pk_bitfield_value (PK_FILTER_ENUM_INSTALLED), package_ids,
 				 cancellable, progress_callback, progress_user_data,
-				 (GAsyncReadyCallback) pk_package_sack_merge_resolve_cb, state);
+				 (GAsyncReadyCallback) pk_package_sack_resolve_cb, state);
 	g_strfreev (package_ids);
 	g_object_unref (res);
 }
@@ -598,6 +719,8 @@ pk_package_sack_merge_resolve_async (PkPackageSack *sack, GCancellable *cancella
  * Gets the result from the asynchronous function.
  *
  * Return value: %TRUE for success
+ *
+ * Since: 0.5.2
  **/
 gboolean
 pk_package_sack_merge_generic_finish (PkPackageSack *sack, GAsyncResult *res, GError **error)
@@ -619,10 +742,10 @@ pk_package_sack_merge_generic_finish (PkPackageSack *sack, GAsyncResult *res, GE
 /***************************************************************************************************/
 
 /**
- * pk_package_sack_merge_details_cb:
+ * pk_package_sack_get_details_cb:
  **/
 static void
-pk_package_sack_merge_details_cb (GObject *source_object, GAsyncResult *res, PkPackageSackState *state)
+pk_package_sack_get_details_cb (GObject *source_object, GAsyncResult *res, PkPackageSackState *state)
 {
 	PkClient *client = PK_CLIENT (source_object);
 	GError *error = NULL;
@@ -705,7 +828,7 @@ out:
 }
 
 /**
- * pk_package_sack_merge_details_async:
+ * pk_package_sack_get_details_async:
  * @sack: a valid #PkPackageSack instance
  * @cancellable: a #GCancellable or %NULL
  * @progress_callback: the function to run when the progress changes
@@ -716,9 +839,9 @@ out:
  * Merges in details about packages.
  **/
 void
-pk_package_sack_merge_details_async (PkPackageSack *sack, GCancellable *cancellable,
-				     PkProgressCallback progress_callback, gpointer progress_user_data,
-				     GAsyncReadyCallback callback, gpointer user_data)
+pk_package_sack_get_details_async (PkPackageSack *sack, GCancellable *cancellable,
+				   PkProgressCallback progress_callback, gpointer progress_user_data,
+				   GAsyncReadyCallback callback, gpointer user_data)
 {
 	GSimpleAsyncResult *res;
 	PkPackageSackState *state;
@@ -727,7 +850,7 @@ pk_package_sack_merge_details_async (PkPackageSack *sack, GCancellable *cancella
 	g_return_if_fail (PK_IS_PACKAGE_SACK (sack));
 	g_return_if_fail (callback != NULL);
 
-	res = g_simple_async_result_new (G_OBJECT (sack), callback, user_data, pk_package_sack_merge_details_async);
+	res = g_simple_async_result_new (G_OBJECT (sack), callback, user_data, pk_package_sack_get_details_async);
 
 	/* save state */
 	state = g_slice_new0 (PkPackageSackState);
@@ -742,7 +865,7 @@ pk_package_sack_merge_details_async (PkPackageSack *sack, GCancellable *cancella
 	package_ids = pk_package_sack_get_package_ids (sack);
 	pk_client_get_details_async (sack->priv->client, package_ids,
 				     cancellable, progress_callback, progress_user_data,
-				     (GAsyncReadyCallback) pk_package_sack_merge_details_cb, state);
+				     (GAsyncReadyCallback) pk_package_sack_get_details_cb, state);
 
 	g_strfreev (package_ids);
 	g_object_unref (res);
@@ -751,10 +874,10 @@ pk_package_sack_merge_details_async (PkPackageSack *sack, GCancellable *cancella
 /***************************************************************************************************/
 
 /**
- * pk_package_sack_merge_update_detail_cb:
+ * pk_package_sack_get_update_detail_cb:
  **/
 static void
-pk_package_sack_merge_update_detail_cb (GObject *source_object, GAsyncResult *res, PkPackageSackState *state)
+pk_package_sack_get_update_detail_cb (GObject *source_object, GAsyncResult *res, PkPackageSackState *state)
 {
 	PkClient *client = PK_CLIENT (source_object);
 	GError *error = NULL;
@@ -860,7 +983,7 @@ out:
 }
 
 /**
- * pk_package_sack_merge_update_detail_async:
+ * pk_package_sack_get_update_detail_async:
  * @sack: a valid #PkPackageSack instance
  * @cancellable: a #GCancellable or %NULL
  * @progress_callback: the function to run when the progress changes
@@ -869,11 +992,13 @@ out:
  * @user_data: the data to pass to @callback
  *
  * Merges in update details about packages.
+ *
+ * Since: 0.5.2
  **/
 void
-pk_package_sack_merge_update_detail_async (PkPackageSack *sack, GCancellable *cancellable,
-					   PkProgressCallback progress_callback, gpointer progress_user_data,
-					   GAsyncReadyCallback callback, gpointer user_data)
+pk_package_sack_get_update_detail_async (PkPackageSack *sack, GCancellable *cancellable,
+					 PkProgressCallback progress_callback, gpointer progress_user_data,
+					 GAsyncReadyCallback callback, gpointer user_data)
 {
 	GSimpleAsyncResult *res;
 	PkPackageSackState *state;
@@ -882,7 +1007,7 @@ pk_package_sack_merge_update_detail_async (PkPackageSack *sack, GCancellable *ca
 	g_return_if_fail (PK_IS_PACKAGE_SACK (sack));
 	g_return_if_fail (callback != NULL);
 
-	res = g_simple_async_result_new (G_OBJECT (sack), callback, user_data, pk_package_sack_merge_update_detail_async);
+	res = g_simple_async_result_new (G_OBJECT (sack), callback, user_data, pk_package_sack_get_update_detail_async);
 
 	/* save state */
 	state = g_slice_new0 (PkPackageSackState);
@@ -897,7 +1022,7 @@ pk_package_sack_merge_update_detail_async (PkPackageSack *sack, GCancellable *ca
 	package_ids = pk_package_sack_get_package_ids (sack);
 	pk_client_get_update_detail_async (sack->priv->client, package_ids,
 					   cancellable, progress_callback, progress_user_data,
-					   (GAsyncReadyCallback) pk_package_sack_merge_update_detail_cb, state);
+					   (GAsyncReadyCallback) pk_package_sack_get_update_detail_cb, state);
 
 	g_strfreev (package_ids);
 	g_object_unref (res);
@@ -965,6 +1090,8 @@ pk_package_sack_finalize (GObject *object)
  * pk_package_sack_new:
  *
  * Return value: a new PkPackageSack object.
+ *
+ * Since: 0.5.2
  **/
 PkPackageSack *
 pk_package_sack_new (void)
@@ -973,201 +1100,3 @@ pk_package_sack_new (void)
 	sack = g_object_new (PK_TYPE_PACKAGE_SACK, NULL);
 	return PK_PACKAGE_SACK (sack);
 }
-
-/***************************************************************************
- ***                          MAKE CHECK TESTS                           ***
- ***************************************************************************/
-#ifdef EGG_TEST
-#include "egg-test.h"
-
-static void
-pk_package_sack_test_resolve_cb (GObject *object, GAsyncResult *res, EggTest *test)
-{
-	PkPackageSack *sack = PK_PACKAGE_SACK (object);
-	GError *error = NULL;
-	gboolean ret;
-
-	/* get the result */
-	ret = pk_package_sack_merge_generic_finish (sack, res, &error);
-	if (!ret) {
-		egg_test_failed (test, "failed to merge resolve: %s", error->message);
-		g_error_free (error);
-		return;
-	}
-
-	egg_test_loop_quit (test);
-}
-
-static void
-pk_package_sack_test_details_cb (GObject *object, GAsyncResult *res, EggTest *test)
-{
-	PkPackageSack *sack = PK_PACKAGE_SACK (object);
-	GError *error = NULL;
-	gboolean ret;
-
-	/* get the result */
-	ret = pk_package_sack_merge_generic_finish (sack, res, &error);
-	if (!ret) {
-		egg_test_failed (test, "failed to merge details: %s", error->message);
-		g_error_free (error);
-		return;
-	}
-
-	egg_test_loop_quit (test);
-}
-
-static void
-pk_package_sack_test_update_detail_cb (GObject *object, GAsyncResult *res, EggTest *test)
-{
-	PkPackageSack *sack = PK_PACKAGE_SACK (object);
-	GError *error = NULL;
-	gboolean ret;
-
-	/* get the result */
-	ret = pk_package_sack_merge_generic_finish (sack, res, &error);
-	if (!ret) {
-		egg_test_failed (test, "failed to merge update detail: %s", error->message);
-		g_error_free (error);
-		return;
-	}
-
-	egg_test_loop_quit (test);
-}
-
-void
-pk_package_sack_test (gpointer user_data)
-{
-	EggTest *test = (EggTest *) user_data;
-	gboolean ret;
-	PkPackageSack *sack;
-	PkPackage *package;
-	gchar *text;
-	guint size;
-	PkInfoEnum info = PK_INFO_ENUM_UNKNOWN;
-	guint64 bytes;
-
-	if (!egg_test_start (test, "PkPackageSack"))
-		return;
-
-	/************************************************************/
-	egg_test_title (test, "get package_sack");
-	sack = pk_package_sack_new ();
-	egg_test_assert (test, sack != NULL);
-
-	/************************************************************/
-	egg_test_title (test, "get size of unused package sack");
-	size = pk_package_sack_get_size (sack);
-	egg_test_assert (test, (size == 0));
-
-	/************************************************************/
-	egg_test_title (test, "remove package not present");
-	ret = pk_package_sack_remove_package_by_id (sack, "powertop;1.8-1.fc8;i386;fedora");
-	egg_test_assert (test, !ret);
-
-	/************************************************************/
-	egg_test_title (test, "find package not present");
-	package = pk_package_sack_find_by_id (sack, "powertop;1.8-1.fc8;i386;fedora");
-	egg_test_assert (test, (package == NULL));
-
-	/************************************************************/
-	egg_test_title (test, "add package");
-	ret = pk_package_sack_add_package_by_id (sack, "powertop;1.8-1.fc8;i386;fedora", NULL);
-	egg_test_assert (test, ret);
-
-	/************************************************************/
-	egg_test_title (test, "get size of package sack");
-	size = pk_package_sack_get_size (sack);
-	egg_test_assert (test, (size == 1));
-
-	/************************************************************/
-	egg_test_title (test, "merge resolve results");
-	pk_package_sack_merge_resolve_async (sack, NULL, NULL, NULL, (GAsyncReadyCallback) pk_package_sack_test_resolve_cb, test);
-	egg_test_loop_wait (test, 5000);
-	egg_test_success (test, "resolved in %i", egg_test_elapsed (test));
-
-	/************************************************************/
-	egg_test_title (test, "find package which is present");
-	package = pk_package_sack_find_by_id (sack, "powertop;1.8-1.fc8;i386;fedora");
-	egg_test_assert (test, (package != NULL));
-
-	/************************************************************/
-	egg_test_title (test, "check new summary");
-	g_object_get (package,
-		      "info", &info,
-		      "summary", &text,
-		      NULL);
-	egg_test_assert (test, (g_strcmp0 (text, "Power consumption monitor") == 0));
-
-	/************************************************************/
-	egg_test_title (test, "check new info");
-	egg_test_assert (test, (info == PK_INFO_ENUM_INSTALLED));
-
-	g_free (text);
-	g_object_unref (package);
-
-	/************************************************************/
-	egg_test_title (test, "merge details results");
-	pk_package_sack_merge_details_async (sack, NULL, NULL, NULL, (GAsyncReadyCallback) pk_package_sack_test_details_cb, test);
-	egg_test_loop_wait (test, 5000);
-	egg_test_success (test, "got details in %i", egg_test_elapsed (test));
-
-	/************************************************************/
-	egg_test_title (test, "find package which is present");
-	package = pk_package_sack_find_by_id (sack, "powertop;1.8-1.fc8;i386;fedora");
-	egg_test_assert (test, (package != NULL));
-
-	/************************************************************/
-	egg_test_title (test, "check new url");
-	g_object_get (package,
-		      "url", &text,
-		      NULL);
-	egg_test_assert (test, (g_strcmp0 (text, "http://live.gnome.org/powertop") == 0));
-	g_object_unref (package);
-	g_free (text);
-
-	/************************************************************/
-	egg_test_title (test, "merge update detail results");
-	pk_package_sack_merge_update_detail_async (sack, NULL, NULL, NULL, (GAsyncReadyCallback) pk_package_sack_test_update_detail_cb, test);
-	egg_test_loop_wait (test, 5000);
-	egg_test_success (test, "got update detail in %i", egg_test_elapsed (test));
-
-	/************************************************************/
-	egg_test_title (test, "find package which is present");
-	package = pk_package_sack_find_by_id (sack, "powertop;1.8-1.fc8;i386;fedora");
-	egg_test_assert (test, (package != NULL));
-
-	/************************************************************/
-	egg_test_title (test, "check new vendor url");
-	g_object_get (package,
-		      "update-vendor-url", &text,
-		      NULL);
-	egg_test_assert (test, (g_strcmp0 (text, "http://www.distro-update.org/page?moo;Bugfix release for powertop") == 0));
-
-	g_free (text);
-	g_object_unref (package);
-
-	/************************************************************/
-	egg_test_title (test, "chck size in bytes");
-	bytes = pk_package_sack_get_total_bytes (sack);
-	egg_test_assert (test, (bytes == 103424));
-
-	/************************************************************/
-	egg_test_title (test, "remove package");
-	ret = pk_package_sack_remove_package_by_id (sack, "powertop;1.8-1.fc8;i386;fedora");
-	egg_test_assert (test, ret);
-
-	/************************************************************/
-	egg_test_title (test, "get size of package sack");
-	size = pk_package_sack_get_size (sack);
-	egg_test_assert (test, (size == 0));
-
-	/************************************************************/
-	egg_test_title (test, "remove already removed package");
-	ret = pk_package_sack_remove_package_by_id (sack, "powertop;1.8-1.fc8;i386;fedora");
-	egg_test_assert (test, !ret);
-
-	g_object_unref (sack);
-	egg_test_end (test);
-}
-#endif
-
