@@ -2,21 +2,21 @@
  *
  * Copyright (C) 2009 Richard Hughes <richard@hughsie.com>
  *
- * Licensed under the GNU General Public License Version 2
+ * Licensed under the GNU Lesser General Public License Version 2.1
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
  */
 
 /**
@@ -52,6 +52,7 @@ struct _PkPackagePrivate
 {
 	PkInfoEnum		 info;
 	gchar			*package_id;
+	gchar			**package_id_split;
 	gchar			*summary;
 	gchar			*license;
 	PkGroupEnum		 group;
@@ -102,7 +103,7 @@ enum {
 
 static guint signals [SIGNAL_LAST] = { 0 };
 
-G_DEFINE_TYPE (PkPackage, pk_package, G_TYPE_OBJECT)
+G_DEFINE_TYPE (PkPackage, pk_package, PK_TYPE_SOURCE)
 
 /**
  * pk_package_equal:
@@ -112,6 +113,8 @@ G_DEFINE_TYPE (PkPackage, pk_package, G_TYPE_OBJECT)
  * Do the #PkPackage's have the same ID.
  *
  * Return value: %TRUE if the packages have the same package_id, info and summary.
+ *
+ * Since: 0.5.4
  **/
 gboolean
 pk_package_equal (PkPackage *package1, PkPackage *package2)
@@ -131,6 +134,8 @@ pk_package_equal (PkPackage *package1, PkPackage *package2)
  * Do the #PkPackage's have the same ID.
  *
  * Return value: %TRUE if the packages have the same package_id.
+ *
+ * Since: 0.5.4
  **/
 gboolean
 pk_package_equal_id (PkPackage *package1, PkPackage *package2)
@@ -149,6 +154,8 @@ pk_package_equal_id (PkPackage *package1, PkPackage *package2)
  * Sets the package object to have the given ID
  *
  * Return value: %TRUE if the package_id was set
+ *
+ * Since: 0.5.4
  **/
 gboolean
 pk_package_set_id (PkPackage *package, const gchar *package_id, GError **error)
@@ -163,8 +170,7 @@ pk_package_set_id (PkPackage *package, const gchar *package_id, GError **error)
 	/* check valid UTF8 */
 	ret = g_utf8_validate (package_id, -1, NULL);
 	if (!ret) {
-		if (error != NULL)
-			*error = g_error_new (1, 0, "invalid UTF8!");
+		g_set_error_literal (error, 1, 0, "invalid UTF8!");
 		goto out;
 	}
 
@@ -172,21 +178,20 @@ pk_package_set_id (PkPackage *package, const gchar *package_id, GError **error)
 	sections = g_strsplit (package_id, ";", -1);
 	ret = (g_strv_length (sections) == 4);
 	if (!ret) {
-		if (error != NULL)
-			*error = g_error_new (1, 0, "invalid number of sections");
+		g_set_error_literal (error, 1, 0, "invalid number of sections");
 		goto out;
 	}
 
 	/* name has to be valid */
 	ret = (sections[0][0] != '\0');
 	if (!ret) {
-		if (error != NULL)
-			*error = g_error_new (1, 0, "name invalid");
+		g_set_error_literal (error, 1, 0, "name invalid");
 		goto out;
 	}
 
 	/* save */
 	priv->package_id = g_strdup (package_id);
+	priv->package_id_split = g_strdupv (sections);
 out:
 	g_strfreev (sections);
 	return ret;
@@ -199,6 +204,8 @@ out:
  * Gets the package object ID
  *
  * Return value: the %PkInfoEnum
+ *
+ * Since: 0.5.4
  **/
 PkInfoEnum
 pk_package_get_info (PkPackage *package)
@@ -214,6 +221,8 @@ pk_package_get_info (PkPackage *package)
  * Gets the package object ID
  *
  * Return value: the ID, or %NULL if unset
+ *
+ * Since: 0.5.4
  **/
 const gchar *
 pk_package_get_id (PkPackage *package)
@@ -229,6 +238,8 @@ pk_package_get_id (PkPackage *package)
  * Gets the package object ID
  *
  * Return value: the summary, or %NULL if unset
+ *
+ * Since: 0.5.4
  **/
 const gchar *
 pk_package_get_summary (PkPackage *package)
@@ -237,26 +248,95 @@ pk_package_get_summary (PkPackage *package)
 	return package->priv->summary;
 }
 
+/**
+ * pk_package_get_name:
+ * @package: a valid #PkPackage instance
+ *
+ * Gets the package name.
+ *
+ * Return value: the name, or %NULL if unset
+ *
+ * Since: 0.6.4
+ **/
+const gchar *
+pk_package_get_name (PkPackage *package)
+{
+	g_return_val_if_fail (PK_IS_PACKAGE (package), NULL);
+	return package->priv->package_id_split[PK_PACKAGE_ID_NAME];
+}
+
+/**
+ * pk_package_get_version:
+ * @package: a valid #PkPackage instance
+ *
+ * Gets the package version.
+ *
+ * Return value: the version, or %NULL if unset
+ *
+ * Since: 0.6.4
+ **/
+const gchar *
+pk_package_get_version (PkPackage *package)
+{
+	g_return_val_if_fail (PK_IS_PACKAGE (package), NULL);
+	return package->priv->package_id_split[PK_PACKAGE_ID_VERSION];
+}
+
+/**
+ * pk_package_get_arch:
+ * @package: a valid #PkPackage instance
+ *
+ * Gets the package arch.
+ *
+ * Return value: the arch, or %NULL if unset
+ *
+ * Since: 0.6.4
+ **/
+const gchar *
+pk_package_get_arch (PkPackage *package)
+{
+	g_return_val_if_fail (PK_IS_PACKAGE (package), NULL);
+	return package->priv->package_id_split[PK_PACKAGE_ID_ARCH];
+}
+
+/**
+ * pk_package_get_data:
+ * @package: a valid #PkPackage instance
+ *
+ * Gets the package data, which is usually the repository ID that contains the
+ * package. Special ID's include "installed" for installed packages, and "local"
+ * for local packages that exist on disk but not in a repoitory.
+ *
+ * Return value: the data, or %NULL if unset
+ *
+ * Since: 0.6.4
+ **/
+const gchar *
+pk_package_get_data (PkPackage *package)
+{
+	g_return_val_if_fail (PK_IS_PACKAGE (package), NULL);
+	return package->priv->package_id_split[PK_PACKAGE_ID_DATA];
+}
+
 /**
  * pk_package_print:
  * @package: a valid #PkPackage instance
  *
  * Prints details about the package to standard out.
+ *
+ * Since: 0.5.4
  **/
 void
 pk_package_print (PkPackage *package)
 {
-	gchar **parts;
+	PkPackagePrivate *priv = package->priv;
 	g_return_if_fail (PK_IS_PACKAGE (package));
-
-	parts = pk_package_id_split (package->priv->package_id);
 	g_print ("%s-%s.%s\t%s\t%s\n",
-		 parts[PK_PACKAGE_ID_NAME],
-		 parts[PK_PACKAGE_ID_VERSION],
-		 parts[PK_PACKAGE_ID_ARCH],
-		 parts[PK_PACKAGE_ID_DATA],
+		 priv->package_id_split[PK_PACKAGE_ID_NAME],
+		 priv->package_id_split[PK_PACKAGE_ID_VERSION],
+		 priv->package_id_split[PK_PACKAGE_ID_ARCH],
+		 priv->package_id_split[PK_PACKAGE_ID_DATA],
 		 package->priv->summary);
-	g_strfreev (parts);
 }
 
 /**
@@ -434,6 +514,8 @@ pk_package_class_init (PkPackageClass *klass)
 
 	/**
 	 * PkPackage:info:
+	 *
+	 * Since: 0.5.4
 	 */
 	pspec = g_param_spec_uint ("info", NULL,
 				   "The PkInfoEnum package type, e.g. PK_INFO_ENUM_NORMAL",
@@ -443,6 +525,8 @@ pk_package_class_init (PkPackageClass *klass)
 
 	/**
 	 * PkPackage:package-id:
+	 *
+	 * Since: 0.5.4
 	 */
 	pspec = g_param_spec_string ("package-id", NULL,
 				     "The full package_id, e.g. 'gnome-power-manager;0.1.2;i386;fedora'",
@@ -452,6 +536,8 @@ pk_package_class_init (PkPackageClass *klass)
 
 	/**
 	 * PkPackage:summary:
+	 *
+	 * Since: 0.5.4
 	 */
 	pspec = g_param_spec_string ("summary", NULL,
 				     "The package summary",
@@ -461,6 +547,8 @@ pk_package_class_init (PkPackageClass *klass)
 
 	/**
 	 * PkPackage:license:
+	 *
+	 * Since: 0.5.4
 	 */
 	pspec = g_param_spec_string ("license", NULL,
 				     "The package license",
@@ -470,6 +558,8 @@ pk_package_class_init (PkPackageClass *klass)
 
 	/**
 	 * PkPackage:group:
+	 *
+	 * Since: 0.5.4
 	 */
 	pspec = g_param_spec_uint ("group", NULL,
 				   "The package group",
@@ -479,6 +569,8 @@ pk_package_class_init (PkPackageClass *klass)
 
 	/**
 	 * PkPackage:description:
+	 *
+	 * Since: 0.5.4
 	 */
 	pspec = g_param_spec_string ("description", NULL,
 				     "The package description",
@@ -488,6 +580,8 @@ pk_package_class_init (PkPackageClass *klass)
 
 	/**
 	 * PkPackage:url:
+	 *
+	 * Since: 0.5.4
 	 */
 	pspec = g_param_spec_string ("url", NULL,
 				     "The package homepage URL",
@@ -497,6 +591,8 @@ pk_package_class_init (PkPackageClass *klass)
 
 	/**
 	 * PkPackage:size:
+	 *
+	 * Since: 0.5.4
 	 */
 	pspec = g_param_spec_uint64 ("size", NULL,
 				     "The package size",
@@ -506,6 +602,8 @@ pk_package_class_init (PkPackageClass *klass)
 
 	/**
 	 * PkPackage:update-updates:
+	 *
+	 * Since: 0.5.4
 	 */
 	pspec = g_param_spec_string ("update-updates", NULL,
 				     "The update packages",
@@ -515,6 +613,8 @@ pk_package_class_init (PkPackageClass *klass)
 
 	/**
 	 * PkPackage:update-obsoletes:
+	 *
+	 * Since: 0.5.4
 	 */
 	pspec = g_param_spec_string ("update-obsoletes", NULL,
 				     "The update packages that are obsoleted",
@@ -524,6 +624,8 @@ pk_package_class_init (PkPackageClass *klass)
 
 	/**
 	 * PkPackage:update-vendor-url:
+	 *
+	 * Since: 0.5.4
 	 */
 	pspec = g_param_spec_string ("update-vendor-url", NULL,
 				     "The update vendor URL",
@@ -533,6 +635,8 @@ pk_package_class_init (PkPackageClass *klass)
 
 	/**
 	 * PkPackage:update-bugzilla-url:
+	 *
+	 * Since: 0.5.4
 	 */
 	pspec = g_param_spec_string ("update-bugzilla-url", NULL,
 				     "The update bugzilla URL",
@@ -542,6 +646,8 @@ pk_package_class_init (PkPackageClass *klass)
 
 	/**
 	 * PkPackage:update-cve-url:
+	 *
+	 * Since: 0.5.4
 	 */
 	pspec = g_param_spec_string ("update-cve-url", NULL,
 				     "The update CVE URL",
@@ -551,6 +657,8 @@ pk_package_class_init (PkPackageClass *klass)
 
 	/**
 	 * PkPackage:update-restart:
+	 *
+	 * Since: 0.5.4
 	 */
 	pspec = g_param_spec_uint ("update-restart", NULL,
 				   "The update restart type",
@@ -560,6 +668,8 @@ pk_package_class_init (PkPackageClass *klass)
 
 	/**
 	 * PkPackage:update-text:
+	 *
+	 * Since: 0.5.4
 	 */
 	pspec = g_param_spec_string ("update-text", NULL,
 				     "The update description",
@@ -569,6 +679,8 @@ pk_package_class_init (PkPackageClass *klass)
 
 	/**
 	 * PkPackage:update-changelog:
+	 *
+	 * Since: 0.5.4
 	 */
 	pspec = g_param_spec_string ("update-changelog", NULL,
 				     "The update ChangeLog",
@@ -578,6 +690,8 @@ pk_package_class_init (PkPackageClass *klass)
 
 	/**
 	 * PkPackage:update-state:
+	 *
+	 * Since: 0.5.4
 	 */
 	pspec = g_param_spec_uint ("update-state", NULL,
 				   "The update state",
@@ -587,6 +701,8 @@ pk_package_class_init (PkPackageClass *klass)
 
 	/**
 	 * PkPackage:update-issued:
+	 *
+	 * Since: 0.5.4
 	 */
 	pspec = g_param_spec_string ("update-issued", NULL,
 				     "When the update was issued",
@@ -596,6 +712,8 @@ pk_package_class_init (PkPackageClass *klass)
 
 	/**
 	 * PkPackage:update-updated:
+	 *
+	 * Since: 0.5.4
 	 */
 	pspec = g_param_spec_string ("update-updated", NULL,
 				     "When the update was last updated",
@@ -653,6 +771,7 @@ pk_package_finalize (GObject *object)
 	g_free (priv->update_changelog);
 	g_free (priv->update_issued);
 	g_free (priv->update_updated);
+	g_strfreev (priv->package_id_split);
 
 	G_OBJECT_CLASS (pk_package_parent_class)->finalize (object);
 }
@@ -661,6 +780,8 @@ pk_package_finalize (GObject *object)
  * pk_package_new:
  *
  * Return value: a new PkPackage object.
+ *
+ * Since: 0.5.4
  **/
 PkPackage *
 pk_package_new (void)
@@ -669,74 +790,3 @@ pk_package_new (void)
 	package = g_object_new (PK_TYPE_PACKAGE, NULL);
 	return PK_PACKAGE (package);
 }
-
-/***************************************************************************
- ***                          MAKE CHECK TESTS                           ***
- ***************************************************************************/
-#ifdef EGG_TEST
-#include "egg-test.h"
-
-void
-pk_package_test (gpointer user_data)
-{
-	EggTest *test = (EggTest *) user_data;
-	gboolean ret;
-	PkPackage *package;
-	const gchar *id;
-	gchar *text;
-
-	if (!egg_test_start (test, "PkPackage"))
-		return;
-
-	/************************************************************/
-	egg_test_title (test, "get package");
-	package = pk_package_new ();
-	egg_test_assert (test, package != NULL);
-
-	/************************************************************/
-	egg_test_title (test, "get id of unset package");
-	id = pk_package_get_id (package);
-	egg_test_assert (test, (id == NULL));
-
-	/************************************************************/
-	egg_test_title (test, "get id of unset package");
-	g_object_get (package, "package-id", &text, NULL);
-	egg_test_assert (test, (text == NULL));
-	g_free (text);
-
-	/************************************************************/
-	egg_test_title (test, "set invalid id");
-	ret = pk_package_set_id (package, "gnome-power-manager", NULL);
-	egg_test_assert (test, !ret);
-
-	/************************************************************/
-	egg_test_title (test, "set invalid id (sections)");
-	ret = pk_package_set_id (package, "gnome-power-manager;0.1.2;i386", NULL);
-	egg_test_assert (test, !ret);
-
-	/************************************************************/
-	egg_test_title (test, "set invalid name");
-	ret = pk_package_set_id (package, ";0.1.2;i386;fedora", NULL);
-	egg_test_assert (test, !ret);
-
-	/************************************************************/
-	egg_test_title (test, "set valid name");
-	ret = pk_package_set_id (package, "gnome-power-manager;0.1.2;i386;fedora", NULL);
-	egg_test_assert (test, ret);
-
-	/************************************************************/
-	egg_test_title (test, "get id of set package");
-	id = pk_package_get_id (package);
-	egg_test_assert (test, (g_strcmp0 (id, "gnome-power-manager;0.1.2;i386;fedora") == 0));
-
-	/************************************************************/
-	egg_test_title (test, "get name of set package");
-	g_object_get (package, "package-id", &text, NULL);
-	egg_test_assert (test, (g_strcmp0 (text, "gnome-power-manager;0.1.2;i386;fedora") == 0));
-	g_free (text);
-
-	g_object_unref (package);
-	egg_test_end (test);
-}
-#endif
-
