@@ -3,21 +3,21 @@
  * Copyright (C) 2008-2009 Richard Hughes <richard@hughsie.com>
  * Copyright (C) 2008 Shishir Goel <crazyontheedge@gmail.com>
  *
- * Licensed under the GNU General Public License Version 2
+ * Licensed under the GNU Lesser General Public License Version 2.1
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
  */
 
 /**
@@ -98,6 +98,8 @@ G_DEFINE_TYPE (PkServicePack, pk_service_pack, G_TYPE_OBJECT)
  * pk_service_pack_error_quark:
  *
  * Return value: Our personal error quark.
+ *
+ * Since: 0.5.2
  **/
 GQuark
 pk_service_pack_error_quark (void)
@@ -151,7 +153,7 @@ pk_service_pack_check_metadata_file (const gchar *full_path, GError **error)
 	file = g_key_file_new ();
 	ret = g_key_file_load_from_file (file, full_path, G_KEY_FILE_NONE, &error_local);
 	if (!ret) {
-		*error = g_error_new (1, 0, "failed to load file: %s", error_local->message);
+		g_set_error (error, 1, 0, "failed to load file: %s", error_local->message);
 		g_error_free (error_local);
 		goto out;
 	}
@@ -159,7 +161,7 @@ pk_service_pack_check_metadata_file (const gchar *full_path, GError **error)
 	/* read the value */
 	distro_id = g_key_file_get_string (file, PK_SERVICE_PACK_GROUP_NAME, "distro_id", &error_local);
 	if (distro_id == NULL) {
-		*error = g_error_new (1, 0, "failed to get value: %s", error_local->message);
+		g_set_error (error, 1, 0, "failed to get value: %s", error_local->message);
 		g_error_free (error_local);
 		ret = FALSE;
 		goto out;
@@ -168,7 +170,7 @@ pk_service_pack_check_metadata_file (const gchar *full_path, GError **error)
 	/* read the value */
 	type = g_key_file_get_string (file, PK_SERVICE_PACK_GROUP_NAME, "type", &error_local);
 	if (type == NULL) {
-		*error = g_error_new (1, 0, "failed to get type: %s", error_local->message);
+		g_set_error (error, 1, 0, "failed to get type: %s", error_local->message);
 		g_error_free (error_local);
 		ret = FALSE;
 		goto out;
@@ -176,7 +178,7 @@ pk_service_pack_check_metadata_file (const gchar *full_path, GError **error)
 
 	/* check the types we support */
 	if (g_strcmp0 (type, "update") != 0 && g_strcmp0 (type, "install") != 0) {
-		*error = g_error_new (1, 0, "does not have correct type key: %s", type);
+		g_set_error (error, 1, 0, "does not have correct type key: %s", type);
 		ret = FALSE;
 		goto out;
 	}
@@ -198,7 +200,7 @@ pk_service_pack_check_metadata_file (const gchar *full_path, GError **error)
 	/* do we match? */
 	ret = (g_strcmp0 (distro_id_us, distro_id) == 0);
 	if (!ret)
-		*error = g_error_new (1, 0, "distro id did not match %s == %s", distro_id_us, distro_id);
+		g_set_error (error, 1, 0, "distro id did not match %s == %s", distro_id_us, distro_id);
 
 out:
 	if (control != NULL)
@@ -234,7 +236,7 @@ pk_service_pack_extract (const gchar *filename, const gchar *directory, GError *
 	/* save the PWD as we chdir to extract */
 	retcwd = getcwd (buf, PATH_MAX);
 	if (retcwd == NULL) {
-		*error = g_error_new (PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_SETUP,
+		g_set_error_literal (error, PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_SETUP,
 				      "failed to get cwd");
 		goto out;
 	}
@@ -246,7 +248,7 @@ pk_service_pack_extract (const gchar *filename, const gchar *directory, GError *
 	/* open the tar file */
 	r = archive_read_open_file (arch, filename, 10240);
 	if (r) {
-		*error = g_error_new (PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_EXTRACTION,
+		g_set_error (error, PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_EXTRACTION,
 				      "cannot open: %s", archive_error_string (arch));
 		goto out;
 	}
@@ -254,7 +256,7 @@ pk_service_pack_extract (const gchar *filename, const gchar *directory, GError *
 	/* switch to our destination directory */
 	retval = chdir (directory);
 	if (retval != 0) {
-		*error = g_error_new (PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_SETUP,
+		g_set_error (error, PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_SETUP,
 				      "failed chdir to %s", directory);
 		goto out;
 	}
@@ -265,13 +267,13 @@ pk_service_pack_extract (const gchar *filename, const gchar *directory, GError *
 		if (r == ARCHIVE_EOF)
 			break;
 		if (r != ARCHIVE_OK) {
-			*error = g_error_new (PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_EXTRACTION,
+			g_set_error (error, PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_EXTRACTION,
 					      "cannot read header: %s", archive_error_string (arch));
 			goto out;
 		}
 		r = archive_read_extract (arch, entry, 0);
 		if (r != ARCHIVE_OK) {
-			*error = g_error_new (PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_EXTRACTION,
+			g_set_error (error, PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_EXTRACTION,
 					      "cannot extract: %s", archive_error_string (arch));
 			goto out;
 		}
@@ -297,7 +299,7 @@ out:
 static gboolean
 pk_service_pack_extract (const gchar *filename, const gchar *directory, GError **error)
 {
-	*error = g_error_new (PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_EXTRACTION,
+	g_set_error (error, PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_EXTRACTION,
 			      "The service pack %s cannot be extracted as PackageKit was not built with libarchive support", filename);
 	return FALSE;
 }
@@ -369,6 +371,8 @@ pk_service_pack_create_temporary_directory (const gchar *prefix)
  * Checks to see if a service pack file is valid, and usable with this system.
  *
  * Return value: %TRUE if the service pack is valid
+ *
+ * Since: 0.5.2
  **/
 gboolean
 pk_service_pack_check_valid (PkServicePack *pack, const gchar *filename, GError **error)
@@ -387,7 +391,7 @@ pk_service_pack_check_valid (PkServicePack *pack, const gchar *filename, GError
 	directory = pk_service_pack_create_temporary_directory ("PackageKit-");
 	ret = pk_service_pack_extract (filename, directory, &error_local);
 	if (!ret) {
-		*error = g_error_new (PK_SERVICE_PACK_ERROR, error_local->code,
+		g_set_error (error, PK_SERVICE_PACK_ERROR, error_local->code,
 				      "failed to check %s: %s", filename, error_local->message);
 		g_error_free (error_local);
 		goto out;
@@ -396,7 +400,7 @@ pk_service_pack_check_valid (PkServicePack *pack, const gchar *filename, GError
 	/* get the files */
 	dir = g_dir_open (directory, 0, NULL);
 	if (dir == NULL) {
-		*error = g_error_new (PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_SETUP,
+		g_set_error (error, PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_SETUP,
 				      "failed to get directory for %s", directory);
 		ret = FALSE;
 		goto out;
@@ -408,7 +412,7 @@ pk_service_pack_check_valid (PkServicePack *pack, const gchar *filename, GError
 		if (g_strcmp0 (filename_entry, "metadata.conf") == 0) {
 			ret = pk_service_pack_check_metadata_file (metafile, &error_local);
 			if (!ret) {
-				*error = g_error_new (PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_NOT_COMPATIBLE,
+				g_set_error (error, PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_NOT_COMPATIBLE,
 						      "Service Pack %s not compatible with your distro: %s", filename, error_local->message);
 				g_error_free (error_local);
 				ret = FALSE;
@@ -433,6 +437,8 @@ out:
  * Sets the directory to use when decompressing the service pack
  *
  * Return value: %TRUE if the directory was set
+ *
+ * Since: 0.5.2
  **/
 gboolean
 pk_service_pack_set_temp_directory (PkServicePack *pack, const gchar *directory)
@@ -539,7 +545,7 @@ pk_service_pack_archive_add_file (struct archive *arch, const gchar *filename, G
 	/* stat file */
 	retval = stat (filename, &st);
 	if (retval != 0) {
-		*error = g_error_new (PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_CREATE,
+		g_set_error (error, PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_CREATE,
 				      "file not found %s", filename);
 		goto out;
 	}
@@ -558,7 +564,7 @@ pk_service_pack_archive_add_file (struct archive *arch, const gchar *filename, G
 	/* write header */
 	retval = archive_write_header (arch, entry);
 	if (retval != ARCHIVE_OK) {
-		*error = g_error_new (PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_CREATE,
+		g_set_error (error, PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_CREATE,
 				      "failed to write header: %s\n", archive_error_string (arch));
 		goto out;
 	}
@@ -566,7 +572,7 @@ pk_service_pack_archive_add_file (struct archive *arch, const gchar *filename, G
 	/* open file to copy */
 	fd = open (filename, O_RDONLY);
 	if (fd < 0) {
-		*error = g_error_new (PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_CREATE,
+		g_set_error (error, PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_CREATE,
 				      "failed to get fd for %s", filename);
 		goto out;
 	}
@@ -610,7 +616,7 @@ pk_service_pack_create_from_files (PkServicePackState *state, gchar **file_array
 	filename = g_build_filename (g_get_tmp_dir (), "metadata.conf", NULL);
 	ret = pk_service_pack_create_metadata_file (state, filename);
 	if (!ret) {
-		*error = g_error_new (PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_CREATE,
+		g_set_error (error, PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_CREATE,
 				      "failed to generate metadata file %s", filename);
 		goto out;
 	}
@@ -653,7 +659,7 @@ out:
 static gboolean
 pk_service_pack_create_from_files (PkServicePackState *state, gchar **file_array, GError **error)
 {
-	*error = g_error_new (PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_CREATE,
+	g_set_error_literal (error, PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_CREATE,
 			      "The service pack cannot be created as PackageKit was not built with libarchive support");
 	return FALSE;
 }
@@ -868,6 +874,8 @@ out:
  * @user_data: the data to pass to @callback
  *
  * Create a service pack for the specified Package IDs
+ *
+ * Since: 0.5.2
  **/
 void
 pk_service_pack_create_for_package_ids_async (PkServicePack *pack, const gchar *filename, gchar **package_ids,
@@ -969,6 +977,8 @@ out:
  * @user_data: the data to pass to @callback
  *
  * Create a service pack for the specified Package IDs
+ *
+ * Since: 0.5.2
  **/
 void
 pk_service_pack_create_for_updates_async (PkServicePack *pack, const gchar *filename,
@@ -1013,6 +1023,8 @@ pk_service_pack_create_for_updates_async (PkServicePack *pack, const gchar *file
  * Gets the result from the asynchronous function.
  *
  * Return value: %TRUE for success
+ *
+ * Since: 0.5.2
  **/
 gboolean
 pk_service_pack_generic_finish (PkServicePack *pack, GAsyncResult *res, GError **error)
@@ -1076,6 +1088,8 @@ pk_service_pack_init (PkServicePack *pack)
  * pk_service_pack_new:
  *
  * Return value: A new service_pack class instance.
+ *
+ * Since: 0.5.2
  **/
 PkServicePack *
 pk_service_pack_new (void)
@@ -1084,76 +1098,3 @@ pk_service_pack_new (void)
 	pack = g_object_new (PK_TYPE_SERVICE_PACK, NULL);
 	return PK_SERVICE_PACK (pack);
 }
-
-/***************************************************************************
- ***                          MAKE CHECK TESTS                           ***
- ***************************************************************************/
-#ifdef EGG_TEST
-#include "egg-test.h"
-
-static void
-pk_service_pack_test_create_cb (GObject *object, GAsyncResult *res, EggTest *test)
-{
-	PkServicePack *pack = PK_SERVICE_PACK (object);
-	GError *error = NULL;
-	gboolean ret;
-
-	/* get the results */
-	ret = pk_service_pack_generic_finish (pack, res, &error);
-	if (!ret) {
-		egg_test_failed (test, "failed to create pack: %s", error->message);
-		g_error_free (error);
-		goto out;
-	}
-out:
-	egg_test_loop_quit (test);
-}
-
-static void
-pk_service_pack_test_progress_cb (PkProgress *progress, PkProgressType type, EggTest *test)
-{
-	PkStatusEnum status;
-	if (type == PK_PROGRESS_TYPE_STATUS) {
-		g_object_get (progress,
-			      "status", &status,
-			      NULL);
-		egg_debug ("now %s", pk_status_enum_to_text (status));
-	}
-}
-
-void
-pk_service_pack_test (gpointer user_data)
-{
-	EggTest *test = (EggTest *) user_data;
-	PkServicePack *pack;
-	gchar **package_ids;
-
-	if (!egg_test_start (test, "PkServicePack"))
-		return;
-
-	/************************************************************/
-	egg_test_title (test, "get an instance");
-	pack = pk_service_pack_new ();
-	egg_test_assert (test, pack != NULL);
-
-	/************************************************************/
-	egg_test_title (test, "get service_pack");
-	pack = pk_service_pack_new ();
-	egg_test_assert (test, pack != NULL);
-
-	/************************************************************/
-	egg_test_title (test, "install package");
-	package_ids = pk_package_ids_from_id ("glib2;2.14.0;i386;fedora");
-	pk_service_pack_create_for_package_ids_async (pack, "dave.servicepack", package_ids, NULL, NULL,
-				        (PkProgressCallback) pk_service_pack_test_progress_cb, test,
-				        (GAsyncReadyCallback) pk_service_pack_test_create_cb, test);
-	g_strfreev (package_ids);
-	egg_test_loop_wait (test, 150000);
-	egg_test_success (test, "installed in %i", egg_test_elapsed (test));
-
-	g_object_unref (pack);
-
-	egg_test_end (test);
-}
-#endif
-
