@@ -22,13 +22,12 @@
 #define CLIENTPRIVATE_H
 
 #include <QtCore>
-#include <QtSql>
 #include "client.h"
 
+class DaemonProxy;
+
 namespace PackageKit {
 
-class Client;
-class DaemonProxy;
 class Transaction;
 
 class ClientPrivate : public QObject
@@ -38,36 +37,23 @@ class ClientPrivate : public QObject
 public:
 	~ClientPrivate();
 
-	DaemonProxy* daemon;
+	::DaemonProxy* daemon;
 	Client* c;
-
-	QString locale;
 	QStringList hints;
-
-	QMutex runningTransactionsLocker;
 	QHash<QString, Transaction*> runningTransactions;
-
-	// Get a tid, creates a new transaction and sets it up (ie call SetLocale)
-	Transaction* createNewTransaction();
-
 	Client::DaemonError error;
 
-public slots:
-	// org.freedesktop.PackageKit
+    QList<Transaction*> transactions(const QStringList& tids, QObject *parent);
+	void destroyTransaction(const QString &tid);
+
+public Q_SLOTS:
 	void transactionListChanged(const QStringList& tids);
-	// locked
-	void networkStateChanged(const QString& state);
-	// restartScheduled
-	// repoListChanged
-	// updatesChanged
-	void serviceOwnerChanged (const QString&, const QString&, const QString&);
+	void serviceOwnerChanged(const QString&, const QString&, const QString&);
 
 private:
-	friend class Client;
-	ClientPrivate(Client* parent);
-
-private slots:
-	void removeTransactionFromPool(const QString& tid);
+    friend class Client;
+    bool startDaemon;
+    ClientPrivate(Client* parent);
 };
 
 } // End namespace PackageKit
