@@ -56,6 +56,7 @@ class PackageKitBaseBackend:
         self.lang = "C"
         self.has_network = False
         self.background = False
+        self.interactive = False
         self.percentage_old = 0
         self.sub_percentage_old = 0
 
@@ -79,6 +80,13 @@ class PackageKitBaseBackend:
         except KeyError, e:
             print "Error: No BACKGROUND envp"
 
+        # try to get INTERACTIVE state
+        try:
+            if os.environ['INTERACTIVE'] == 'TRUE':
+                self.interactive = True
+        except KeyError, e:
+            print "Error: No INTERACTIVE envp"
+
     def doLock(self):
         ''' Generic locking, overide and extend in child class'''
         self._locked = True
@@ -127,8 +135,11 @@ class PackageKitBaseBackend:
         print "error\t%s\t%s" % (err, description)
         sys.stdout.flush()
         if exit:
-            print "finished"
-            sys.stdout.flush()
+            # Paradoxically, we don't want to print "finished" to stdout here.
+            # Python takes an _enormous_ amount of time to exit, and leaves a
+            # huge race when you try to change a dispatcher because of an error.
+            #
+            # Leave PackageKit to clean up for us in this case.
             sys.exit(1)
 
     def message(self, typ, msg):
@@ -183,7 +194,7 @@ class PackageKitBaseBackend:
         @param repoid: The repo id tag
         @param state: false is repo is disabled else true.
         '''
-        print >> sys.stdout, "repo-detail\t%s\t%s\t%s" % (repoid, name, _bool_to_text(state))
+        print >> sys.stdout, "repo-detail\t%s\t%s\t%s" % (repoid, name, _bool_to_string(state))
         sys.stdout.flush()
 
     def data(self, data):
@@ -291,6 +302,19 @@ class PackageKitBaseBackend:
             )
         sys.stdout.flush()
 
+    def eula_required(self, eula_id, package_id, vendor_name, license_agreement):
+        '''
+        send 'eula-required' signal:
+        @param eula_id:         Id of the EULA
+        @param package_id:      Id of the package needing a signature
+        @param vendor_name:     Name of the vendor that wrote the EULA
+        @param license_agreement: The license text
+        '''
+        print "eula-required\t%s\t%s\t%s\t%s" % (
+            eula_id, package_id, vendor_name, license_agreement
+            )
+        sys.stdout.flush()
+
 #
 # Backend Action Methods
 #
@@ -323,7 +347,7 @@ class PackageKitBaseBackend:
         '''
         self.error(ERROR_NOT_SUPPORTED, "This function is not implemented in this backend", exit=False)
 
-    def get_update_detail(self, package_ids_ids):
+    def get_update_detail(self, package_ids):
         '''
         Implement the {backend}-get-update-detail functionality
         Needed to be implemented in a sub class
@@ -379,14 +403,14 @@ class PackageKitBaseBackend:
         '''
         self.error(ERROR_NOT_SUPPORTED, "This function is not implemented in this backend", exit=False)
 
-    def install_signature(self, sigtype, key_id, package):
+    def install_signature(self, sigtype, key_id, package_id):
         '''
         Implement the {backend}-install-signature functionality
         Needed to be implemented in a sub class
         '''
         self.error(ERROR_NOT_SUPPORTED, "This function is not implemented in this backend", exit=False)
 
-    def install_files (self, only_trusted, inst_files):
+    def install_files(self, only_trusted, inst_files):
         '''
         Implement the {backend}-install_files functionality
         Install the package containing the inst_file file
@@ -394,7 +418,7 @@ class PackageKitBaseBackend:
         '''
         self.error(ERROR_NOT_SUPPORTED, "This function is not implemented in this backend", exit=False)
 
-    def resolve(self, filters, name):
+    def resolve(self, filters, values):
         '''
         Implement the {backend}-resolve functionality
         Needed to be implemented in a sub class
@@ -422,7 +446,7 @@ class PackageKitBaseBackend:
         '''
         self.error(ERROR_NOT_SUPPORTED, "This function is not implemented in this backend", exit=False)
 
-    def get_files(self, package):
+    def get_files(self, package_ids):
         '''
         Implement the {backend}-get-files functionality
         Needed to be implemented in a sub class
@@ -464,7 +488,7 @@ class PackageKitBaseBackend:
         '''
         self.error(ERROR_NOT_SUPPORTED, "This function is not implemented in this backend", exit=False)
 
-    def repo_signature_install(self, package):
+    def repo_signature_install(self, package_id):
         '''
         Implement the {backend}-repo-signature-install functionality
         Needed to be implemented in a sub class
@@ -547,7 +571,7 @@ class PackageKitBaseBackend:
             self.download_packages(directory, package_ids)
             self.finished()
         elif cmd == 'get-depends':
-            filters = args[0]
+            filters = args[0].split(';')
             package_ids = args[1].split(PACKAGE_IDS_DELIM)
             recursive = _text_to_bool(args[2])
             self.get_depends(filters, package_ids, recursive)
@@ -561,15 +585,15 @@ class PackageKitBaseBackend:
             self.get_files(package_ids)
             self.finished()
         elif cmd == 'get-packages':
-            filters = args[0]
+            filters = args[0].split(';')
             self.get_packages(filters)
             self.finished()
         elif cmd == 'get-repo-list':
-            filters = args[0]
+            filters = args[0].split(';')
             self.get_repo_list(filters)
             self.finished()
         elif cmd == 'get-requires':
-            filters = args[0]
+            filters = args[0].split(';')
             package_ids = args[1].split(PACKAGE_IDS_DELIM)
             recursive = _text_to_bool(args[2])
             self.get_requires(filters, package_ids, recursive)
@@ -582,7 +606,7 @@ class PackageKitBaseBackend:
             self.get_distro_upgrades()
             self.finished()
         elif cmd == 'get-updates':
-            filters = args[0]
+            filters = args[0].split(';')
             self.get_updates(filters)
             self.finished()
         elif cmd == 'install-files':
@@ -623,29 +647,29 @@ class PackageKitBaseBackend:
             self.repo_set_data(repoid, para, value)
             self.finished()
         elif cmd == 'resolve':
-            filters = args[0]
+            filters = args[0].split(';')
             package_ids = args[1].split(PACKAGE_IDS_DELIM)
             self.resolve(filters, package_ids)
             self.finished()
         elif cmd == 'search-details':
-            options = args[0]
-            values = _to_unicode(args[1])
-            self.search_details(options, values)
+            filters = args[0].split(';')
+            values = _to_unicode(args[1]).split(PACKAGE_IDS_DELIM)
+            self.search_details(filters, values)
             self.finished()
         elif cmd == 'search-file':
-            options = args[0]
-            values = args[1]
-            self.search_file(options, values)
+            filters = args[0].split(';')
+            values = args[1].split(PACKAGE_IDS_DELIM)
+            self.search_file(filters, values)
             self.finished()
         elif cmd == 'search-group':
-            options = args[0]
-            values = args[1]
-            self.search_group(options, values)
+            filters = args[0].split(';')
+            values = args[1].split(PACKAGE_IDS_DELIM)
+            self.search_group(filters, values)
             self.finished()
         elif cmd == 'search-name':
-            options = args[0]
-            values = _to_unicode(args[1])
-            self.search_name(options, values)
+            filters = args[0].split(';')
+            values = _to_unicode(args[1]).split(PACKAGE_IDS_DELIM)
+            self.search_name(filters, values)
             self.finished()
         elif cmd == 'signature-install':
             package = args[0]
@@ -661,9 +685,9 @@ class PackageKitBaseBackend:
             self.update_system(only_trusted)
             self.finished()
         elif cmd == 'what-provides':
-            filters = args[0]
+            filters = args[0].split(';')
             provides_type = args[1]
-            values = _to_unicode(args[2])
+            values = _to_unicode(args[2]).split(PACKAGE_IDS_DELIM)
             self.what_provides(filters, provides_type, values)
             self.finished()
         elif cmd == 'set-locale':
@@ -720,7 +744,7 @@ def format_string(text, encoding='utf-8'):
     Format a string to be used on stdout for communication with the daemon.
     '''
     if not isinstance(text, unicode):
-        txt = unicode(text, encoding, errors='replace')
+        text = unicode(text, encoding, errors='replace')
     return text.replace("\n", ";")
 
 def _text_to_bool(text):
@@ -729,7 +753,7 @@ def _text_to_bool(text):
         return True
     return False
 
-def _bool_to_text(value):
+def _bool_to_string(value):
     if value:
         return "true"
     return "false"
