@@ -55,12 +55,18 @@ PkBackendSpawn	*pk_backend_spawn_new			(void);
 gboolean	 pk_backend_spawn_helper		(PkBackendSpawn	*backend_spawn,
 							 const gchar	*first_element, ...);
 gboolean	 pk_backend_spawn_kill			(PkBackendSpawn	*backend_spawn);
+gboolean	 pk_backend_spawn_exit			(PkBackendSpawn	*backend_spawn);
 const gchar	*pk_backend_spawn_get_name		(PkBackendSpawn	*backend_spawn);
 gboolean	 pk_backend_spawn_set_name		(PkBackendSpawn	*backend_spawn,
 							 const gchar	*name);
 gboolean	 pk_backend_spawn_set_allow_sigkill	(PkBackendSpawn	*backend_spawn,
 							 gboolean	 allow_sigkill);
 
+PkBackend	*pk_backend_spawn_get_backend		(PkBackendSpawn	*backend_spawn);
+gchar		*pk_backend_spawn_convert_uri		(const gchar	*proxy);
+gboolean	 pk_backend_spawn_inject_data		(PkBackendSpawn *backend_spawn,
+							 const gchar	*line);
+
 /* filtering */
 typedef gboolean (*PkBackendSpawnFilterFunc)		(PkBackend	*backend,
 							 const gchar	*data);
