@@ -230,64 +230,3 @@ pk_conf_new (void)
 	return PK_CONF (pk_conf_object);
 }
 
-/***************************************************************************
- ***                          MAKE CHECK TESTS                           ***
- ***************************************************************************/
-#ifdef EGG_TEST
-#include "egg-test.h"
-
-void
-pk_conf_test (EggTest *test)
-{
-	PkConf *conf;
-	gchar *text;
-	gint value;
-
-	if (!egg_test_start (test, "PkConf"))
-		return;
-
-	/************************************************************/
-	egg_test_title (test, "get an instance");
-	conf = pk_conf_new ();
-	egg_test_assert (test, conf != NULL);
-
-	/************************************************************/
-	egg_test_title (test, "get the default backend");
-	text = pk_conf_get_string (conf, "DefaultBackend");
-	if (text != PK_CONF_VALUE_STRING_MISSING)
-		egg_test_success (test, "got default backend '%s'", text);
-	else
-		egg_test_failed (test, "got NULL!");
-	g_free (text);
-
-	/************************************************************/
-	egg_test_title (test, "get a string that doesn't exist");
-	text = pk_conf_get_string (conf, "FooBarBaz");
-	if (text == PK_CONF_VALUE_STRING_MISSING)
-		egg_test_success (test, "got NULL", text);
-	else
-		egg_test_failed (test, "got return value '%s'", text);
-	g_free (text);
-
-	/************************************************************/
-	egg_test_title (test, "get the shutdown timeout");
-	value = pk_conf_get_int (conf, "ShutdownTimeout");
-	if (value != PK_CONF_VALUE_INT_MISSING)
-		egg_test_success (test, "got ShutdownTimeout '%i'", value);
-	else
-		egg_test_failed (test, "got %i", value);
-
-	/************************************************************/
-	egg_test_title (test, "get an int that doesn't exist");
-	value = pk_conf_get_int (conf, "FooBarBaz");
-	if (value == PK_CONF_VALUE_INT_MISSING)
-		egg_test_success (test, "got %i", value);
-	else
-		egg_test_failed (test, "got return value '%i'", value);
-
-	g_object_unref (conf);
-
-	egg_test_end (test);
-}
-#endif
-
