@@ -72,6 +72,7 @@ struct PkEnginePrivate
 	GTimer			*timer;
 	gboolean		 notify_clients_of_upgrade;
 	gboolean		 shutdown_as_soon_as_possible;
+	gboolean		 using_hardcoded_proxy;
 	PkTransactionList	*transaction_list;
 	PkTransactionDb		*transaction_db;
 	PkCache			*cache;
@@ -98,18 +99,13 @@ struct PkEnginePrivate
 #ifdef USE_SECURITY_POLKIT
 	PolkitAuthority		*authority;
 #endif
-	gchar			*proxy_http;
-	gchar			*proxy_ftp;
-	gchar			*sender;
 	gboolean		 locked;
 	PkNetworkEnum		 network_state;
 };
 
 enum {
-	SIGNAL_LOCKED,
 	SIGNAL_TRANSACTION_LIST_CHANGED,
 	SIGNAL_REPO_LIST_CHANGED,
-	SIGNAL_NETWORK_STATE_CHANGED,
 	SIGNAL_RESTART_SCHEDULE,
 	SIGNAL_UPDATES_CHANGED,
 	SIGNAL_CHANGED,
@@ -170,6 +166,7 @@ pk_engine_error_get_type (void)
 			ENUM_ENTRY (PK_ENGINE_ERROR_INVALID_STATE, "InvalidState"),
 			ENUM_ENTRY (PK_ENGINE_ERROR_REFUSED_BY_POLICY, "RefusedByPolicy"),
 			ENUM_ENTRY (PK_ENGINE_ERROR_CANNOT_SET_PROXY, "CannotSetProxy"),
+			ENUM_ENTRY (PK_ENGINE_ERROR_CANNOT_SET_ROOT, "CannotSetRoot"),
 			ENUM_ENTRY (PK_ENGINE_ERROR_NOT_SUPPORTED, "NotSupported"),
 			ENUM_ENTRY (PK_ENGINE_ERROR_CANNOT_ALLOCATE_TID, "CannotAllocateTid"),
 			ENUM_ENTRY (PK_ENGINE_ERROR_CANNOT_CHECK_AUTH, "CannotCheckAuth"),
@@ -216,8 +213,6 @@ static void
 pk_engine_inhibit_locked_cb (PkInhibit *inhibit, gboolean is_locked, PkEngine *engine)
 {
 	g_return_if_fail (PK_IS_ENGINE (engine));
-	egg_debug ("emitting locked %i", is_locked);
-	g_signal_emit (engine, signals[SIGNAL_LOCKED], 0, is_locked);
 
 	/* already set */
 	if (engine->priv->locked == is_locked)
@@ -302,24 +297,6 @@ out:
 	g_free (sender);
 }
 
-/**
- * pk_engine_get_network_state:
- **/
-gboolean
-pk_engine_get_network_state (PkEngine *engine, gchar **state, GError **error)
-{
-	PkNetworkEnum network;
-	g_return_val_if_fail (PK_IS_ENGINE (engine), FALSE);
-	/* get the network state */
-	network = pk_network_get_network_state (engine->priv->network);
-	*state = g_strdup (pk_network_enum_to_text (network));
-
-	/* reset the timer */
-	pk_engine_reset_timer (engine);
-
-	return TRUE;
-}
-
 /**
  * pk_get_os_release:
  *
@@ -413,6 +390,19 @@ pk_engine_get_distro_id (PkEngine *engine)
 		goto out;
 	}
 
+	/* check for meego */
+	ret = g_file_get_contents ("/etc/meego-release", &contents, NULL, NULL);
+	if (ret) {
+		/* Meego release 1.0 (MeeGo) */
+		split = g_strsplit (contents, " ", 0);
+		if (split == NULL)
+			goto out;
+
+		/* complete! */
+		distro_id = g_strdup_printf ("meego;%s;%s", split[2], arch);
+		goto out;
+	}
+
 	/* check for foresight or foresight derivatives */
 	ret = g_file_get_contents ("/etc/distro-release", &contents, NULL, NULL);
 	if (ret) {
@@ -513,7 +503,8 @@ pk_engine_get_daemon_state (PkEngine *engine, gchar **state, GError **error)
 {
 	g_return_val_if_fail (PK_IS_ENGINE (engine), FALSE);
 
-	*state = pk_transaction_list_get_state (engine->priv->transaction_list);
+	if (state != NULL)
+		*state = pk_transaction_list_get_state (engine->priv->transaction_list);
 
 	/* reset the timer */
 	pk_engine_reset_timer (engine);
@@ -531,7 +522,8 @@ pk_engine_get_transaction_list (PkEngine *engine, gchar ***transaction_list, GEr
 	g_return_val_if_fail (PK_IS_ENGINE (engine), FALSE);
 
 	egg_debug ("GetTransactionList method called");
-	*transaction_list = pk_transaction_list_get_array (engine->priv->transaction_list);
+	if (transaction_list != NULL)
+		*transaction_list = pk_transaction_list_get_array (engine->priv->transaction_list);
 
 	/* reset the timer */
 	pk_engine_reset_timer (engine);
@@ -609,15 +601,23 @@ pk_engine_state_has_changed (PkEngine *engine, const gchar *reason, GError **err
 	if (is_priority && engine->priv->timeout_normal_id != 0) {
 		/* clear normal, as we are about to schedule a priority */
 		g_source_remove (engine->priv->timeout_normal_id);
-		engine->priv->timeout_normal_id = 0;	}
+		engine->priv->timeout_normal_id = 0;
+	}
 
 	/* wait a little delay in case we get multiple requests */
-	if (is_priority)
+	if (is_priority) {
 		engine->priv->timeout_priority_id = g_timeout_add_seconds (engine->priv->timeout_priority,
 									   pk_engine_state_changed_cb, engine);
-	else
+#if GLIB_CHECK_VERSION(2,25,8)
+		g_source_set_name_by_id (engine->priv->timeout_priority_id, "[PkEngine] priority");
+#endif
+	} else {
 		engine->priv->timeout_normal_id = g_timeout_add_seconds (engine->priv->timeout_normal,
 									 pk_engine_state_changed_cb, engine);
+#if GLIB_CHECK_VERSION(2,25,8)
+		g_source_set_name_by_id (engine->priv->timeout_normal_id, "[PkEngine] normal");
+#endif
+	}
 
 	/* reset the timer */
 	pk_engine_reset_timer (engine);
@@ -625,92 +625,6 @@ out:
 	return TRUE;
 }
 
-/**
- * pk_engine_get_actions:
- **/
-gboolean
-pk_engine_get_actions (PkEngine *engine, gchar **roles, GError **error)
-{
-	g_return_val_if_fail (PK_IS_ENGINE (engine), FALSE);
-	*roles = pk_role_bitfield_to_text (engine->priv->roles);
-
-	/* reset the timer */
-	pk_engine_reset_timer (engine);
-
-	return TRUE;
-}
-
-/**
- * pk_engine_get_groups:
- **/
-gboolean
-pk_engine_get_groups (PkEngine *engine, gchar **groups, GError **error)
-{
-	g_return_val_if_fail (PK_IS_ENGINE (engine), FALSE);
-	*groups = pk_group_bitfield_to_text (engine->priv->groups);
-
-	/* reset the timer */
-	pk_engine_reset_timer (engine);
-
-	return TRUE;
-}
-
-/**
- * pk_engine_get_mime_types:
- **/
-gboolean
-pk_engine_get_mime_types (PkEngine *engine, gchar **types, GError **error)
-{
-	g_return_val_if_fail (PK_IS_ENGINE (engine), FALSE);
-
-	/* not compulsory for backends */
-	if (engine->priv->mime_types == NULL) {
-		g_set_error (error, PK_ENGINE_ERROR, PK_ENGINE_ERROR_NOT_SUPPORTED,
-			     "Backend does not provide this information");
-		return FALSE;
-	}
-
-	*types = g_strdup (engine->priv->mime_types);
-
-	/* reset the timer */
-	pk_engine_reset_timer (engine);
-
-	return TRUE;
-}
-
-/**
- * pk_engine_get_filters:
- **/
-gboolean
-pk_engine_get_filters (PkEngine *engine, gchar **filters, GError **error)
-{
-	g_return_val_if_fail (PK_IS_ENGINE (engine), FALSE);
-	*filters = pk_filter_bitfield_to_text (engine->priv->filters);
-
-	/* reset the timer */
-	pk_engine_reset_timer (engine);
-
-	return TRUE;
-}
-
-/**
- * pk_engine_get_backend_detail:
- **/
-gboolean
-pk_engine_get_backend_detail (PkEngine *engine, gchar **name, gchar **author, GError **error)
-{
-	g_return_val_if_fail (PK_IS_ENGINE (engine), FALSE);
-
-	egg_debug ("GetBackendDetail method called");
-	*name = pk_backend_get_description (engine->priv->backend);
-	*author = pk_backend_get_author (engine->priv->backend);
-
-	/* reset the timer */
-	pk_engine_reset_timer (engine);
-
-	return TRUE;
-}
-
 /**
  * pk_engine_get_time_since_action:
  *
@@ -723,7 +637,7 @@ pk_engine_get_time_since_action	(PkEngine *engine, const gchar *role_text, guint
 
 	g_return_val_if_fail (PK_IS_ENGINE (engine), FALSE);
 
-	role = pk_role_enum_from_text (role_text);
+	role = pk_role_enum_from_string (role_text);
 	*seconds = pk_transaction_db_action_time_since (engine->priv->transaction_db, role);
 
 	/* reset the timer */
@@ -797,36 +711,35 @@ pk_engine_suggest_daemon_quit (PkEngine *engine, GError **error)
  * pk_engine_set_proxy_internal:
  **/
 static gboolean
-pk_engine_set_proxy_internal (PkEngine *engine)
+pk_engine_set_proxy_internal (PkEngine *engine, const gchar *sender, const gchar *proxy_http, const gchar *proxy_ftp)
 {
 	gboolean ret;
 	guint uid;
 	gchar *session = NULL;
 
 	/* try to set the new proxy */
-	ret = pk_backend_set_proxy (engine->priv->backend, engine->priv->proxy_http, engine->priv->proxy_ftp);
+	ret = pk_backend_set_proxy (engine->priv->backend, proxy_http, proxy_ftp);
 	if (!ret) {
 		egg_warning ("setting the proxy failed");
 		goto out;
 	}
 
 	/* get uid */
-	uid = pk_dbus_get_uid (engine->priv->dbus, engine->priv->sender);
+	uid = pk_dbus_get_uid (engine->priv->dbus, sender);
 	if (uid == G_MAXUINT) {
 		egg_warning ("failed to get the uid");
 		goto out;
 	}
 
 	/* get session */
-	session = pk_dbus_get_session (engine->priv->dbus, engine->priv->sender);
+	session = pk_dbus_get_session (engine->priv->dbus, sender);
 	if (session == NULL) {
 		egg_warning ("failed to get the session");
 		goto out;
 	}
 
 	/* save to database */
-	ret = pk_transaction_db_set_proxy (engine->priv->transaction_db, uid, session,
-					   engine->priv->proxy_http, engine->priv->proxy_ftp);
+	ret = pk_transaction_db_set_proxy (engine->priv->transaction_db, uid, session, proxy_http, proxy_ftp);
 	if (!ret) {
 		egg_warning ("failed to save the proxy in the database");
 		goto out;
@@ -836,42 +749,81 @@ out:
 	return ret;
 }
 
+#ifdef USE_SECURITY_POLKIT
+typedef struct {
+	DBusGMethodInvocation	*context;
+	PkEngine		*engine;
+	gchar			*sender;
+	gchar			*value1;
+	gchar			*value2;
+} PkEngineDbusState;
+#endif
+
 #ifdef USE_SECURITY_POLKIT
 /**
  * pk_engine_action_obtain_authorization:
  **/
 static void
-pk_engine_action_obtain_authorization_finished_cb (GObject *source_object, GAsyncResult *res, PkEngine *engine)
+pk_engine_action_obtain_proxy_authorization_finished_cb (PolkitAuthority *authority, GAsyncResult *res, PkEngineDbusState *state)
 {
 	PolkitAuthorizationResult *result;
-	GError *error = NULL;
+	GError *error_local = NULL;
+	GError *error;
 	gboolean ret;
+	PkEnginePrivate *priv = state->engine->priv;
 
 	/* finish the call */
-	result = polkit_authority_check_authorization_finish (engine->priv->authority, res, &error);
+	result = polkit_authority_check_authorization_finish (priv->authority, res, &error_local);
 
 	/* failed */
 	if (result == NULL) {
-		egg_warning ("failed to check for auth: %s", error->message);
-		g_error_free (error);
+		error = g_error_new (PK_ENGINE_ERROR, PK_ENGINE_ERROR_CANNOT_SET_PROXY,
+				     "setting the proxy failed, could not check for auth: %s", error_local->message);
+		dbus_g_method_return_error (state->context, error);
+		g_error_free (error_local);
 		goto out;
 	}
 
 	/* did not auth */
 	if (!polkit_authorization_result_get_is_authorized (result)) {
-		egg_warning ("failed to obtain auth");
+		error = g_error_new_literal (PK_ENGINE_ERROR, PK_ENGINE_ERROR_CANNOT_SET_PROXY,
+					     "failed to obtain auth");
+		dbus_g_method_return_error (state->context, error);
+		goto out;
+	}
+
+	/* admin already set value, so silently refuse value */
+	if (priv->using_hardcoded_proxy) {
+		egg_debug ("cannot override admin set proxy");
+		dbus_g_method_return (state->context);
 		goto out;
 	}
 
 	/* try to set the new proxy and save to database */
-	ret = pk_engine_set_proxy_internal (engine);
+	ret = pk_engine_set_proxy_internal (state->engine, state->sender, state->value1, state->value2);
 	if (!ret) {
-		egg_warning ("setting the proxy failed");
+		error = g_error_new_literal (PK_ENGINE_ERROR, PK_ENGINE_ERROR_CANNOT_SET_PROXY,
+					     "setting the proxy failed");
+		dbus_g_method_return_error (state->context, error);
 		goto out;
 	}
+
+	/* only set after the auth success */
+	egg_debug ("changing http proxy to %s for %s", state->value1, state->sender);
+	egg_debug ("changing ftp proxy to %s for %s", state->value2, state->sender);
+
+	/* all okay */
+	dbus_g_method_return (state->context);
 out:
 	if (result != NULL)
 		g_object_unref (result);
+
+	/* unref state, we're done */
+	g_object_unref (state->engine);
+	g_free (state->sender);
+	g_free (state->value1);
+	g_free (state->value2);
+	g_free (state);
 }
 #endif
 
@@ -879,7 +831,7 @@ out:
  * pk_engine_is_proxy_unchanged:
  **/
 static gboolean
-pk_engine_is_proxy_unchanged (PkEngine *engine, const gchar *proxy_http, const gchar *proxy_ftp)
+pk_engine_is_proxy_unchanged (PkEngine *engine, const gchar *sender, const gchar *proxy_http, const gchar *proxy_ftp)
 {
 	guint uid;
 	gboolean ret = FALSE;
@@ -888,16 +840,16 @@ pk_engine_is_proxy_unchanged (PkEngine *engine, const gchar *proxy_http, const g
 	gchar *proxy_ftp_tmp = NULL;
 
 	/* get uid */
-	uid = pk_dbus_get_uid (engine->priv->dbus, engine->priv->sender);
+	uid = pk_dbus_get_uid (engine->priv->dbus, sender);
 	if (uid == G_MAXUINT) {
-		egg_warning ("failed to get the uid for %s", engine->priv->sender);
+		egg_warning ("failed to get the uid for %s", sender);
 		goto out;
 	}
 
 	/* get session */
-	session = pk_dbus_get_session (engine->priv->dbus, engine->priv->sender);
+	session = pk_dbus_get_session (engine->priv->dbus, sender);
 	if (session == NULL) {
-		egg_warning ("failed to get the session for %s", engine->priv->sender);
+		egg_warning ("failed to get the session for %s", sender);
 		goto out;
 	}
 
@@ -926,9 +878,11 @@ pk_engine_set_proxy (PkEngine *engine, const gchar *proxy_http, const gchar *pro
 	guint len;
 	GError *error = NULL;
 	gboolean ret;
+	gchar *sender = NULL;
 #ifdef USE_SECURITY_POLKIT
 	PolkitSubject *subject;
 	PolkitDetails *details;
+	PkEngineDbusState *state;
 #endif
 	g_return_if_fail (PK_IS_ENGINE (engine));
 
@@ -945,7 +899,7 @@ pk_engine_set_proxy (PkEngine *engine, const gchar *proxy_http, const gchar *pro
 	if (len == 1024) {
 		error = g_error_new (PK_ENGINE_ERROR, PK_ENGINE_ERROR_CANNOT_SET_PROXY, "%s", "http proxy was too long");
 		dbus_g_method_return_error (context, error);
-		return;
+		goto out;
 	}
 
 	/* check length of ftp */
@@ -953,36 +907,35 @@ pk_engine_set_proxy (PkEngine *engine, const gchar *proxy_http, const gchar *pro
 	if (len == 1024) {
 		error = g_error_new (PK_ENGINE_ERROR, PK_ENGINE_ERROR_CANNOT_SET_PROXY, "%s", "ftp proxy was too long");
 		dbus_g_method_return_error (context, error);
-		return;
+		goto out;
 	}
 
 	/* save sender */
-	g_free (engine->priv->sender);
-	engine->priv->sender = dbus_g_method_get_sender (context);
+	sender = dbus_g_method_get_sender (context);
 
 	/* is exactly the same proxy? */
-	ret = pk_engine_is_proxy_unchanged (engine, proxy_http, proxy_ftp);
+	ret = pk_engine_is_proxy_unchanged (engine, sender, proxy_http, proxy_ftp);
 	if (ret) {
 		egg_debug ("not changing proxy as the same as before");
 		dbus_g_method_return (context);
-		return;
+		goto out;
 	}
 
-	/* save these so we can set them after the auth success */
-	g_free (engine->priv->proxy_http);
-	g_free (engine->priv->proxy_ftp);
-	engine->priv->proxy_http = g_strdup (proxy_http);
-	engine->priv->proxy_ftp = g_strdup (proxy_ftp);
-	egg_debug ("changing http proxy to %s for %s", proxy_http, engine->priv->sender);
-	egg_debug ("changing ftp proxy to %s for %s", proxy_ftp, engine->priv->sender);
-
 #ifdef USE_SECURITY_POLKIT
 	/* check subject */
-	subject = polkit_system_bus_name_new (engine->priv->sender);
+	subject = polkit_system_bus_name_new (sender);
 
 	/* insert details about the authorization */
 	details = polkit_details_new ();
-	polkit_details_insert (details, "role", pk_role_enum_to_text (PK_ROLE_ENUM_UNKNOWN));
+	polkit_details_insert (details, "role", pk_role_enum_to_string (PK_ROLE_ENUM_UNKNOWN));
+
+	/* cache state */
+	state = g_new0 (PkEngineDbusState, 1);
+	state->context = context;
+	state->engine = g_object_ref (engine);
+	state->sender = g_strdup (sender);
+	state->value1 = g_strdup (proxy_http);
+	state->value2 = g_strdup (proxy_ftp);
 
 	/* do authorization async */
 	polkit_authority_check_authorization (engine->priv->authority, subject,
@@ -990,8 +943,8 @@ pk_engine_set_proxy (PkEngine *engine, const gchar *proxy_http, const gchar *pro
 					      details,
 					      POLKIT_CHECK_AUTHORIZATION_FLAGS_ALLOW_USER_INTERACTION,
 					      NULL,
-					      (GAsyncReadyCallback) pk_engine_action_obtain_authorization_finished_cb,
-					      engine);
+					      (GAsyncReadyCallback) pk_engine_action_obtain_proxy_authorization_finished_cb,
+					      state);
 
 	/* check_authorization ref's this */
 	g_object_unref (details);
@@ -999,15 +952,16 @@ pk_engine_set_proxy (PkEngine *engine, const gchar *proxy_http, const gchar *pro
 	egg_warning ("*** THERE IS NO SECURITY MODEL BEING USED!!! ***");
 
 	/* try to set the new proxy and save to database */
-	ret = pk_engine_set_proxy_internal (engine);
+	ret = pk_engine_set_proxy_internal (engine, sender, proxy_http, proxy_ftp);
 	if (!ret) {
 		error = g_error_new (PK_ENGINE_ERROR, PK_ENGINE_ERROR_CANNOT_SET_PROXY, "%s", "setting the proxy failed");
 		dbus_g_method_return_error (context, error);
-		return;
+		goto out;
 	}
-#endif
+
 	/* all okay */
 	dbus_g_method_return (context);
+#endif
 
 	/* reset the timer */
 	pk_engine_reset_timer (engine);
@@ -1015,6 +969,263 @@ pk_engine_set_proxy (PkEngine *engine, const gchar *proxy_http, const gchar *pro
 #ifdef USE_SECURITY_POLKIT
 	g_object_unref (subject);
 #endif
+out:
+	g_free (sender);
+}
+
+/**
+ * pk_engine_set_root_internal:
+ **/
+static gboolean
+pk_engine_set_root_internal (PkEngine *engine, const gchar *root, const gchar *sender)
+{
+	gboolean ret;
+	guint uid;
+	gchar *session = NULL;
+
+	/* try to set the new root */
+	ret = pk_backend_set_root (engine->priv->backend, root);
+	if (!ret) {
+		egg_warning ("setting the root failed");
+		goto out;
+	}
+
+	/* get uid */
+	uid = pk_dbus_get_uid (engine->priv->dbus, sender);
+	if (uid == G_MAXUINT) {
+		egg_warning ("failed to get the uid");
+		goto out;
+	}
+
+	/* get session */
+	session = pk_dbus_get_session (engine->priv->dbus, sender);
+	if (session == NULL) {
+		egg_warning ("failed to get the session");
+		goto out;
+	}
+
+	/* save to database */
+	ret = pk_transaction_db_set_root (engine->priv->transaction_db, uid, session, root);
+	if (!ret) {
+		egg_warning ("failed to save the root in the database");
+		goto out;
+	}
+out:
+	g_free (session);
+	return ret;
+}
+
+#ifdef USE_SECURITY_POLKIT
+/**
+ * pk_engine_action_obtain_authorization:
+ **/
+static void
+pk_engine_action_obtain_root_authorization_finished_cb (PolkitAuthority *authority, GAsyncResult *res, PkEngineDbusState *state)
+{
+	PolkitAuthorizationResult *result;
+	GError *error_local = NULL;
+	GError *error;
+	gboolean ret;
+	PkEnginePrivate *priv = state->engine->priv;
+
+	/* finish the call */
+	result = polkit_authority_check_authorization_finish (priv->authority, res, &error_local);
+
+	/* failed */
+	if (result == NULL) {
+		error = g_error_new (PK_ENGINE_ERROR, PK_ENGINE_ERROR_CANNOT_SET_PROXY,
+				     "could not check for auth: %s", error_local->message);
+		dbus_g_method_return_error (state->context, error);
+		g_error_free (error_local);
+		goto out;
+	}
+
+	/* did not auth */
+	if (!polkit_authorization_result_get_is_authorized (result)) {
+		error = g_error_new_literal (PK_ENGINE_ERROR, PK_ENGINE_ERROR_CANNOT_SET_PROXY,
+					     "failed to obtain auth");
+		dbus_g_method_return_error (state->context, error);
+		goto out;
+	}
+
+	/* try to set the new root and save to database */
+	ret = pk_engine_set_root_internal (state->engine, state->value1, state->sender);
+	if (!ret) {
+		error = g_error_new_literal (PK_ENGINE_ERROR, PK_ENGINE_ERROR_CANNOT_SET_PROXY,
+					     "setting the root failed");
+		dbus_g_method_return_error (state->context, error);
+		goto out;
+	}
+
+	/* save these so we can set them after the auth success */
+	egg_debug ("changing root to %s for %s", state->value1, state->sender);
+
+	/* all okay */
+	dbus_g_method_return (state->context);
+out:
+	if (result != NULL)
+		g_object_unref (result);
+
+	/* unref state, we're done */
+	g_object_unref (state->engine);
+	g_free (state->sender);
+	g_free (state->value1);
+	g_free (state->value2);
+	g_free (state);
+}
+#endif
+
+/**
+ * pk_engine_is_root_unchanged:
+ **/
+static gboolean
+pk_engine_is_root_unchanged (PkEngine *engine, const gchar *sender, const gchar *root)
+{
+	guint uid;
+	gboolean ret = FALSE;
+	gchar *session = NULL;
+	gchar *root_tmp = NULL;
+
+	/* get uid */
+	uid = pk_dbus_get_uid (engine->priv->dbus, sender);
+	if (uid == G_MAXUINT) {
+		egg_warning ("failed to get the uid for %s", sender);
+		goto out;
+	}
+
+	/* get session */
+	session = pk_dbus_get_session (engine->priv->dbus, sender);
+	if (session == NULL) {
+		egg_warning ("failed to get the session for %s", sender);
+		goto out;
+	}
+
+	/* find out if they are the same as what we tried to set before */
+	ret = pk_transaction_db_get_root (engine->priv->transaction_db, uid, session, &root_tmp);
+	if (!ret)
+		goto out;
+
+	/* are different? */
+	if (g_strcmp0 (root_tmp, root) != 0)
+		ret = FALSE;
+out:
+	g_free (session);
+	g_free (root_tmp);
+	return ret;
+}
+
+/**
+ * pk_engine_set_root:
+ **/
+void
+pk_engine_set_root (PkEngine *engine, const gchar *root, DBusGMethodInvocation *context)
+{
+	guint len;
+	GError *error = NULL;
+	gboolean ret;
+	gchar *sender = NULL;
+#ifdef USE_SECURITY_POLKIT
+	PolkitSubject *subject;
+	PolkitDetails *details;
+	PkEngineDbusState *state;
+#endif
+	g_return_if_fail (PK_IS_ENGINE (engine));
+
+	/* blank is default */
+	if (root == NULL ||
+	    root[0] == '\0')
+		root = "/";
+
+	egg_debug ("SetRoot method called: %s", root);
+
+	/* check length of root */
+	len = egg_strlen (root, 1024);
+	if (len == 1024) {
+		error = g_error_new (PK_ENGINE_ERROR, PK_ENGINE_ERROR_CANNOT_SET_ROOT, "root was too long: %s", root);
+		dbus_g_method_return_error (context, error);
+		goto out;
+	}
+
+	/* check prefix of root */
+	if (root[0] != '/') {
+		error = g_error_new (PK_ENGINE_ERROR, PK_ENGINE_ERROR_CANNOT_SET_ROOT, "root is not absolute: %s", root);
+		dbus_g_method_return_error (context, error);
+		goto out;
+	}
+
+	/* save sender */
+	sender = dbus_g_method_get_sender (context);
+
+	/* is exactly the same root? */
+	ret = pk_engine_is_root_unchanged (engine, sender, root);
+	if (ret) {
+		egg_debug ("not changing root as the same as before");
+		dbus_g_method_return (context);
+		goto out;
+	}
+
+	/* '/' is the default root, which doesn't need additional authentication */
+	if (g_strcmp0 (root, "/") == 0) {
+		ret = pk_engine_set_root_internal (engine, root, sender);
+		if (ret) {
+			egg_debug ("using default root, so no need to authenticate");
+			dbus_g_method_return (context);
+		} else {
+			error = g_error_new (PK_ENGINE_ERROR, PK_ENGINE_ERROR_CANNOT_SET_ROOT, "%s", "setting the root failed");
+			dbus_g_method_return_error (context, error);
+		}
+		goto out;
+	}
+
+#ifdef USE_SECURITY_POLKIT
+	/* check subject */
+	subject = polkit_system_bus_name_new (sender);
+
+	/* insert details about the authorization */
+	details = polkit_details_new ();
+	polkit_details_insert (details, "role", pk_role_enum_to_string (PK_ROLE_ENUM_UNKNOWN));
+
+	/* cache state */
+	state = g_new0 (PkEngineDbusState, 1);
+	state->context = context;
+	state->engine = g_object_ref (engine);
+	state->sender = g_strdup (sender);
+	state->value1 = g_strdup (root);
+
+	/* do authorization async */
+	polkit_authority_check_authorization (engine->priv->authority, subject,
+					      "org.freedesktop.packagekit.system-change-install-root",
+					      details,
+					      POLKIT_CHECK_AUTHORIZATION_FLAGS_ALLOW_USER_INTERACTION,
+					      NULL,
+					      (GAsyncReadyCallback) pk_engine_action_obtain_root_authorization_finished_cb,
+					      state);
+
+	/* check_authorization ref's this */
+	g_object_unref (details);
+#else
+	egg_warning ("*** THERE IS NO SECURITY MODEL BEING USED!!! ***");
+
+	/* try to set the new root and save to database */
+	ret = pk_engine_set_root_internal (engine, root, sender);
+	if (!ret) {
+		error = g_error_new (PK_ENGINE_ERROR, PK_ENGINE_ERROR_CANNOT_SET_ROOT, "%s", "setting the root failed");
+		dbus_g_method_return_error (context, error);
+		goto out;
+	}
+
+	/* all okay */
+	dbus_g_method_return (context);
+#endif
+
+	/* reset the timer */
+	pk_engine_reset_timer (engine);
+
+#ifdef USE_SECURITY_POLKIT
+	g_object_unref (subject);
+#endif
+out:
+	g_free (sender);
 }
 
 /**
@@ -1102,7 +1313,7 @@ pk_engine_can_authorize (PkEngine *engine, const gchar *action_id, DBusGMethodIn
 	}
 
 	/* all okay */
-	dbus_g_method_return (context, pk_authorize_type_enum_to_text (result_enum));
+	dbus_g_method_return (context, pk_authorize_type_enum_to_string (result_enum));
 }
 
 /**
@@ -1134,15 +1345,15 @@ pk_engine_get_property (GObject *object, guint prop_id, GValue *value, GParamSpe
 		g_value_set_string (value, engine->priv->backend_author);
 		break;
 	case PROP_ROLES:
-		tmp = pk_role_bitfield_to_text (engine->priv->roles);
+		tmp = pk_role_bitfield_to_string (engine->priv->roles);
 		g_value_set_string (value, tmp);
 		break;
 	case PROP_GROUPS:
-		tmp = pk_group_bitfield_to_text (engine->priv->groups);
+		tmp = pk_group_bitfield_to_string (engine->priv->groups);
 		g_value_set_string (value, tmp);
 		break;
 	case PROP_FILTERS:
-		tmp = pk_filter_bitfield_to_text (engine->priv->filters);
+		tmp = pk_filter_bitfield_to_string (engine->priv->filters);
 		g_value_set_string (value, tmp);
 		break;
 	case PROP_MIME_TYPES:
@@ -1152,7 +1363,7 @@ pk_engine_get_property (GObject *object, guint prop_id, GValue *value, GParamSpe
 		g_value_set_boolean (value, engine->priv->locked);
 		break;
 	case PROP_NETWORK_STATE:
-		g_value_set_string (value, pk_network_enum_to_text (engine->priv->network_state));
+		g_value_set_string (value, pk_network_enum_to_string (engine->priv->network_state));
 		break;
 	case PROP_DISTRO_ID:
 		g_value_set_string (value, engine->priv->distro_id);
@@ -1296,11 +1507,6 @@ pk_engine_class_init (PkEngineClass *klass)
 	g_object_class_install_property (object_class, PROP_DISTRO_ID, pspec);
 
 	/* signals */
-	signals[SIGNAL_LOCKED] =
-		g_signal_new ("locked",
-			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
-			      0, NULL, NULL, g_cclosure_marshal_VOID__BOOLEAN,
-			      G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
 	signals[SIGNAL_TRANSACTION_LIST_CHANGED] =
 		g_signal_new ("transaction-list-changed",
 			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
@@ -1316,11 +1522,6 @@ pk_engine_class_init (PkEngineClass *klass)
 			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
 			      0, NULL, NULL, g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
-	signals[SIGNAL_NETWORK_STATE_CHANGED] =
-		g_signal_new ("network-state-changed",
-			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
-			      0, NULL, NULL, g_cclosure_marshal_VOID__STRING,
-			      G_TYPE_NONE, 1, G_TYPE_STRING);
 	signals[SIGNAL_UPDATES_CHANGED] =
 		g_signal_new ("updates-changed",
 			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
@@ -1370,11 +1571,7 @@ pk_engine_binary_file_changed_cb (PkFileMonitor *file_monitor, PkEngine *engine)
 static void
 pk_engine_network_state_changed_cb (PkNetwork *network, PkNetworkEnum network_state, PkEngine *engine)
 {
-	const gchar *state_text;
 	g_return_if_fail (PK_IS_ENGINE (engine));
-	state_text = pk_network_enum_to_text (network_state);
-	egg_debug ("emitting network-state-changed: %s", state_text);
-	g_signal_emit (engine, signals[SIGNAL_NETWORK_STATE_CHANGED], 0, state_text);
 
 	/* already set */
 	if (engine->priv->network_state == network_state)
@@ -1396,6 +1593,12 @@ pk_engine_init (PkEngine *engine)
 	DBusGConnection *connection;
 	gboolean ret;
 	gchar *filename;
+	gchar *root;
+	gchar *proxy_http;
+	gchar *proxy_ftp;
+#ifdef USE_SECURITY_POLKIT_NEW
+	GError *error = NULL;
+#endif
 
 	engine->priv = PK_ENGINE_GET_PRIVATE (engine);
 	engine->priv->notify_clients_of_upgrade = FALSE;
@@ -1404,7 +1607,6 @@ pk_engine_init (PkEngine *engine)
 	engine->priv->backend_name = NULL;
 	engine->priv->backend_description = NULL;
 	engine->priv->backend_author = NULL;
-	engine->priv->sender = NULL;
 	engine->priv->locked = FALSE;
 	engine->priv->distro_id = NULL;
 
@@ -1487,7 +1689,15 @@ pk_engine_init (PkEngine *engine)
 
 #ifdef USE_SECURITY_POLKIT
 	/* protect the session SetProxy with a PolicyKit action */
+#ifdef USE_SECURITY_POLKIT_NEW
+	engine->priv->authority = polkit_authority_get_sync (NULL, &error);
+	if (engine->priv->authority == NULL) {
+		g_error ("failed to get pokit authority: %s", error->message);
+		g_error_free (error);
+	}
+#else
 	engine->priv->authority = polkit_authority_get ();
+#endif
 #endif
 
 	/* monitor the binary file for changes */
@@ -1497,9 +1707,21 @@ pk_engine_init (PkEngine *engine)
 			  G_CALLBACK (pk_engine_binary_file_changed_cb), engine);
 
 	/* set the default proxy */
-	engine->priv->proxy_http = pk_conf_get_string (engine->priv->conf, "ProxyHTTP");
-	engine->priv->proxy_ftp = pk_conf_get_string (engine->priv->conf, "ProxyFTP");
-	pk_backend_set_proxy (engine->priv->backend, engine->priv->proxy_http, engine->priv->proxy_ftp);
+	proxy_http = pk_conf_get_string (engine->priv->conf, "ProxyHTTP");
+	proxy_ftp = pk_conf_get_string (engine->priv->conf, "ProxyFTP");
+	pk_backend_set_proxy (engine->priv->backend, proxy_http, proxy_ftp);
+
+	/* if either of these is set, we ignore the users proxy setting */
+	if (proxy_http != NULL || proxy_ftp != NULL)
+		engine->priv->using_hardcoded_proxy = TRUE;
+
+	g_free (proxy_http);
+	g_free (proxy_ftp);
+
+	/* set the default root */
+	root = pk_conf_get_string (engine->priv->conf, "UseRoot");
+	pk_backend_set_root (engine->priv->backend, root);
+	g_free (root);
 
 	/* get the StateHasChanged timeouts */
 	engine->priv->timeout_priority = (guint) pk_conf_get_int (engine->priv->conf, "StateChangedTimeoutPriority");
@@ -1566,9 +1788,6 @@ pk_engine_finalize (GObject *object)
 	g_object_unref (engine->priv->conf);
 	g_object_unref (engine->priv->dbus);
 	g_free (engine->priv->mime_types);
-	g_free (engine->priv->proxy_http);
-	g_free (engine->priv->proxy_ftp);
-	g_free (engine->priv->sender);
 	g_free (engine->priv->backend_name);
 	g_free (engine->priv->backend_description);
 	g_free (engine->priv->backend_author);
@@ -1590,257 +1809,3 @@ pk_engine_new (void)
 	return PK_ENGINE (engine);
 }
 
-/***************************************************************************
- ***                          MAKE CHECK TESTS                           ***
- ***************************************************************************/
-#ifdef EGG_TEST
-#include "egg-test.h"
-
-static PkNotify *notify = NULL;
-static gboolean _quit = FALSE;
-static gboolean _locked = FALSE;
-static gboolean _restart_schedule = FALSE;
-
-/**
- * pk_test_quit_cb:
- **/
-static void
-pk_test_quit_cb (PkEngine *engine, EggTest *test)
-{
-	_quit = TRUE;
-}
-
-/**
- * pk_test_locked_cb:
- **/
-static void
-pk_test_locked_cb (PkEngine *engine, gboolean is_locked, EggTest *test)
-{
-	_locked = is_locked;
-}
-
-/**
- * pk_test_updates_changed_cb:
- **/
-static void
-pk_test_updates_changed_cb (PkEngine *engine, EggTest *test)
-{
-	egg_test_loop_quit (test);
-}
-
-/**
- * pk_test_repo_list_changed_cb:
- **/
-static void
-pk_test_repo_list_changed_cb (PkEngine *engine, EggTest *test)
-{
-	egg_test_loop_quit (test);
-}
-
-/**
- * pk_test_restart_schedule_cb:
- **/
-static void
-pk_test_restart_schedule_cb (PkEngine *engine, EggTest *test)
-{
-	_restart_schedule = TRUE;
-	egg_test_loop_quit (test);
-}
-
-/**
- * pk_test_emit_updates_changed_cb:
- **/
-static gboolean
-pk_test_emit_updates_changed_cb (EggTest *test)
-{
-	PkNotify *notify2;
-	notify2 = pk_notify_new ();
-	pk_notify_updates_changed (notify2);
-	g_object_unref (notify2);
-	return FALSE;
-}
-
-/**
- * pk_test_emit_repo_list_changed_cb:
- **/
-static gboolean
-pk_test_emit_repo_list_changed_cb (EggTest *test)
-{
-	PkNotify *notify2;
-	notify2 = pk_notify_new ();
-	pk_notify_repo_list_changed (notify2);
-	g_object_unref (notify2);
-	return FALSE;
-}
-
-void
-pk_engine_test (EggTest *test)
-{
-	gboolean ret;
-	PkEngine *engine;
-	PkBackend *backend;
-	PkInhibit *inhibit;
-	guint idle;
-	gchar *actions;
-	guint elapsed;
-
-	if (!egg_test_start (test, "PkEngine"))
-		return;
-
-	/************************************************************/
-	egg_test_title (test, "get a backend instance");
-	backend = pk_backend_new ();
-	egg_test_assert (test, backend != NULL);
-
-	/************************************************************/
-	egg_test_title (test, "get a notify instance");
-	notify = pk_notify_new ();
-	egg_test_assert (test, notify != NULL);
-
-	/* set the type, as we have no pk-main doing this for us */
-	/************************************************************/
-	egg_test_title (test, "set the backend name");
-	ret = pk_backend_set_name (backend, "dummy");
-	egg_test_assert (test, ret);
-
-	/************************************************************/
-	egg_test_title (test, "get an engine instance");
-	engine = pk_engine_new ();
-	egg_test_assert (test, engine != NULL);
-
-	/* connect up signals */
-	g_signal_connect (engine, "quit",
-			  G_CALLBACK (pk_test_quit_cb), test);
-	g_signal_connect (engine, "locked",
-			  G_CALLBACK (pk_test_locked_cb), test);
-	g_signal_connect (engine, "updates-changed",
-			  G_CALLBACK (pk_test_updates_changed_cb), test);
-	g_signal_connect (engine, "repo-list-changed",
-			  G_CALLBACK (pk_test_repo_list_changed_cb), test);
-	g_signal_connect (engine, "restart-schedule",
-			  G_CALLBACK (pk_test_restart_schedule_cb), test);
-
-	/************************************************************
-	 **********           GET IDLE TIMES              ***********
-	 ************************************************************/
-	egg_test_title (test, "get idle at startup");
-	idle = pk_engine_get_seconds_idle (engine);
-	if (idle < 1)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "idle = %i", idle);
-
-	/* wait 5 seconds */
-	egg_test_loop_wait (test, 5000);
-
-	/************************************************************/
-	egg_test_title (test, "get idle at idle");
-	idle = pk_engine_get_seconds_idle (engine);
-	if (idle < 6 && idle > 4)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "idle = %i", idle);
-
-	/************************************************************/
-	egg_test_title (test, "get idle after method");
-	pk_engine_get_actions (engine, &actions, NULL);
-	g_free (actions);
-	idle = pk_engine_get_seconds_idle (engine);
-	if (idle < 1)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "idle = %i", idle);
-
-	/************************************************************
-	 **********        TEST PROXY NOTIFY              ***********
-	 ************************************************************/
-	egg_test_title (test, "force test notify updates-changed");
-	g_timeout_add (25, (GSourceFunc) pk_test_emit_updates_changed_cb, test);
-	egg_test_success (test, NULL);
-	egg_test_loop_wait (test, 50);
-	egg_test_loop_check (test);
-
-	/************************************************************/
-	egg_test_title (test, "force test notify repo-list-changed");
-	g_timeout_add (25, (GSourceFunc) pk_test_emit_repo_list_changed_cb, test);
-	egg_test_success (test, NULL);
-	egg_test_loop_wait (test, 50);
-	egg_test_loop_check (test);
-
-	/************************************************************/
-	egg_test_title (test, "force test notify wait updates-changed");
-	pk_notify_wait_updates_changed (notify, 500);
-	egg_test_loop_wait (test, 1000);
-	elapsed = egg_test_elapsed (test);
-	if (elapsed > 400 && elapsed < 600)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "failed to quit (%i)", elapsed);
-
-	/************************************************************
-	 **********               LOCKING                 ***********
-	 ************************************************************/
-	egg_test_title (test, "test locked");
-	inhibit = pk_inhibit_new ();
-	pk_inhibit_add (inhibit, GUINT_TO_POINTER (999));
-	if (_locked)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "not locked");
-
-	/************************************************************/
-	egg_test_title (test, "test locked");
-	pk_inhibit_remove (inhibit, GUINT_TO_POINTER (999));
-	if (!_locked)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "not locked");
-	g_object_unref (inhibit);
-
-	/************************************************************/
-	egg_test_title (test, "test not locked");
-	if (!_locked)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "still locked");
-
-	/************************************************************
-	 **********          BINARY UPGRADE TEST          ***********
-	 ************************************************************/
-	egg_test_title_assert (test, "restart_schedule not set", !_restart_schedule);
-	ret = g_file_set_contents (SBINDIR "/packagekitd", "overwrite", -1, NULL);
-
-	/************************************************************/
-	egg_test_title_assert (test, "touched binary file", ret);
-	egg_test_loop_wait (test, 5000);
-
-	/************************************************************/
-	egg_test_title (test, "get idle after we touched the binary");
-	idle = pk_engine_get_seconds_idle (engine);
-	if (idle == G_MAXUINT)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "idle = %i", idle);
-
-	/************************************************************/
-	egg_test_title_assert (test, "restart_schedule set", _restart_schedule);
-
-	/************************************************************
-	 **********             DAEMON QUIT               ***********
-	 ************************************************************/
-	egg_test_title_assert (test, "not already quit", !_quit);
-	egg_test_title (test, "suggest quit with no transactions (should get quit signal)");
-	pk_engine_suggest_daemon_quit (engine, NULL);
-	if (_quit)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "did not quit");
-
-	g_object_unref (backend);
-	g_object_unref (notify);
-	g_object_unref (engine);
-
-	egg_test_end (test);
-}
-#endif
-
