@@ -288,111 +288,3 @@ pk_inhibit_new (void)
 	return PK_INHIBIT (pk_inhibit_object);
 }
 
-/***************************************************************************
- ***                          MAKE CHECK TESTS                           ***
- ***************************************************************************/
-#ifdef EGG_TEST
-#include "egg-test.h"
-
-void
-pk_inhibit_test (EggTest *test)
-{
-	PkInhibit *inhibit;
-	gboolean ret;
-
-	if (!egg_test_start (test, "PkInhibit"))
-		return;
-
-	/************************************************************/
-	egg_test_title (test, "get an instance");
-	inhibit = pk_inhibit_new ();
-	if (inhibit != NULL)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, NULL);
-
-	/************************************************************/
-	egg_test_title (test, "check we have a connection");
-	if (inhibit->priv->proxy != NULL)
-		egg_test_success (test, "got proxy");
-	else
-		egg_test_failed (test, "could not get proxy");
-
-	/************************************************************/
-	egg_test_title (test, "check we are not inhibited");
-	ret = pk_inhibit_locked (inhibit);
-	if (ret == FALSE)
-		egg_test_success (test, "marked correctly");
-	else
-		egg_test_failed (test, "not marked correctly");
-
-	/************************************************************/
-	egg_test_title (test, "add 123");
-	ret = pk_inhibit_add (inhibit, GUINT_TO_POINTER (123));
-	if (ret)
-		egg_test_success (test, "inhibited");
-	else
-		egg_test_failed (test, "could not inhibit");
-
-	/************************************************************/
-	egg_test_title (test, "check we are inhibited");
-	ret = pk_inhibit_locked (inhibit);
-	if (ret)
-		egg_test_success (test, "marked correctly");
-	else
-		egg_test_failed (test, "not marked correctly");
-
-	/************************************************************/
-	egg_test_title (test, "add 123 (again)");
-	ret = pk_inhibit_add (inhibit, GUINT_TO_POINTER (123));
-	if (ret == FALSE)
-		egg_test_success (test, "correctly ignored second");
-	else
-		egg_test_failed (test, "added the same number twice");
-
-	/************************************************************/
-	egg_test_title (test, "add 456");
-	ret = pk_inhibit_add (inhibit, GUINT_TO_POINTER (456));
-	if (ret)
-		egg_test_success (test, "inhibited");
-	else
-		egg_test_failed (test, "could not inhibit");
-
-	/************************************************************/
-	egg_test_title (test, "remove 123");
-	ret = pk_inhibit_remove (inhibit, GUINT_TO_POINTER (123));
-	if (ret)
-		egg_test_success (test, "removed first inhibit");
-	else
-		egg_test_failed (test, "could not remove inhibit");
-
-	/************************************************************/
-	egg_test_title (test, "check we are still inhibited");
-	ret = pk_inhibit_locked (inhibit);
-	if (ret)
-		egg_test_success (test, "marked correctly");
-	else
-		egg_test_failed (test, "not marked correctly");
-
-	/************************************************************/
-	egg_test_title (test, "remove 456");
-	ret = pk_inhibit_remove (inhibit, GUINT_TO_POINTER (456));
-	if (ret)
-		egg_test_success (test, "removed second inhibit");
-	else
-		egg_test_failed (test, "could not remove inhibit");
-
-	/************************************************************/
-	egg_test_title (test, "check we are not inhibited");
-	ret = pk_inhibit_locked (inhibit);
-	if (ret == FALSE)
-		egg_test_success (test, "marked correctly");
-	else
-		egg_test_failed (test, "not marked correctly");
-
-	g_object_unref (inhibit);
-
-	egg_test_end (test);
-}
-#endif
-
