@@ -59,7 +59,7 @@ static void     pk_spawn_finalize	(GObject       *object);
 
 struct PkSpawnPrivate
 {
-	gint			 child_pid;
+	pid_t			 child_pid;
 	gint			 stdin_fd;
 	gint			 stdout_fd;
 	gint			 stderr_fd;
@@ -153,10 +153,10 @@ pk_spawn_emit_whole_lines (PkSpawn *spawn, GString *string)
 }
 
 /**
- * pk_spawn_exit_type_enum_to_text:
+ * pk_spawn_exit_type_enum_to_string:
  **/
 static const gchar *
-pk_spawn_exit_type_enum_to_text (PkSpawnExitType type)
+pk_spawn_exit_type_enum_to_string (PkSpawnExitType type)
 {
 	if (type == PK_SPAWN_EXIT_TYPE_SUCCESS)
 		return "success";
@@ -179,7 +179,9 @@ pk_spawn_exit_type_enum_to_text (PkSpawnExitType type)
 static gboolean
 pk_spawn_check_child (PkSpawn *spawn)
 {
+	pid_t pid;
 	int status;
+	gint retval;
 	static guint limit_printing = 0;
 
 	/* this shouldn't happen */
@@ -203,11 +205,23 @@ pk_spawn_check_child (PkSpawn *spawn)
 
 	/* Only print one in twenty times to avoid filling the screen */
 	if (limit_printing++ % 20 == 0)
-		egg_debug ("polling child_pid=%i (1/20)", spawn->priv->child_pid);
+		egg_debug ("polling child_pid=%ld (1/20)", (long)spawn->priv->child_pid);
 
 	/* check if the child exited */
-	if (waitpid (spawn->priv->child_pid, &status, WNOHANG) != spawn->priv->child_pid)
+	pid = waitpid (spawn->priv->child_pid, &status, WNOHANG);
+	if (pid == -1) {
+		egg_warning ("failed to get the child PID data for %ld", (long)spawn->priv->child_pid);
 		return TRUE;
+	}
+	if (pid == 0) {
+		/* process still exist, but has not changed state */
+		return TRUE;
+	}
+	if (pid != spawn->priv->child_pid) {
+		egg_warning ("some other process id was returned: got %ld and wanted %ld",
+			     (long)pid, (long)spawn->priv->child_pid);
+		return TRUE;
+	}
 
 	/* disconnect the poll as there will be no more updates */
 	if (spawn->priv->poll_id > 0) {
@@ -224,13 +238,37 @@ pk_spawn_check_child (PkSpawn *spawn)
 	spawn->priv->stderr_fd = -1;
 	spawn->priv->child_pid = -1;
 
-	if (WEXITSTATUS (status) > 0) {
-		egg_debug ("Running fork failed with return value %d", WEXITSTATUS (status));
-		if (spawn->priv->exit == PK_SPAWN_EXIT_TYPE_UNKNOWN)
-			spawn->priv->exit = PK_SPAWN_EXIT_TYPE_FAILED;
+	/* use this to detect SIGKILL and SIGQUIT */
+	if (WIFSIGNALED (status)) {
+		retval = WTERMSIG (status);
+		if (retval == SIGQUIT) {
+			egg_debug ("the child process was terminated by SIGQUIT");
+			spawn->priv->exit = PK_SPAWN_EXIT_TYPE_SIGQUIT;
+		} else if (retval == SIGKILL) {
+			egg_debug ("the child process was terminated by SIGKILL");
+			spawn->priv->exit = PK_SPAWN_EXIT_TYPE_SIGKILL;
+		} else {
+			egg_warning ("the child process was terminated by signal %i", WTERMSIG (status));
+			spawn->priv->exit = PK_SPAWN_EXIT_TYPE_SIGKILL;
+		}
 	} else {
-		if (spawn->priv->exit == PK_SPAWN_EXIT_TYPE_UNKNOWN)
-			spawn->priv->exit = PK_SPAWN_EXIT_TYPE_SUCCESS;
+		/* check we are dead and buried */
+		if (!WIFEXITED (status)) {
+			egg_warning ("the process did not exit, but waitpid() returned!");
+			return TRUE;
+		}
+
+		/* get the exit code */
+		retval = WEXITSTATUS (status);
+		if (retval == 0) {
+			egg_debug ("the child exited with success");
+			if (spawn->priv->exit == PK_SPAWN_EXIT_TYPE_UNKNOWN)
+				spawn->priv->exit = PK_SPAWN_EXIT_TYPE_SUCCESS;
+		} else {
+			egg_warning ("the child exited with return code %i", retval);
+			if (spawn->priv->exit == PK_SPAWN_EXIT_TYPE_UNKNOWN)
+				spawn->priv->exit = PK_SPAWN_EXIT_TYPE_FAILED;
+		}
 	}
 
 	/* officially done, although no signal yet */
@@ -249,7 +287,7 @@ pk_spawn_check_child (PkSpawn *spawn)
 		spawn->priv->exit = PK_SPAWN_EXIT_TYPE_DISPATCHER_EXIT;
 
 	/* don't emit if we just closed an invalid dispatcher */
-	egg_debug ("emitting exit %s", pk_spawn_exit_type_enum_to_text (spawn->priv->exit));
+	egg_debug ("emitting exit %s", pk_spawn_exit_type_enum_to_string (spawn->priv->exit));
 	g_signal_emit (spawn, signals [SIGNAL_EXIT], 0, spawn->priv->exit);
 
 	return FALSE;
@@ -269,10 +307,10 @@ pk_spawn_sigkill_cb (PkSpawn *spawn)
 		return FALSE;
 	}
 
-	/* we won't overwrite this if not unknown */
+	/* set this in case the script catches the signal and exits properly */
 	spawn->priv->exit = PK_SPAWN_EXIT_TYPE_SIGKILL;
 
-	egg_debug ("sending SIGKILL %i", spawn->priv->child_pid);
+	egg_debug ("sending SIGKILL %ld", (long)spawn->priv->child_pid);
 	retval = kill (spawn->priv->child_pid, SIGKILL);
 	if (retval == EINVAL) {
 		egg_warning ("The signum argument is an invalid or unsupported number");
@@ -302,6 +340,9 @@ pk_spawn_is_running (PkSpawn *spawn)
  * pk_spawn_kill:
  *
  * We send SIGQUIT and after a few ms SIGKILL (if allowed)
+ *
+ * IMPORTANT: This is not a syncronous operation, and client programs will need
+ * to wait for the ::exit signal.
  **/
 gboolean
 pk_spawn_kill (PkSpawn *spawn)
@@ -311,16 +352,22 @@ pk_spawn_kill (PkSpawn *spawn)
 	g_return_val_if_fail (PK_IS_SPAWN (spawn), FALSE);
 	g_return_val_if_fail (spawn->priv->kill_id == 0, FALSE);
 
+	/* is there a process running? */
+	if (spawn->priv->child_pid == -1) {
+		egg_warning ("no child pid to kill!");
+		return FALSE;
+	}
+
 	/* check if process has already gone */
 	if (spawn->priv->finished) {
 		egg_warning ("already finished, ignoring");
 		return FALSE;
 	}
 
-	/* we won't overwrite this if not unknown */
+	/* set this in case the script catches the signal and exits properly */
 	spawn->priv->exit = PK_SPAWN_EXIT_TYPE_SIGQUIT;
 
-	egg_debug ("sending SIGQUIT %i", spawn->priv->child_pid);
+	egg_debug ("sending SIGQUIT %ld", (long)spawn->priv->child_pid);
 	retval = kill (spawn->priv->child_pid, SIGQUIT);
 	if (retval == EINVAL) {
 		egg_warning ("The signum argument is an invalid or unsupported number");
@@ -331,9 +378,12 @@ pk_spawn_kill (PkSpawn *spawn)
 	}
 
 	/* the program might not be able to handle SIGQUIT, give it a few seconds and then SIGKILL it */
-	if (spawn->priv->allow_sigkill)
+	if (spawn->priv->allow_sigkill) {
 		spawn->priv->kill_id = g_timeout_add (PK_SPAWN_SIGKILL_DELAY, (GSourceFunc) pk_spawn_sigkill_cb, spawn);
-
+#if GLIB_CHECK_VERSION(2,25,8)
+		g_source_set_name_by_id (spawn->priv->kill_id, "[PkSpawn] sigkill");
+#endif
+	}
 	return TRUE;
 }
 
@@ -360,6 +410,13 @@ pk_spawn_send_stdin (PkSpawn *spawn, const gchar *command)
 		goto out;
 	}
 
+	/* is there a process running? */
+	if (spawn->priv->child_pid == -1) {
+		egg_debug ("no child pid");
+		ret = FALSE;
+		goto out;
+	}
+
 	/* buffer always has to have trailing newline */
 	egg_debug ("sending '%s'", command);
 	buffer = g_strdup_printf ("%s\n", command);
@@ -401,8 +458,10 @@ pk_spawn_exit (PkSpawn *spawn)
 	/* send command */
 	spawn->priv->is_sending_exit = TRUE;
 	ret = pk_spawn_send_stdin (spawn, "exit");
-	if (!ret)
+	if (!ret) {
+		egg_warning ("failed to send exit");
 		goto out;
+	}
 
 	/* block until the previous script exited */
 	do {
@@ -411,13 +470,15 @@ pk_spawn_exit (PkSpawn *spawn)
 		 * If we run the loop, other idle events can be processed,
 		 * and this includes sending data to a new instance,
 		 * which of course will fail as the 'old' script is exiting */
-		g_usleep (100*1000); /* 100 ms */
+		g_usleep (10*1000); /* 10 ms */
 		ret = pk_spawn_check_child (spawn);
-	} while (ret && count++ < 50);
+	} while (ret && count++ < 500);
 
 	/* the script exited okay */
-	if (count < 50)
+	if (count < 500)
 		ret = TRUE;
+	else
+		egg_warning ("failed to exit script");
 out:
 	spawn->priv->is_sending_exit = FALSE;
 	return ret;
@@ -430,9 +491,10 @@ out:
  *
  **/
 gboolean
-pk_spawn_argv (PkSpawn *spawn, gchar **argv, gchar **envp)
+pk_spawn_argv (PkSpawn *spawn, gchar **argv, gchar **envp, GError **error)
 {
 	gboolean ret;
+	GError *error_local = NULL;
 	gboolean idleio;
 	guint i;
 	guint len;
@@ -441,6 +503,7 @@ pk_spawn_argv (PkSpawn *spawn, gchar **argv, gchar **envp)
 	const gchar *key;
 
 	g_return_val_if_fail (PK_IS_SPAWN (spawn), FALSE);
+	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 	g_return_val_if_fail (argv != NULL, FALSE);
 
 	len = g_strv_length (argv);
@@ -454,7 +517,7 @@ pk_spawn_argv (PkSpawn *spawn, gchar **argv, gchar **envp)
 
 	/* check we are not using a closing instance */
 	if (spawn->priv->is_sending_exit) {
-		egg_warning ("trying to use instance that is in the process of exiting");
+		g_set_error_literal (error, 1, 0, "trying to use instance that is in the process of exiting");
 		return FALSE;
 	}
 
@@ -506,7 +569,13 @@ pk_spawn_argv (PkSpawn *spawn, gchar **argv, gchar **envp)
 				 &spawn->priv->stdin_fd,
 				 &spawn->priv->stdout_fd,
 				 &spawn->priv->stderr_fd,
-				 NULL);
+				 &error_local);
+	/* we failed to invoke the helper */
+	if (!ret) {
+		g_set_error (error, 1, 0, "failed to spawn %s: %s", argv[0], error_local->message);
+		g_error_free (error_local);
+		return FALSE;
+	}
 
 	/* get the nice value and ensure we are in the valid range */
 	key = "BackendSpawnNiceValue";
@@ -515,11 +584,13 @@ pk_spawn_argv (PkSpawn *spawn, gchar **argv, gchar **envp)
 	nice_value = pk_conf_get_int (spawn->priv->conf, key);
 	nice_value = CLAMP(nice_value, -20, 19);
 
+#if HAVE_SETPRIORITY
 	/* don't completely bog the system down */
 	if (nice_value != 0) {
 		egg_debug ("renice to %i", nice_value);
 		setpriority (PRIO_PROCESS, spawn->priv->child_pid, nice_value);
 	}
+#endif
 
 	/* perhaps set idle IO priority */
 	key = "BackendSpawnIdleIO";
@@ -531,12 +602,6 @@ pk_spawn_argv (PkSpawn *spawn, gchar **argv, gchar **envp)
 		pk_ioprio_set_idle (spawn->priv->child_pid);
 	}
 
-	/* we failed to invoke the helper */
-	if (!ret) {
-		egg_warning ("failed to spawn '%s'", argv[0]);
-		return FALSE;
-	}
-
 	/* save this so we can check the dispatcher name */
 	g_free (spawn->priv->last_argv0);
 	spawn->priv->last_argv0 = g_strdup (argv[0]);
@@ -550,11 +615,16 @@ pk_spawn_argv (PkSpawn *spawn, gchar **argv, gchar **envp)
 	fcntl (spawn->priv->stderr_fd, F_SETFL, O_NONBLOCK);
 
 	/* sanity check */
-	if (spawn->priv->poll_id != 0)
-		egg_error ("trying to set timeout when already set");
+	if (spawn->priv->poll_id != 0) {
+		egg_warning ("trying to set timeout when already set");
+		g_source_remove (spawn->priv->poll_id);
+	}
 
 	/* poll quickly */
 	spawn->priv->poll_id = g_timeout_add (PK_SPAWN_POLL_DELAY, (GSourceFunc) pk_spawn_check_child, spawn);
+#if GLIB_CHECK_VERSION(2,25,8)
+	g_source_set_name_by_id (spawn->priv->poll_id, "[PkSpawn] main poll");
+#endif
 
 	return TRUE;
 }
@@ -744,386 +814,3 @@ pk_spawn_new (void)
 	return PK_SPAWN (spawn);
 }
 
-/***************************************************************************
- ***                          MAKE CHECK TESTS                           ***
- ***************************************************************************/
-#ifdef EGG_TEST
-#include "egg-test.h"
-
-PkSpawnExitType mexit = PK_SPAWN_EXIT_TYPE_UNKNOWN;
-guint stdout_count = 0;
-guint finished_count = 0;
-
-/**
- * pk_test_exit_cb:
- **/
-static void
-pk_test_exit_cb (PkSpawn *spawn, PkSpawnExitType exit, EggTest *test)
-{
-	egg_debug ("spawn exit=%i", exit);
-	mexit = exit;
-	finished_count++;
-	egg_test_loop_quit (test);
-}
-
-/**
- * pk_test_stdout_cb:
- **/
-static void
-pk_test_stdout_cb (PkSpawn *spawn, const gchar *line, EggTest *test)
-{
-	egg_debug ("stdout '%s'", line);
-	stdout_count++;
-}
-
-static gboolean
-cancel_cb (gpointer data)
-{
-	PkSpawn *spawn = PK_SPAWN(data);
-	pk_spawn_kill (spawn);
-	return FALSE;
-}
-
-static void
-new_spawn_object (EggTest *test, PkSpawn **pspawn)
-{
-	if (*pspawn != NULL)
-		g_object_unref (*pspawn);
-	*pspawn = pk_spawn_new ();
-	g_signal_connect (*pspawn, "exit",
-			  G_CALLBACK (pk_test_exit_cb), test);
-	g_signal_connect (*pspawn, "stdout",
-			  G_CALLBACK (pk_test_stdout_cb), test);
-	stdout_count = 0;
-}
-
-static gboolean
-idle_cb (gpointer data)
-{
-	EggTest *test = (EggTest*) data;
-
-	egg_test_title (test, "make sure dispatcher has closed when run idle add");
-	if (mexit == PK_SPAWN_EXIT_TYPE_DISPATCHER_EXIT)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "mexit was %i", mexit);
-
-	/* never repeat */
-	return FALSE;
-}
-
-void
-pk_spawn_test (EggTest *test)
-{
-	PkSpawn *spawn = NULL;
-	gboolean ret;
-	gchar *file;
-	gchar *path;
-	gchar **argv;
-	gchar **envp;
-	guint elapsed;
-
-	if (!egg_test_start (test, "PkSpawn"))
-		return;
-
-	/* get new object */
-	new_spawn_object (test, &spawn);
-
-	/************************************************************
-	 **********           Generic tests               ***********
-	 ************************************************************/
-	egg_test_title (test, "make sure return error for missing file");
-	mexit = PK_SPAWN_EXIT_TYPE_UNKNOWN;
-	argv = g_strsplit ("pk-spawn-test-xxx.sh", " ", 0);
-	ret = pk_spawn_argv (spawn, argv, NULL);
-	g_strfreev (argv);
-	if (!ret)
-		egg_test_success (test, "failed to run invalid file");
-	else
-		egg_test_failed (test, "ran incorrect file");
-
-	/************************************************************/
-	egg_test_title (test, "make sure finished wasn't called");
-	if (mexit == PK_SPAWN_EXIT_TYPE_UNKNOWN)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "Called finish for bad file!");
-
-	/************************************************************/
-	egg_test_title (test, "make sure run correct helper");
-	mexit = -1;
-	path = egg_test_get_data_file ("pk-spawn-test.sh");
-	argv = g_strsplit (path, " ", 0);
-	ret = pk_spawn_argv (spawn, argv, NULL);
-	g_free (path);
-	g_strfreev (argv);
-	if (ret)
-		egg_test_success (test, "ran correct file");
-	else
-		egg_test_failed (test, "did not run helper");
-
-	/* wait for finished */
-	egg_test_loop_wait (test, 10000);
-	egg_test_loop_check (test);
-
-	/************************************************************/
-	egg_test_title (test, "make sure finished okay");
-	if (mexit == PK_SPAWN_EXIT_TYPE_SUCCESS)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "finish was okay!");
-
-	/************************************************************/
-	egg_test_title (test, "make sure finished was called only once");
-	if (finished_count == 1)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "finish was called %i times!", finished_count);
-
-	/************************************************************/
-	egg_test_title (test, "make sure we got the right stdout data");
-	if (stdout_count == 4+11)
-		egg_test_success (test, "correct stdout count");
-	else
-		egg_test_failed (test, "wrong stdout count %i", stdout_count);
-
-	/* get new object */
-	new_spawn_object (test, &spawn);
-
-	/************************************************************
-	 **********            envp tests                 ***********
-	 ************************************************************/
-	egg_test_title (test, "make sure we set the proxy");
-	mexit = -1;
-	path = egg_test_get_data_file ("pk-spawn-proxy.sh");
-	argv = g_strsplit (path, " ", 0);
-	envp = g_strsplit ("http_proxy=username:password@server:port "
-			   "ftp_proxy=username:password@server:port", " ", 0);
-	ret = pk_spawn_argv (spawn, argv, envp);
-	g_free (path);
-	g_strfreev (argv);
-	g_strfreev (envp);
-	if (ret)
-		egg_test_success (test, "ran correct file");
-	else
-		egg_test_failed (test, "did not run helper");
-
-	/* wait for finished */
-	egg_test_loop_wait (test, 10000);
-	egg_test_loop_check (test);
-
-	/* get new object */
-	new_spawn_object (test, &spawn);
-
-	/************************************************************
-	 **********           Killing tests               ***********
-	 ************************************************************/
-	egg_test_title (test, "make sure run correct helper, and cancel it using SIGKILL");
-	mexit = PK_SPAWN_EXIT_TYPE_UNKNOWN;
-	path = egg_test_get_data_file ("pk-spawn-test.sh");
-	argv = g_strsplit (path, " ", 0);
-	ret = pk_spawn_argv (spawn, argv, NULL);
-	g_free (path);
-	g_strfreev (argv);
-	if (ret)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "did not run helper");
-
-	g_timeout_add_seconds (1, cancel_cb, spawn);
-	/* wait for finished */
-	egg_test_loop_wait (test, 5000);
-	egg_test_loop_check (test);
-
-	/************************************************************/
-	egg_test_title (test, "make sure finished in SIGKILL");
-	if (mexit == PK_SPAWN_EXIT_TYPE_SIGKILL)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "finish %i!", mexit);
-
-	/* get new object */
-	new_spawn_object (test, &spawn);
-
-	/************************************************************/
-	egg_test_title (test, "make sure dumb helper ignores SIGQUIT");
-	mexit = PK_SPAWN_EXIT_TYPE_UNKNOWN;
-	path = egg_test_get_data_file ("pk-spawn-test.sh");
-	argv = g_strsplit (path, " ", 0);
-	g_object_set (spawn,
-		      "allow-sigkill", FALSE,
-		      NULL);
-	ret = pk_spawn_argv (spawn, argv, NULL);
-	g_free (path);
-	g_strfreev (argv);
-	if (ret)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "did not run helper");
-
-	g_timeout_add_seconds (1, cancel_cb, spawn);
-	/* wait for finished */
-	egg_test_loop_wait (test, 10000);
-	egg_test_loop_check (test);
-
-	/************************************************************/
-	egg_test_title (test, "make sure finished in SIGQUIT");
-	if (mexit == PK_SPAWN_EXIT_TYPE_SIGQUIT)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "finish %i!", mexit);
-
-	/* get new object */
-	new_spawn_object (test, &spawn);
-
-	/************************************************************/
-	egg_test_title (test, "make sure run correct helper, and SIGQUIT it");
-	mexit = PK_SPAWN_EXIT_TYPE_UNKNOWN;
-	path = egg_test_get_data_file ("pk-spawn-test-sigquit.sh");
-	argv = g_strsplit (path, " ", 0);
-	ret = pk_spawn_argv (spawn, argv, NULL);
-	g_free (path);
-	g_strfreev (argv);
-	if (ret)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "did not run helper");
-
-	g_timeout_add_seconds (1, cancel_cb, spawn);
-	/* wait for finished */
-	egg_test_loop_wait (test, 2000);
-	egg_test_loop_check (test);
-
-	/************************************************************/
-	egg_test_title (test, "make sure finished in SIGQUIT");
-	if (mexit == PK_SPAWN_EXIT_TYPE_SIGQUIT)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "finish %i!", mexit);
-
-	/************************************************************/
-	egg_test_title (test, "run lots of data for profiling");
-	path = egg_test_get_data_file ("pk-spawn-test-profiling.sh");
-	argv = g_strsplit (path, " ", 0);
-	ret = pk_spawn_argv (spawn, argv, NULL);
-	g_free (path);
-	g_strfreev (argv);
-	if (ret)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "did not run profiling helper");
-
-	/* get new object */
-	new_spawn_object (test, &spawn);
-
-	/************************************************************
-	 **********  Can we send commands to a dispatcher ***********
-	 ************************************************************/
-	egg_test_title (test, "run the dispatcher");
-	mexit = PK_SPAWN_EXIT_TYPE_UNKNOWN;
-	file = egg_test_get_data_file ("pk-spawn-dispatcher.py");
-	path = g_strdup_printf ("%s\tsearch-name\tnone\tpower manager", file);
-	argv = g_strsplit (path, "\t", 0);
-	envp = g_strsplit ("NETWORK=TRUE LANG=C BACKGROUND=TRUE", " ", 0);
-	ret = pk_spawn_argv (spawn, argv, envp);
-	g_free (file);
-	g_free (path);
-	if (ret)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "did not run dispatcher");
-
-	/************************************************************/
-	egg_test_title (test, "wait 2+2 seconds for the dispatcher");
-	/* wait 2 seconds, and make sure we are still running */
-	egg_test_loop_wait (test, 4000);
-	elapsed = egg_test_elapsed (test);
-	if (elapsed > 3900 && elapsed < 4100)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "dispatcher exited");
-
-	/************************************************************/
-	egg_test_title (test, "we got a package (+finished)?");
-	if (stdout_count == 2)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "did not get a package");
-
-	/************************************************************/
-	egg_test_title (test, "dispatcher still alive?");
-	if (spawn->priv->stdin_fd != -1)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "dispatcher no longer alive");
-
-	/************************************************************/
-	egg_test_title (test, "run the dispatcher with new input");
-	ret = pk_spawn_argv (spawn, argv, envp);
-	if (ret)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "did not run dispatcher with new input");
-
-	/* this may take a while */
-	egg_test_loop_wait (test, 100);
-
-	/************************************************************/
-	egg_test_title (test, "we got another package (+finished)?");
-	if (stdout_count == 4)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "did not get a package");
-
-	/* see if pk_spawn_exit blocks (required) */
-	g_idle_add (idle_cb, test);
-
-	/************************************************************/
-	egg_test_title (test, "ask dispatcher to close");
-	ret = pk_spawn_exit (spawn);
-	if (ret)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "failed to close dispatcher");
-
-	/************************************************************/
-	egg_test_title (test, "ask dispatcher to close (again, should be closing)");
-	ret = pk_spawn_exit (spawn);
-	if (!ret)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "attempted to close twice");
-
-	/* this may take a while */
-	egg_test_loop_wait (test, 100);
-
-	/************************************************************/
-	egg_test_title (test, "did dispatcher close?");
-	if (spawn->priv->stdin_fd == -1)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "dispatcher still running");
-
-	/************************************************************/
-	egg_test_title (test, "did we get the right exit code");
-	if (mexit == PK_SPAWN_EXIT_TYPE_DISPATCHER_EXIT)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "finish %i!", mexit);
-
-	/************************************************************/
-	egg_test_title (test, "ask dispatcher to close (again)");
-	ret = pk_spawn_exit (spawn);
-	if (!ret)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "dispatcher closed twice");
-
-	g_strfreev (argv);
-	g_strfreev (envp);
-	g_object_unref (spawn);
-
-	egg_test_end (test);
-}
-#endif
-
