@@ -50,12 +50,6 @@ static void     pk_transaction_db_finalize	(GObject        *object);
 
 #define PK_TRANSACTION_DB_ID_FILE_OBSOLETE	LOCALSTATEDIR "/lib/PackageKit/job_count.dat"
 
-#if PK_BUILD_LOCAL
-#define PK_TRANSACTION_DB_FILE			"./transactions.db"
-#else
-#define PK_TRANSACTION_DB_FILE			PK_DB_DIR "/transactions.db"
-#endif
-
 struct PkTransactionDbPrivate
 {
 	sqlite3			*db;
@@ -109,7 +103,7 @@ pk_transaction_sqlite_transaction_cb (void *data, gint argc, gchar **argv, gchar
 				g_object_set (item, "succeeded", FALSE, NULL);
 		} else if (g_strcmp0 (col, "role") == 0) {
 			if (value != NULL)
-				g_object_set (item, "role", pk_role_enum_from_text (value), NULL);
+				g_object_set (item, "role", pk_role_enum_from_string (value), NULL);
 		} else if (g_strcmp0 (col, "transaction_id") == 0) {
 			if (value != NULL)
 				g_object_set (item, "tid", value, NULL);
@@ -237,7 +231,7 @@ pk_transaction_db_action_time_since (PkTransactionDb *tdb, PkRoleEnum role)
 	g_return_val_if_fail (PK_IS_TRANSACTION_DB (tdb), 0);
 	g_return_val_if_fail (tdb->priv->db != NULL, 0);
 
-	role_text = pk_role_enum_to_text (role);
+	role_text = pk_role_enum_to_string (role);
 	egg_debug ("get_time_since_action=%s", role_text);
 
 	statement = g_strdup_printf ("SELECT timespec FROM last_action WHERE role = '%s'", role_text);
@@ -279,7 +273,7 @@ pk_transaction_db_action_time_reset (PkTransactionDb *tdb, PkRoleEnum role)
 	g_return_val_if_fail (tdb->priv->db != NULL, FALSE);
 
 	timespec = pk_iso8601_present ();
-	role_text = pk_role_enum_to_text (role);
+	role_text = pk_role_enum_to_string (role);
 
 	/* get the previous entry */
 	since = pk_transaction_db_action_time_since (tdb, role);
@@ -364,7 +358,7 @@ pk_transaction_db_set_role (PkTransactionDb *tdb, const gchar *tid, PkRoleEnum r
 
 	g_return_val_if_fail (PK_IS_TRANSACTION_DB (tdb), FALSE);
 
-	role_text = pk_role_enum_to_text (role);
+	role_text = pk_role_enum_to_string (role);
 	statement = g_strdup_printf ("UPDATE transactions SET role = '%s' WHERE transaction_id = '%s'", role_text, tid);
 	pk_transaction_db_sql_statement (tdb, statement);
 	g_free (statement);
@@ -561,7 +555,12 @@ pk_transaction_db_generate_id (PkTransactionDb *tdb)
 	 * next time we are idle (but ensure we do this on shutdown) */
 	if (tdb->priv->database_save_id == 0) {
 		egg_debug ("deferring low priority write until idle");
-		tdb->priv->database_save_id = g_idle_add_full (G_PRIORITY_LOW, (GSourceFunc) pk_transaction_db_defer_write_job_count_cb, tdb, NULL);
+		tdb->priv->database_save_id =
+			g_idle_add_full (G_PRIORITY_LOW, (GSourceFunc)
+					 pk_transaction_db_defer_write_job_count_cb, tdb, NULL);
+#if GLIB_CHECK_VERSION(2,25,8)
+		g_source_set_name_by_id (tdb->priv->database_save_id, "[PkTransactionDb] save");
+#endif
 	}
 
 	/* make the tid */
@@ -880,6 +879,169 @@ pk_transaction_db_init (PkTransactionDb *tdb)
 		statement = "CREATE TABLE proxy (created TEXT, proxy_http TEXT, proxy_ftp TEXT, uid INTEGER, session TEXT);";
 		sqlite3_exec (tdb->priv->db, statement, NULL, NULL, NULL);
 	}
+
+	/* transaction root (since 0.6.4) */
+	rc = sqlite3_exec (tdb->priv->db, "SELECT * FROM root LIMIT 1", NULL, NULL, &error_msg);
+	if (rc != SQLITE_OK) {
+		egg_debug ("adding table root: %s", error_msg);
+		statement = "CREATE TABLE root (created TEXT, root TEXT, uid INTEGER, session TEXT);";
+		sqlite3_exec (tdb->priv->db, statement, NULL, NULL, NULL);
+	}
+}
+
+/**
+ * pk_transaction_sqlite_root_cb:
+ **/
+static gint
+pk_transaction_sqlite_root_cb (void *data, gint argc, gchar **argv, gchar **col_name)
+{
+	gchar **root = (gchar **) data;
+	gint i;
+
+	g_return_val_if_fail (root != NULL, 0);
+
+	for (i=0; i<argc; i++) {
+		if (g_strcmp0 (col_name[i], "root") == 0) {
+			*root = g_strdup (argv[i]);
+		} else {
+			egg_warning ("%s = %s\n", col_name[i], argv[i]);
+		}
+	}
+	return 0;
+}
+
+/**
+ * pk_transaction_db_get_root:
+ * @tdb: the #PkTransactionDb instance
+ * @uid: the user ID of the user
+ * @session: the ConsoleKit session
+ * @root: the install root
+ *
+ * Retrieves the root information from the database.
+ *
+ * Return value: %TRUE if we matched a root
+ **/
+gboolean
+pk_transaction_db_get_root (PkTransactionDb *tdb, guint uid, const gchar *session,
+			     gchar **root)
+{
+	gchar *error_msg = NULL;
+	gchar *statement;
+	gboolean ret = FALSE;
+	gint rc;
+
+	g_return_val_if_fail (PK_IS_TRANSACTION_DB (tdb), FALSE);
+	g_return_val_if_fail (uid != G_MAXUINT, FALSE);
+	g_return_val_if_fail (root != NULL, FALSE);
+
+	/* get existing data */
+	statement = g_strdup_printf ("SELECT root FROM root WHERE uid = '%i' AND session = '%s' LIMIT 1",
+				     uid, session);
+	rc = sqlite3_exec (tdb->priv->db, statement, pk_transaction_sqlite_root_cb, root, &error_msg);
+	if (rc != SQLITE_OK) {
+		egg_warning ("SQL error: %s\n", error_msg);
+		sqlite3_free (error_msg);
+		goto out;
+	}
+
+	/* nothing matched */
+	if (*root == NULL) {
+		egg_debug ("no data");
+		goto out;
+	}
+
+	/* success, even if we got no data */
+	ret = TRUE;
+out:
+	g_free (statement);
+	return ret;
+}
+
+/**
+ * pk_transaction_db_set_root:
+ * @tdb: the #PkTransactionDb instance
+ * @uid: the user ID of the user
+ * @session: the ConsoleKit session
+ * @root: the HTTP root
+ *
+ * Saves the root install prefix to the database.
+ *
+ * Return value: %TRUE for success
+ **/
+gboolean
+pk_transaction_db_set_root (PkTransactionDb *tdb, guint uid, const gchar *session,
+			    const gchar *root)
+{
+	gchar *timespec = NULL;
+	gchar *root_tmp = NULL;
+	gboolean ret = FALSE;
+	gint rc;
+	sqlite3_stmt *statement = NULL;
+
+	g_return_val_if_fail (PK_IS_TRANSACTION_DB (tdb), FALSE);
+	g_return_val_if_fail (uid != G_MAXUINT, FALSE);
+
+	/* check for previous entries */
+	ret = pk_transaction_db_get_root (tdb, uid, session, &root_tmp);
+	if (ret) {
+		egg_debug ("updated root %s for uid:%i and session:%s", root, uid, session);
+
+		/* prepare statement */
+		rc = sqlite3_prepare_v2 (tdb->priv->db,
+					 "UPDATE root SET root = ? WHERE uid = ? AND session = ?",
+					 -1, &statement, NULL);
+		if (rc != SQLITE_OK) {
+			egg_warning ("failed to prepare statement: %s", sqlite3_errmsg (tdb->priv->db));
+			goto out;
+		}
+
+		/* bind data, so that the freeform root text cannot be used to inject SQL */
+		sqlite3_bind_text (statement, 1, root, -1, SQLITE_STATIC);
+		sqlite3_bind_int (statement, 2, uid);
+		sqlite3_bind_text (statement, 3, session, -1, SQLITE_STATIC);
+
+		/* execute statement */
+		rc = sqlite3_step (statement);
+		if (rc != SQLITE_DONE) {
+			egg_warning ("failed to execute statement: %s", sqlite3_errmsg (tdb->priv->db));
+			goto out;
+		}
+		goto out;
+	}
+
+	/* insert new entry */
+	timespec = pk_iso8601_present ();
+	egg_debug ("set root %s for uid:%i and session:%s", root, uid, session);
+
+	/* prepare statement */
+	rc = sqlite3_prepare_v2 (tdb->priv->db,
+				 "INSERT INTO root (created, uid, session, root) VALUES (?, ?, ?, ?)",
+				 -1, &statement, NULL);
+	if (rc != SQLITE_OK) {
+		egg_warning ("failed to prepare statement: %s", sqlite3_errmsg (tdb->priv->db));
+		goto out;
+	}
+
+	/* bind data, so that the freeform root text cannot be used to inject SQL */
+	sqlite3_bind_text (statement, 1, timespec, -1, SQLITE_STATIC);
+	sqlite3_bind_int (statement, 2, uid);
+	sqlite3_bind_text (statement, 3, session, -1, SQLITE_STATIC);
+	sqlite3_bind_text (statement, 4, root, -1, SQLITE_STATIC);
+
+	/* execute statement */
+	rc = sqlite3_step (statement);
+	if (rc != SQLITE_DONE) {
+		egg_warning ("failed to execute statement: %s", sqlite3_errmsg (tdb->priv->db));
+		goto out;
+	}
+
+	ret = TRUE;
+out:
+	if (statement != NULL)
+		sqlite3_finalize (statement);
+	g_free (timespec);
+	g_free (root_tmp);
+	return ret;
 }
 
 /**
@@ -919,157 +1081,3 @@ pk_transaction_db_new (void)
 	return PK_TRANSACTION_DB (tdb);
 }
 
-/***************************************************************************
- ***                          MAKE CHECK TESTS                           ***
- ***************************************************************************/
-#ifdef EGG_TEST
-#include <glib/gstdio.h>
-#include "egg-test.h"
-
-void
-pk_transaction_db_test (EggTest *test)
-{
-	PkTransactionDb *db;
-	guint value;
-	gchar *tid;
-	gboolean ret;
-	guint ms;
-	gchar *proxy_http = NULL;
-	gchar *proxy_ftp = NULL;
-	guint seconds;
-
-	if (!egg_test_start (test, "PkTransactionDb"))
-		return;
-
-	/* remove the self check file */
-#if PK_BUILD_LOCAL
-	ret = g_file_test (PK_TRANSACTION_DB_FILE, G_FILE_TEST_EXISTS);
-	if (ret) {
-		egg_test_title (test, "remove old local database");
-		egg_warning ("Removing %s", PK_TRANSACTION_DB_FILE);
-		value = g_unlink (PK_TRANSACTION_DB_FILE);
-		egg_test_assert (test, (value == 0));
-	}
-#endif
-
-	/************************************************************/
-	egg_test_title (test, "check we created quickly");
-	db = pk_transaction_db_new ();
-	ms = egg_test_elapsed (test);
-	if (ms < 1500)
-		egg_test_success (test, "acceptable time %ims", ms);
-	else
-		egg_test_failed (test, "took a long time: %ims", ms);
-	g_object_unref (db);
-
-	/************************************************************/
-	egg_test_title (test, "check we opened quickly");
-	db = pk_transaction_db_new ();
-	ms = egg_test_elapsed (test);
-	if (ms < 100)
-		egg_test_success (test, "acceptable time %ims", ms);
-	else
-		egg_test_failed (test, "took a long time: %ims", ms);
-
-	/************************************************************/
-	egg_test_title (test, "do we get the correct time on a blank database");
-	value = pk_transaction_db_action_time_since (db, PK_ROLE_ENUM_REFRESH_CACHE);
-	if (value == G_MAXUINT)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "failed to get correct time, got %i", value);
-
-	/************************************************************
-	 ****************          IDENT           ******************
-	 ************************************************************/
-	egg_test_title (test, "get an tid object");
-	tid = pk_transaction_db_generate_id (db);
-	ms = egg_test_elapsed (test);
-	if (ms < 10)
-		egg_test_success (test, "acceptable time %ims", ms);
-	else
-		egg_test_failed (test, "took a long time: %ims", ms);
-	g_free (tid);
-
-	/************************************************************/
-	egg_test_title (test, "get an tid object (no wait)");
-	tid = pk_transaction_db_generate_id (db);
-	ms = egg_test_elapsed (test);
-	if (ms < 5)
-		egg_test_success (test, "acceptable time %ims", ms);
-	else
-		egg_test_failed (test, "took a long time: %ims", ms);
-	g_free (tid);
-
-	/************************************************************/
-	egg_test_title (test, "set the correct time");
-	ret = pk_transaction_db_action_time_reset (db, PK_ROLE_ENUM_REFRESH_CACHE);
-	if (ret)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "failed to reset value");
-
-	/************************************************************/
-	egg_test_title (test, "do the deferred write");
-	while (g_main_context_pending (NULL))
-		g_main_context_iteration (NULL, TRUE);
-	ms = egg_test_elapsed (test);
-	if (ms > 1)
-		egg_test_success (test, "acceptable time %ims", ms);
-	else
-		egg_test_failed (test, "took too short time: %ims", ms);
-
-	g_usleep (2*1000*1000);
-
-	/************************************************************/
-	egg_test_title (test, "do we get the correct time");
-	value = pk_transaction_db_action_time_since (db, PK_ROLE_ENUM_REFRESH_CACHE);
-	if (value > 1 && value <= 4)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "failed to get correct time, %u", value);
-
-	/************************************************************
-	 ****************          PROXIES         ******************
-	 ************************************************************/
-	egg_test_title (test, "can we set the proxies");
-	ret = pk_transaction_db_set_proxy (db, 500, "session1", "127.0.0.1:80", "127.0.0.1:21");
-	egg_test_assert (test, ret);
-
-	/************************************************************/
-	egg_test_title (test, "can we set the proxies (overwrite)");
-	ret = pk_transaction_db_set_proxy (db, 500, "session1", "127.0.0.1:8000", "127.0.0.1:21");
-	egg_test_assert (test, ret);
-
-	/************************************************************/
-	egg_test_title (test, "can we get the proxies (non-existant user)");
-	ret = pk_transaction_db_get_proxy (db, 501, "session1", &proxy_http, &proxy_ftp);
-	if (proxy_http == NULL && proxy_ftp == NULL)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "failed to get correct proxies, %s and %s", proxy_http, proxy_ftp);
-
-	/************************************************************/
-	egg_test_title (test, "can we get the proxies (non-existant session)");
-	ret = pk_transaction_db_get_proxy (db, 500, "session2", &proxy_http, &proxy_ftp);
-	if (proxy_http == NULL && proxy_ftp == NULL)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "failed to get correct proxies, %s and %s", proxy_http, proxy_ftp);
-
-	/************************************************************/
-	egg_test_title (test, "can we get the proxies (match)");
-	ret = pk_transaction_db_get_proxy (db, 500, "session1", &proxy_http, &proxy_ftp);
-	if (g_strcmp0 (proxy_http, "127.0.0.1:8000") == 0 &&
-	    g_strcmp0 (proxy_ftp, "127.0.0.1:21") == 0)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "failed to get correct proxies, %s and %s", proxy_http, proxy_ftp);
-
-	g_free (proxy_http);
-	g_free (proxy_ftp);
-	g_object_unref (db);
-	egg_test_end (test);
-}
-#endif
-
