@@ -66,13 +66,12 @@ struct PkTransactionExtraPrivate
 };
 
 enum {
-	SIGNAL_STATUS_CHANGED,
-	SIGNAL_PROGRESS_CHANGED,
-	SIGNAL_REQUIRE_RESTART,
-	SIGNAL_LAST
+	PK_POST_TRANS_STATUS_CHANGED,
+	PK_POST_TRANS_PROGRESS_CHANGED,
+	PK_POST_TRANS_LAST_SIGNAL
 };
 
-static guint signals [SIGNAL_LAST] = { 0 };
+static guint signals [PK_POST_TRANS_LAST_SIGNAL] = { 0 };
 G_DEFINE_TYPE (PkTransactionExtra, pk_transaction_extra, G_TYPE_OBJECT)
 
 /**
@@ -81,8 +80,14 @@ G_DEFINE_TYPE (PkTransactionExtra, pk_transaction_extra, G_TYPE_OBJECT)
 static void
 pk_transaction_extra_finished_cb (PkBackend *backend, PkExitEnum exit_enum, PkTransactionExtra *extra)
 {
-	if (g_main_loop_is_running (extra->priv->loop))
+	if (g_main_loop_is_running (extra->priv->loop)) {
+		if (exit_enum != PK_EXIT_ENUM_SUCCESS) {
+			egg_warning ("%s failed with exit code: %s",
+				     pk_role_enum_to_string (pk_backend_get_role (backend)),
+				     pk_exit_enum_to_string (exit_enum));
+		}
 		g_main_loop_quit (extra->priv->loop);
+	}
 }
 
 /**
@@ -94,29 +99,14 @@ pk_transaction_extra_package_cb (PkBackend *backend, PkPackage *package, PkTrans
 	g_ptr_array_add (extra->priv->list, g_object_ref (package));
 }
 
-/**
- * pk_transaction_extra_set_require_restart:
- **/
-static void
-pk_transaction_extra_set_require_restart (PkTransactionExtra *extra, PkRestartEnum restart, const gchar *package_id)
-{
-	PkRequireRestart *item;
-	item = pk_require_restart_new ();
-	g_object_set (item,
-		      "restart", restart,
-		      "package-id", package_id,
-		      NULL);
-	g_signal_emit (extra, signals [SIGNAL_REQUIRE_RESTART], 0, item);
-	g_object_unref (item);
-}
-
 /**
  * pk_transaction_extra_set_status_changed:
  **/
 static void
 pk_transaction_extra_set_status_changed (PkTransactionExtra *extra, PkStatusEnum status)
 {
-	g_signal_emit (extra, signals [SIGNAL_STATUS_CHANGED], 0, status);
+	egg_debug ("emiting status-changed %s", pk_status_enum_to_string (status));
+	g_signal_emit (extra, signals [PK_POST_TRANS_STATUS_CHANGED], 0, status);
 }
 
 /**
@@ -125,7 +115,8 @@ pk_transaction_extra_set_status_changed (PkTransactionExtra *extra, PkStatusEnum
 static void
 pk_transaction_extra_set_progress_changed (PkTransactionExtra *extra, guint percentage)
 {
-	g_signal_emit (extra, signals [SIGNAL_PROGRESS_CHANGED], 0, percentage, 0, 0, 0);
+	egg_debug ("emiting progress-changed %i", percentage);
+	g_signal_emit (extra, signals [PK_POST_TRANS_PROGRESS_CHANGED], 0, percentage, 0, 0, 0);
 }
 
 /**
@@ -135,12 +126,15 @@ static PkPackage *
 pk_transaction_extra_get_installed_package_for_file (PkTransactionExtra *extra, const gchar *filename)
 {
 	PkPackage *package = NULL;
+	gchar **filenames;
 
 	/* use PK to find the correct package */
 	if (extra->priv->list->len > 0)
 		g_ptr_array_remove_range (extra->priv->list, 0, extra->priv->list->len);
 	pk_backend_reset (extra->priv->backend);
-	pk_backend_search_file (extra->priv->backend, pk_bitfield_value (PK_FILTER_ENUM_INSTALLED), filename);
+	filenames = g_strsplit (filename, "|||", -1);
+	pk_backend_search_files (extra->priv->backend, pk_bitfield_value (PK_FILTER_ENUM_INSTALLED), filenames);
+	g_strfreev (filenames);
 
 	/* wait for finished */
 	g_main_loop_run (extra->priv->loop);
@@ -473,7 +467,7 @@ pk_transaction_extra_package_list_to_string (GPtrArray *array)
 			      "package-id", &package_id,
 			      "summary", &summary,
 			      NULL);
-		g_string_append_printf (string, "%s\t%s\t%s\n", pk_info_enum_to_text (info), package_id, summary);
+		g_string_append_printf (string, "%s\t%s\t%s\n", pk_info_enum_to_string (info), package_id, summary);
 		g_free (package_id);
 		g_free (summary);
 	}
@@ -872,7 +866,7 @@ pk_transaction_extra_check_library_restart (PkTransactionExtra *extra)
 			egg_debug ("failed to find package for %s", filename);
 			continue;
 		}
-		pk_transaction_extra_set_require_restart (extra, PK_RESTART_ENUM_SECURITY_SESSION, pk_package_get_id (package));
+		pk_backend_require_restart (extra->priv->backend, PK_RESTART_ENUM_SECURITY_SESSION, pk_package_get_id (package));
 	}
 
 	/* process all system restarts */
@@ -884,7 +878,7 @@ pk_transaction_extra_check_library_restart (PkTransactionExtra *extra)
 			egg_debug ("failed to find package for %s", filename);
 			continue;
 		}
-		pk_transaction_extra_set_require_restart (extra, PK_RESTART_ENUM_SECURITY_SYSTEM, pk_package_get_id (package));
+		pk_backend_require_restart (extra->priv->backend, PK_RESTART_ENUM_SECURITY_SYSTEM, pk_package_get_id (package));
 	}
 
 out:
@@ -1021,7 +1015,7 @@ pk_transaction_extra_applications_are_running (PkTransactionExtra *extra, gchar
 	/* there is a file we can't COW */
 	if (extra->priv->files_list->len != 0) {
 		file = g_ptr_array_index (extra->priv->files_list, 0);
-		*error = g_error_new (1, 0, "failed to run as %s is running", file);
+		g_set_error (error, 1, 0, "failed to run as %s is running", file);
 		ret = FALSE;
 		goto out;
 	}
@@ -1045,6 +1039,8 @@ out:
  *  4) The list of pids are converted to a list of files
  *  5) The list of files is converted to a list of packages
  *  6) For each package, emit a RequireRestart of the correct type (according to the UID)
+ *
+ * Return value: success, so %TRUE means the library check completed okay
  **/
 gboolean
 pk_transaction_extra_check_library_restart_pre (PkTransactionExtra *extra, gchar **package_ids)
@@ -1165,21 +1161,16 @@ pk_transaction_extra_class_init (PkTransactionExtraClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	object_class->finalize = pk_transaction_extra_finalize;
-	signals [SIGNAL_STATUS_CHANGED] =
+	signals [PK_POST_TRANS_STATUS_CHANGED] =
 		g_signal_new ("status-changed",
 			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
 			      0, NULL, NULL, g_cclosure_marshal_VOID__UINT,
 			      G_TYPE_NONE, 1, G_TYPE_UINT);
-	signals [SIGNAL_PROGRESS_CHANGED] =
+	signals [PK_POST_TRANS_PROGRESS_CHANGED] =
 		g_signal_new ("progress-changed",
 			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
 			      0, NULL, NULL, pk_marshal_VOID__UINT_UINT_UINT_UINT,
 			      G_TYPE_NONE, 4, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT);
-	signals [SIGNAL_REQUIRE_RESTART] =
-		g_signal_new ("require-restart",
-			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
-			      0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT,
-			      G_TYPE_NONE, 1, G_TYPE_OBJECT);
 	g_type_class_add_private (klass, sizeof (PkTransactionExtraPrivate));
 }
 
@@ -1263,28 +1254,3 @@ pk_transaction_extra_new (void)
 	return PK_POST_TRANS (extra);
 }
 
-/***************************************************************************
- ***                          MAKE CHECK TESTS                           ***
- ***************************************************************************/
-#ifdef EGG_TEST
-#include "egg-test.h"
-
-void
-egg_test_extra_trans (EggTest *test)
-{
-	PkTransactionExtra *extra;
-
-	if (!egg_test_start (test, "PkTransactionExtra"))
-		return;
-
-	/************************************************************/
-	egg_test_title (test, "get an instance");
-	extra = pk_transaction_extra_new ();
-	egg_test_assert (test, extra != NULL);
-
-	g_object_unref (extra);
-
-	egg_test_end (test);
-}
-#endif
-
