@@ -60,7 +60,7 @@ gboolean	 pk_transaction_list_create		(PkTransactionList	*tlist,
 							 GError			**error);
 gboolean	 pk_transaction_list_remove		(PkTransactionList	*tlist,
 							 const gchar		*tid);
-gboolean	 pk_transaction_list_set_background		(PkTransactionList	*tlist,
+gboolean	 pk_transaction_list_set_background	(PkTransactionList	*tlist,
 							 const gchar		*tid,
 							 gboolean		 background);
 gboolean	 pk_transaction_list_commit		(PkTransactionList	*tlist,
@@ -73,6 +73,11 @@ gchar		**pk_transaction_list_get_array		(PkTransactionList	*tlist)
 gchar		*pk_transaction_list_get_state		(PkTransactionList	*tlist)
 							 G_GNUC_WARN_UNUSED_RESULT;
 guint		 pk_transaction_list_get_size		(PkTransactionList	*tlist);
+PkTransaction	*pk_transaction_list_get_transaction	(PkTransactionList	*tlist,
+							 const gchar		*tid,
+							 gboolean		*running,
+							 gboolean		*committed,
+							 gboolean		*finished);
 
 G_END_DECLS
 
