@@ -106,9 +106,6 @@ void		 pk_transaction_cancel				(PkTransaction	*transaction,
 void		 pk_transaction_download_packages		(PkTransaction  *transaction,
 								 gchar		**package_ids,
 								 DBusGMethodInvocation *context);
-gboolean	 pk_transaction_get_allow_cancel		(PkTransaction	*transaction,
-								 gboolean	*allow_cancel,
-								 GError		**error);
 void		 pk_transaction_get_categories			(PkTransaction	*transaction,
 								 DBusGMethodInvocation *context);
 void		 pk_transaction_get_depends			(PkTransaction	*transaction,
@@ -127,18 +124,9 @@ void		 pk_transaction_get_files			(PkTransaction	*transaction,
 gboolean	 pk_transaction_get_old_transactions		(PkTransaction	*transaction,
 								 guint		 number,
 								 GError		**error);
-gboolean	 pk_transaction_get_package_last		(PkTransaction	*transaction,
-								 gchar		**package,
-								 GError		**error);
 void		 pk_transaction_get_packages			(PkTransaction	*transaction,
 								 const gchar	*filter,
 								 DBusGMethodInvocation *context);
-gboolean	 pk_transaction_get_progress			(PkTransaction	*transaction,
-								 guint		*percentage,
-								 guint		*subpercentage,
-								 guint		*elapsed,
-								 guint		*remaining,
-								 GError		**error);
 void		 pk_transaction_get_repo_list			(PkTransaction	*transaction,
 								 const gchar	*filter,
 								 DBusGMethodInvocation *context);
@@ -147,13 +135,6 @@ void		 pk_transaction_get_requires			(PkTransaction	*transaction,
 								 gchar		**package_ids,
 								 gboolean	 recursive,
 								 DBusGMethodInvocation *context);
-gboolean	 pk_transaction_get_role			(PkTransaction	*transaction,
-								 const gchar	**role,
-								 const gchar	**text,
-								 GError		**error);
-gboolean	 pk_transaction_get_status			(PkTransaction	*transaction,
-								 const gchar	**status,
-								 GError		**error);
 void		 pk_transaction_get_update_detail		(PkTransaction	*transaction,
 								 gchar		**package_ids,
 								 DBusGMethodInvocation *context);
@@ -173,9 +154,6 @@ void		 pk_transaction_install_signature		(PkTransaction	*transaction,
 								 const gchar	*key_id,
 								 const gchar	*package_id,
 								DBusGMethodInvocation *context);
-gboolean	 pk_transaction_is_caller_active		(PkTransaction	*transaction,
-								 gboolean	*is_active,
-								 GError		**error);
 void		 pk_transaction_refresh_cache			(PkTransaction	*transaction,
 								 gboolean	 force,
 								 DBusGMethodInvocation *context);
@@ -202,22 +180,19 @@ void		 pk_transaction_rollback			(PkTransaction	*transaction,
 								 DBusGMethodInvocation *context);
 void		 pk_transaction_search_details			(PkTransaction	*transaction,
 								 const gchar	*filter,
-								 const gchar	*values,
+								 gchar		**values,
 								 DBusGMethodInvocation *context);
-void		 pk_transaction_search_file			(PkTransaction	*transaction,
+void		 pk_transaction_search_files			(PkTransaction	*transaction,
 								 const gchar	*filter,
-								 const gchar	*values,
+								 gchar		**values,
 								 DBusGMethodInvocation *context);
-void		 pk_transaction_search_group			(PkTransaction	*transaction,
+void		 pk_transaction_search_groups			(PkTransaction	*transaction,
 								 const gchar	*filter,
-								 const gchar	*values,
+								 gchar		**values,
 								 DBusGMethodInvocation *context);
-void		 pk_transaction_search_name			(PkTransaction	*transaction,
+void		 pk_transaction_search_names			(PkTransaction	*transaction,
 								 const gchar	*filter,
-								 const gchar	*values,
-								 DBusGMethodInvocation *context);
-void		 pk_transaction_set_locale			(PkTransaction	*transaction,
-								 const gchar	*code,
+								 gchar		**values,
 								 DBusGMethodInvocation *context);
 void		 pk_transaction_set_hints			(PkTransaction	*transaction,
 								 gchar		**hints,
@@ -230,6 +205,7 @@ void		 pk_transaction_simulate_install_packages	(PkTransaction  *transaction,
 								 DBusGMethodInvocation *context);
 void		 pk_transaction_simulate_remove_packages	(PkTransaction  *transaction,
 								 gchar		**package_ids,
+								 gboolean	 autoremove,
 								 DBusGMethodInvocation *context);
 void		 pk_transaction_simulate_update_packages	(PkTransaction  *transaction,
 								 gchar		**package_ids,
@@ -244,8 +220,12 @@ void		 pk_transaction_update_system			(PkTransaction	*transaction,
 void		 pk_transaction_what_provides			(PkTransaction	*transaction,
 								 const gchar	*filter,
 								 const gchar	*type,
-								 const gchar	*values,
+								 gchar		**values,
 								 DBusGMethodInvocation *context);
+gboolean	 pk_transaction_filter_check			(const gchar	*filter,
+								 GError		**error);
+gboolean	 pk_transaction_strvalidate			(const gchar	*textr,
+								 GError		**error);
 
 G_END_DECLS
 
